@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,8 @@ class AVG_API AudioBuffer
         int getRate();
         void clear();
 
+        void volumize(float lastVol, float curVol);
+
     private:
         int m_NumFrames;
         short* m_pData;
@@ -56,4 +58,4 @@ typedef boost::shared_ptr<AudioBuffer> AudioBufferPtr;
 
 }
 
-#endif /*AUDIOFRAME_H_*/
+#endif
