@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -66,29 +66,45 @@ ConfigMgr* ConfigMgr::get()
 ConfigMgr::ConfigMgr()
 {
     addSubsys("scr");
+    addOption("scr", "gles", "false");
     addOption("scr", "bpp", "24");
     addOption("scr", "fullscreen", "false");
     addOption("scr", "windowwidth", "0");
     addOption("scr", "windowheight", "0");
     addOption("scr", "dotspermm", "0");
     addOption("scr", "usepow2textures", "false");
-    addOption("scr", "useshaders", "true");
     addOption("scr", "usepixelbuffers", "true");
-    addOption("scr", "multisamplesamples", "1");
+    addOption("scr", "multisamplesamples", "8");
+    addOption("scr", "shaderusage", "auto");
     addOption("scr", "gamma", "-1,-1,-1");
     addOption("scr", "vsyncmode", "auto");
+    addOption("scr", "videoaccel", "true");
     
     addSubsys("aud");
     addOption("aud", "channels", "2");
     addOption("aud", "samplerate", "44100");
     addOption("aud", "outputbuffersamples", "1024");
 
+    addSubsys("gesture");
+    addOption("gesture", "maxtapdist", "15");
+    addOption("gesture", "maxdoubletaptime", "300");
+    addOption("gesture", "minswipedist", "50");
+    addOption("gesture", "swipedirectiontolerance", "0.393"); // pi/8
+    addOption("gesture", "maxswipecontactdist", "100");
+    addOption("gesture", "holddelay", "500");
+    addOption("gesture", "mindragdist", "5");
+    addOption("gesture", "filtermincutoff", "0.1");
+    addOption("gesture", "filterbeta", "0.03");
+    addOption("gesture", "friction", "-1");
+
+    addSubsys("touch");
+    addOption("touch", "area", "0, 0");
+    addOption("touch", "offset", "0, 0");
+
     m_sFName = "avgrc";
     loadFile(getGlobalConfigDir()+m_sFName);
     char * pHome = getenv("HOME");
-    if (!pHome) {
-        AVG_TRACE(Logger::WARNING, "No home directory set.");
-    } else {
+    if (pHome) {
         loadFile(string(pHome)+"/."+m_sFName);
     }
 }
@@ -131,8 +147,8 @@ const string* ConfigMgr::getOption(const string& sSubsys,
     }
 }
 
-bool ConfigMgr::getBoolOption(const std::string& sSubsys, 
-            const std::string& sName, bool bDefault) const
+bool ConfigMgr::getBoolOption(const string& sSubsys, 
+            const string& sName, bool bDefault) const
 {
     const string * psOption = getOption(sSubsys, sName);
     if (psOption == 0) {
@@ -143,15 +159,14 @@ bool ConfigMgr::getBoolOption(const std::string& sSubsys,
     } else if (*psOption == "false") {
         return false;
     } else {
-        AVG_TRACE(Logger::ERROR, 
-                m_sFName << ": Unrecognized value for option "<<sName<<": " 
+        AVG_LOG_ERROR(m_sFName << ": Unrecognized value for option " << sName << ": " 
                 << *psOption << ". Must be true or false. Aborting.");
         exit(-1);
     }
 }
 
-int ConfigMgr::getIntOption(const std::string& sSubsys, 
-        const std::string& sName, int Default) const
+int ConfigMgr::getIntOption(const string& sSubsys, 
+        const string& sName, int Default) const
 {
     errno = 0;
     const string * psOption = getOption(sSubsys, sName);
@@ -161,30 +176,57 @@ int ConfigMgr::getIntOption(const std::string& sSubsys,
     int Result = strtol(psOption->c_str(), 0, 10);
     int rc = errno;
     if (rc == EINVAL || rc == ERANGE) {
-        AVG_TRACE(Logger::ERROR,
-                m_sFName << ": Unrecognized value for option "<<sName<<": " 
+        AVG_LOG_ERROR(m_sFName << ": Unrecognized value for option "<<sName<<": " 
                 << *psOption << ". Must be an integer. Aborting.");
         exit(-1);
     }
     return Result;
 }
 
-void ConfigMgr::getGammaOption(const std::string& sSubsys, 
-            const std::string& sName, double* Val) const
+void ConfigMgr::getGammaOption(const string& sSubsys, 
+            const string& sName, float* Val) const
 {
     const string * psOption = getOption(sSubsys, sName);
     if (psOption == 0) {
         return;
     }
-    int rc = sscanf(psOption->c_str(), "%lf,%lf,%lf", Val, Val+1, Val+2);
+    int rc = sscanf(psOption->c_str(), "%f,%f,%f", Val, Val+1, Val+2);
     if (rc < 3) {
-        AVG_TRACE(Logger::ERROR,
-                m_sFName << ": Unrecognized value for option "<<sName<<": " 
+        AVG_LOG_ERROR(m_sFName << ": Unrecognized value for option "<<sName<<": " 
                 << *psOption << ". Must be three comma-separated numbers. Aborting.");
         exit(-1);
     }
 }
 
+glm::vec2 ConfigMgr::getSizeOption(const string& sSubsys,
+        const string& sName) const
+{
+    const string * psOption = getOption(sSubsys, sName);
+    if (psOption == 0) {
+        return glm::vec2(0, 0);
+    }
+    float val[2];
+    int rc = sscanf(psOption->c_str(), "%f,%f", val, val+1);
+    if (rc < 2) {
+        AVG_LOG_ERROR(m_sFName << ": Unrecognized value for option " << sName << ": "
+                << *psOption << ". Must be 2 comma-separated numbers(x, y). Aborting.");
+        exit(-1);
+    }
+    return glm::vec2(val[0], val[1]);
+}
+
+void ConfigMgr::getStringOption(const string& sSubsys, 
+        const string& sName, const string& sDefault, string& sVal) const
+{
+    const string * psOption = getOption(sSubsys, sName);
+    if (psOption == 0) {
+        sVal = sDefault;
+    } else {
+        sVal = *psOption;
+    }
+}
+
+
 bool ConfigMgr::loadFile(const std::string& sPath)
 {
     string sSubsys;
@@ -194,8 +236,8 @@ bool ConfigMgr::loadFile(const std::string& sPath)
         int err = access(sPath.c_str(), R_OK);
         if (err == -1) {
             if (errno == EACCES) {
-                AVG_TRACE(Logger::WARNING,
-                        sPath+": File exists, but process doesn't have read permissions!");
+                AVG_LOG_WARNING(sPath+
+                        ": File exists, but process doesn't have read permissions!");
             }
             return false;
         }
@@ -215,8 +257,7 @@ bool ConfigMgr::loadFile(const std::string& sPath)
         }
         xmlNodePtr pRoot = xmlDocGetRootElement(doc);
         if (xmlStrcmp(pRoot->name, (const xmlChar *)(m_sFName.c_str()))) {
-            AVG_TRACE(Logger::ERROR, 
-                    sPath+": Root node must be <"+m_sFName+">, found " 
+            AVG_LOG_ERROR(sPath+": Root node must be <"+m_sFName+">, found " 
                     << pRoot->name << ". Aborting.");
             exit(255);
         }
@@ -228,8 +269,7 @@ bool ConfigMgr::loadFile(const std::string& sPath)
                 sSubsys = ((const char *)pSubsysNode->name);
                 xmlNodePtr pOptionNode = pSubsysNode->xmlChildrenNode;
                 if (!pOptionNode) {
-                    AVG_TRACE(Logger::ERROR,
-                            sPath << ": Option " << sSubsys
+                    AVG_LOG_ERROR(sPath << ": Option " << sSubsys 
                             << " has no value. Ignoring.");
                 } else {
                     ConfigOptionVector& CurSubsys = getSubsys(sSubsys);
@@ -249,13 +289,12 @@ bool ConfigMgr::loadFile(const std::string& sPath)
     } catch (Exception& e) {
         switch (e.getCode()) {
             case AVG_ERR_OPTION_SUBSYS_UNKNOWN:
-                AVG_TRACE(Logger::ERROR, "While parsing " << sPath 
-                        << ": Option group " << e.getStr() << " unknown. Aborting.");
+                AVG_LOG_ERROR("While parsing " << sPath << ": Option group " <<
+                        e.getStr() << " unknown. Aborting.");
                 exit(255);
             case AVG_ERR_OPTION_UNKNOWN: 
-                AVG_TRACE(Logger::ERROR, "While parsing " << sPath 
-                        << ": Option " << sSubsys << ":" << e.getStr() 
-                        << " unknown. Aborting.");
+                AVG_LOG_ERROR("While parsing " << sPath  << ": Option " << sSubsys <<
+                        ":" << e.getStr() << " unknown. Aborting.");
                 exit(255);
             default:
                 throw;
