@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,12 +33,15 @@
 #elif defined(__APPLE__)
 #include <mach-o/dyld.h>
 #include <mach/mach.h>
+#include <sys/utsname.h>
 #elif defined(__linux)
 #include <fstream>
 #include <unistd.h>
+#include <string.h>
 #endif
 
 #include <stdlib.h>
+#include <iostream>
 #include <cstdlib>
 
 using namespace std;
@@ -59,6 +62,107 @@ string getWinErrMsg(unsigned err)
 }
 #endif
 
+#if defined(__linux)
+// Adapted from binreloc
+static char *
+_br_find_exe_for_symbol (const void *symbol)
+{
+    #define SIZE 1024 
+    FILE *f;
+    size_t address_string_len;
+    char *address_string, line[SIZE], *found;
+
+    if (symbol == NULL)
+        return (char *) NULL;
+
+    f = fopen ("/proc/self/maps", "r");
+    if (f == NULL)
+        return (char *) NULL;
+
+    address_string_len = 4;
+    address_string = (char *) malloc(address_string_len);
+    found = (char *) NULL;
+
+
+    while (!feof (f)) {
+        char *start_addr, *end_addr, *end_addr_end, *file;
+        void *start_addr_p, *end_addr_p;
+        size_t len;
+
+        if (fgets (line, SIZE, f) == NULL)
+            break;
+
+        /* Sanity check. */
+        if (strstr (line, " r-xp ") == NULL || strchr (line, '/') == NULL)
+            continue;
+
+        /* Parse line. */
+        start_addr = line;
+        end_addr = strchr (line, '-');
+        file = strchr (line, '/');
+
+        /* More sanity check. */
+        if (!(file > end_addr && end_addr != NULL && end_addr[0] == '-'))
+            continue;
+
+        end_addr[0] = '\0';
+        end_addr++;
+        end_addr_end = strchr (end_addr, ' ');
+        if (end_addr_end == NULL)
+            continue;
+
+        end_addr_end[0] = '\0';
+        len = strlen (file);
+        if (len == 0)
+            continue;
+        if (file[len - 1] == '\n')
+            file[len - 1] = '\0';
+
+        /* Get rid of "(deleted)" from the filename. */
+        len = strlen (file);
+        if (len > 10 && strcmp (file + len - 10, " (deleted)") == 0)
+            file[len - 10] = '\0';
+
+        /* I don't know whether this can happen but better safe than sorry. */
+        len = strlen (start_addr);
+        if (len != strlen (end_addr))
+            continue;
+
+
+        /* Transform the addresses into a string in the form of 0xdeadbeef,
+         * then transform that into a pointer. */
+        if (address_string_len < len + 3) {
+            address_string_len = len + 3;
+            address_string = (char *) realloc (address_string, address_string_len);
+        }
+
+        memcpy (address_string, "0x", 2);
+        memcpy (address_string + 2, start_addr, len);
+        address_string[2 + len] = '\0';
+        sscanf (address_string, "%p", &start_addr_p);
+
+        memcpy (address_string, "0x", 2);
+        memcpy (address_string + 2, end_addr, len);
+        address_string[2 + len] = '\0';
+        sscanf (address_string, "%p", &end_addr_p);
+
+
+        if (symbol >= start_addr_p && symbol < end_addr_p) {
+            found = file;
+            break;
+        }
+    }
+
+    free (address_string);
+    fclose (f);
+
+    if (found == NULL)
+        return (char *) NULL;
+    else
+        return strdup (found);
+}
+#endif
+
 string getAvgLibPath()
 {
 #if defined(_WIN32)
@@ -66,7 +170,7 @@ string getAvgLibPath()
     char szFilename[1024];
     DWORD ok = GetModuleFileName(hModule, szFilename, sizeof(szFilename));
     if (ok == 0) {
-        AVG_TRACE(Logger::ERROR, "getAvgLibPath(): " << getWinErrMsg(GetLastError()));
+        AVG_LOG_ERROR("getAvgLibPath(): " << getWinErrMsg(GetLastError()));
         exit(5);
     } 
     string sPath=getPath(szFilename);
@@ -89,8 +193,9 @@ string getAvgLibPath()
     _NSGetExecutablePath(path, &pathLen);
     return getPath(path);
 #else
-    // For a linux solution, see http://www.autopackage.org/docs/binreloc/
-    return "";
+    char* pszFilename;
+    pszFilename = _br_find_exe_for_symbol((const void *)"");
+    return pszFilename;
 #endif
 }
 
@@ -154,8 +259,7 @@ std::string convertUTF8ToFilename(const std::string & sName)
     int err1 = MultiByteToWideChar(CP_UTF8, 0, sName.c_str(), sName.size()+1, 
             wideString, 2048);
     if (err1 == 0) {
-        AVG_TRACE(Logger::WARNING, 
-                "Error in unicode conversion (MultiByteToWideChar): " <<
+        AVG_LOG_WARNING("Error in unicode conversion (MultiByteToWideChar): " <<
                 getWinErrMsg(GetLastError()));
         return sName;
     }
@@ -166,8 +270,7 @@ std::string convertUTF8ToFilename(const std::string & sName)
         err1 = WideCharToMultiByte(CP_UTF8, 0, wideShortFName, -1, pShortName, 
                 1024, 0, 0);
         if (err1 == 0) {
-            AVG_TRACE(Logger::WARNING, 
-                    "Error in unicode conversion (MultiByteToWideChar): " <<
+            AVG_LOG_WARNING("Error in unicode conversion (MultiByteToWideChar): " <<
                     getWinErrMsg(GetLastError()));
         }
         return pShortName;
@@ -179,4 +282,27 @@ std::string convertUTF8ToFilename(const std::string & sName)
 #endif
 }
 
+#ifdef __APPLE__
+int reallyGetOSXMajorVersion()
+{
+    utsname sysInfo;
+    int rc = uname(&sysInfo);
+    AVG_ASSERT(rc == 0);
+//    cerr << sysInfo.sysname << ", " << sysInfo.nodename << ", " << sysInfo.release <<
+//            ", " << sysInfo.version << ", " << sysInfo.machine << endl;
+    istringstream ss(sysInfo.release);
+    int major;
+    int minor;
+    int dot;
+    char c;
+    ss >> major >> c >> minor >> c >> dot;
+    return major;    
+}
+
+int getOSXMajorVersion()
+{
+    static int major = reallyGetOSXMajorVersion(); // only called once for speed reasons.
+    return major;
+}
+#endif
 }
