@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,13 +20,16 @@
 //
 
 #include "ProfilingZoneID.h"
+#include "ThreadProfiler.h"
 
 using namespace std;
 
 namespace avg {
 
-ProfilingZoneID::ProfilingZoneID(const string& sName)
-    : m_sName(sName)
+ProfilingZoneID::ProfilingZoneID(const string& sName, bool bMultithreaded)
+    : m_sName(sName),
+      m_bMultithreaded(bMultithreaded),
+      m_pProfiler(0)
 {
 }
 
@@ -39,4 +42,16 @@ const string& ProfilingZoneID::getName() const
     return m_sName;
 }
 
+ThreadProfiler* ProfilingZoneID::getProfiler()
+{
+    if (!m_pProfiler) {
+        if (m_bMultithreaded) {
+            return ThreadProfiler::get();
+        } else {
+            m_pProfiler = ThreadProfiler::get();
+        }
+    }
+    return m_pProfiler;
+}
+
 }
