@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -28,16 +28,21 @@
 
 namespace avg {
 
+class ThreadProfiler;
+
 class AVG_API ProfilingZoneID
 {
 public:
-    ProfilingZoneID(const std::string& sName);
-    virtual ~ProfilingZoneID();
+    ProfilingZoneID(const std::string& sName, bool bMultithreaded=false);
+    ~ProfilingZoneID();
     
     const std::string& getName() const;
+    ThreadProfiler* getProfiler();
 
 private:
     std::string m_sName;
+    bool m_bMultithreaded;
+    ThreadProfiler* m_pProfiler;
 };
 
 }
