@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,21 +20,16 @@
 //
 
 #include "ScopeTimer.h"
-#include "ThreadProfiler.h"
 
 using namespace std;
 
 namespace avg {
 
-ScopeTimer::ScopeTimer(ProfilingZoneID& zoneID)
-    : m_ZoneID(zoneID)
-{
-    ThreadProfiler::get()->startZone(zoneID);
-}
+bool ScopeTimer::s_bTimersEnabled = false;
 
-ScopeTimer::~ScopeTimer() 
+void ScopeTimer::enableTimers(bool bEnable)
 {
-    ThreadProfiler::get()->stopZone(m_ZoneID);
+    s_bTimersEnabled = bEnable;
 }
 
 }
