@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -71,9 +71,9 @@ int stringToInt(const string& s)
     return i;
 }
 
-double stringToDouble(const string& s)
+float stringToFloat(const string& s)
 {
-    double d;
+    float d;
     fromString(s, d);
     return d;
 }
@@ -92,28 +92,17 @@ bool stringToBool(const string& s)
     throw (Exception(AVG_ERR_TYPE, string("Could not convert ")+s+" to bool."));
 }
 
-DPoint stringToDPoint(const std::string& s)
-{
-    DPoint pt;
-    fromString(s, pt);
-    return pt;
-}
-
-IntTriple stringToIntTriple(const std::string& s)
-{
-    IntTriple pt;
-    fromString(s, pt);
-    return pt;
-}
-
 std::string removeStartEndSpaces(const string& s)
 {
     string sResult = s;
-    while (sResult[0] == ' ' || sResult[0] == '\n' || sResult[0] == '\r' 
-            || sResult[0] == '\t') 
+    while (sResult.size() > 0 && (sResult[0] == ' ' || sResult[0] == '\n' 
+            || sResult[0] == '\r' || sResult[0] == '\t')) 
     {
         sResult.erase(0, 1);
     }
+    if (sResult.size() == 0) {
+        return sResult;
+    }
     char c = sResult[sResult.length()-1];
     while (c == ' ' || c == '\n' || c == '\r' || c == '\t') {
         sResult.erase(sResult.length()-1, 1);
@@ -143,5 +132,14 @@ bool equalIgnoreCase(const string& s1, const string& s2)
     return sUpper1 == sUpper2;
 }
 
+string toString(const bool& b)
+{
+    if (b) {
+        return "true";
+    } else {
+        return "false";
+    }
+}
+
 }
 
