@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine.
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,13 +19,13 @@
 //  Current versions can be found at www.libavg.de
 //
 
+#include "DAG.h"
 #include "Queue.h"
 #include "Command.h"
 #include "WorkerThread.h"
 #include "ObjectCounter.h"
-#include "Point.h"
-#include "Matrix3x4.h"
-#include "Triangulate.h"
+#include "triangulate/Triangulate.h"
+#include "GLMHelper.h"
 #include "GeomHelper.h"
 #include "OSHelper.h"
 #include "FileHelper.h"
@@ -35,9 +35,13 @@
 #include "BezierCurve.h"
 #include "Signal.h"
 #include "Backtrace.h"
-
+#include "WideLine.h"
+#include "Rect.h"
+#include "Triangle.h"
 #include "TestSuite.h"
 #include "TimeSource.h"
+#include "XMLHelper.h"
+#include "Logger.h"
 
 #include <boost/thread/thread.hpp>
 
@@ -52,6 +56,102 @@ using namespace avg;
 using namespace std;
 using namespace boost;
 
+class DAGTest: public Test
+{
+public:
+    DAGTest()
+        : Test("DAGTest", 2)
+    {
+    }
+
+    void runTests() 
+    {
+        {
+            DAG dag;
+
+            dag.addNode(1, set<long>());
+            long outgoing2[] = {1};
+            dag.addNode(0, makeOutgoing(1, outgoing2));
+            
+            long expected[] = {0, 1};
+            checkResults(&dag, expected);
+        }
+        {
+            DAG dag;
+
+            dag.addNode(2, set<long>());
+            long outgoing2[] = {1};
+            dag.addNode(0, makeOutgoing(1, outgoing2));
+            long outgoing3[] = {2};
+            dag.addNode(1, makeOutgoing(1, outgoing3));
+
+            long expected[] = {0, 1, 2};
+            checkResults(&dag, expected);
+        }
+        {
+            DAG dag;
+
+            long outgoing2[] = {1};
+            dag.addNode(0, makeOutgoing(1, outgoing2));
+            dag.addNode(2, set<long>());
+            long outgoing3[] = {2};
+            dag.addNode(1, makeOutgoing(1, outgoing3));
+
+            long expected[] = {0, 1, 2};
+            checkResults(&dag, expected);
+        }
+        {
+            DAG dag;
+
+            dag.addNode(2, set<long>());
+            long outgoing2[] = {1, 2};
+            dag.addNode(0, makeOutgoing(1, outgoing2));
+            long outgoing3[] = {2};
+            dag.addNode(1, makeOutgoing(1, outgoing3));
+
+            long expected[] = {0, 1, 2};
+            checkResults(&dag, expected);
+        }
+        {
+            DAG dag;
+
+            long outgoing2[] = {1};
+            dag.addNode(0, makeOutgoing(1, outgoing2));
+            long outgoing3[] = {0};
+            dag.addNode(1, makeOutgoing(1, outgoing3));
+
+            bool bExceptionThrown = false;
+            long expected[] = {0, 1, 2};
+            try {
+                checkResults(&dag, expected);
+            } catch (const Exception&) {
+                bExceptionThrown = true;
+            }
+            TEST(bExceptionThrown);
+        }
+    }
+
+private:
+    set<long> makeOutgoing(int n, long ids[])
+    {
+        set<long> v;
+        for (int i=0; i<n; ++i) {
+            v.insert(ids[i]);
+        }
+        return v;
+    }
+
+    void checkResults(DAG* pDAG, long expected[])
+    {
+        vector<long> results;
+        pDAG->sort(results);
+
+        for (unsigned i=0; i<results.size(); ++i) {
+            QUIET_TEST(results[i] == expected[i]);
+        }
+    }
+};
+
 class QueueTest: public Test
 {
 public:
@@ -67,6 +167,8 @@ public:
     }
 
 private:
+    typedef Queue<int>::QElementPtr ElemPtr;
+    
     void runSingleThreadTests()
     {
         Queue<string> q;
@@ -92,7 +194,7 @@ private:
     void runMultiThreadTests()
     {
         {
-            Queue<string> q(10);
+            Queue<int> q(10);
             thread pusher(boost::bind(&pushThread, &q, 100));
             thread popper(boost::bind(&popThread, &q, 100));
             pusher.join();
@@ -100,7 +202,7 @@ private:
             TEST(q.empty());
         }
         {
-            Queue<string> q(10);
+            Queue<int> q(10);
             thread pusher1(boost::bind(&pushThread, &q, 100));
             thread pusher2(boost::bind(&pushThread, &q, 100));
             thread popper(boost::bind(&popThread, &q, 200));
@@ -109,21 +211,25 @@ private:
             popper.join();
             TEST(q.empty());
         }
+        {
+            Queue<int> q(10);
+            thread pusher(boost::bind(&pushClearThread, &q, 100));
+            thread popper(boost::bind(&popClearThread, &q));
+            pusher.join();
+            popper.join();
+            TEST(q.empty());
+        }
     }
 
-    static void pushThread(Queue<string>* pq, int numPushes)
+    static void pushThread(Queue<int>* pq, int numPushes)
     {
-        typedef Queue<string>::QElementPtr ElemPtr;
         for (int i=0; i<numPushes; ++i) {
-            stringstream ss;
-            ss << i;
-            string s = ss.str();
-            pq->push(ElemPtr(new string(s)));
+            pq->push(ElemPtr(new int(i)));
             msleep(1);
         }
     }
 
-    static void popThread(Queue<string>* pq, int numPops)
+    static void popThread(Queue<int>* pq, int numPops)
     {
         for (int i=0; i<numPops; ++i) {
             pq->peek();
@@ -131,6 +237,27 @@ private:
             msleep(3);
         }
     }
+
+    static void pushClearThread(Queue<int>* pq, int numPushes)
+    {
+        typedef Queue<int>::QElementPtr ElemPtr;
+        for (int i=0; i<numPushes; ++i) {
+            pq->push(ElemPtr(new int(i)));
+            if (i%7 == 0) {
+                pq->clear();
+            }
+            msleep(1);
+        }
+        pq->push(ElemPtr(new int(-1)));
+    }
+
+    static void popClearThread(Queue<int>* pq)
+    {
+        ElemPtr pElem;
+        do {
+            pElem = pq->pop();
+        } while (*pElem != -1);
+    }
 };
 
 class TestWorkerThread: public WorkerThread<TestWorkerThread>
@@ -248,103 +375,79 @@ public:
 #pragma warning(push)
 #pragma warning(disable:4723)
 #endif
-class PointTest: public Test
+class GeomTest: public Test
 {
 public:
-    PointTest()
-        : Test("PointTest", 2)
+    GeomTest()
+        : Test("GeomTest", 2)
     {
     }
 
     void runTests() 
     {
-        double one = 1;
-        double zero = 0;
-
-        TEST(isinf(-one/zero) != 0);
-        TEST(isinf(one/zero) != 0);
-        TEST(isinf(one) == 0);
-        TEST(isnan(sqrt(-one)) != 0);
-        TEST(isnan(sqrt(one+one)) == 0);
-
         // TODO: Move to a separate math test once we're done here.
         TEST(almostEqual(invSqrt(1), 1));
         TEST(almostEqual(invSqrt(4), 0.5));
 
-        // TODO: The point tests aren't complete!
-        DPoint pt1(0,0);
-        DPoint pt2(3,4);
-        TEST(calcDist(pt1, pt2)-5 < 0.0001);
-        TEST(!almostEqual(pt1, pt2));
-        TEST(almostEqual(pt1, pt1));
-        std::vector<double> v;
-        v.push_back(3);
-        v.push_back(4);
-        DPoint pt3(v);
-        TEST(almostEqual(pt2, pt3));
-        TEST(almostEqual(pt3.getNorm(), 5));
-        DPoint pt4 = pt3.getNormalized();
-        TEST(almostEqual(pt4.getNorm(), 1, 0.0001));
         {
-            DLineSegment l1(DPoint(0,0), DPoint(2,2));
-            DLineSegment l2(DPoint(2,0), DPoint(0,2));
+            LineSegment l1(glm::vec2(0,0), glm::vec2(2,2));
+            LineSegment l2(glm::vec2(2,0), glm::vec2(0,2));
             TEST(lineSegmentsIntersect(l1, l2));
             TEST(lineSegmentsIntersect(l2, l1));
         }
         {
-            DLineSegment l1(DPoint(0,0), DPoint(0,2));
-            DLineSegment l2(DPoint(2,0), DPoint(2,2));
+            LineSegment l1(glm::vec2(0,0), glm::vec2(0,2));
+            LineSegment l2(glm::vec2(2,0), glm::vec2(2,2));
             TEST(!lineSegmentsIntersect(l1, l2));
         }
         {
-            DLineSegment l1(DPoint(0,0), DPoint(2,0));
-            DLineSegment l2(DPoint(0,2), DPoint(2,2));
+            LineSegment l1(glm::vec2(0,0), glm::vec2(2,0));
+            LineSegment l2(glm::vec2(0,2), glm::vec2(2,2));
             TEST(!lineSegmentsIntersect(l1, l2));
         }
         {
-            DLineSegment l1(DPoint(0,0), DPoint(2,0));
-            TEST(l1.isPointOver(DPoint(1,23)));
-            TEST(l1.isPointOver(DPoint(1.9,-5)));
-            TEST(!l1.isPointOver(DPoint(-1,1)));
-            TEST(!l1.isPointOver(DPoint(3,-1)));
+            LineSegment l1(glm::vec2(0,0), glm::vec2(2,0));
+            TEST(l1.isPointOver(glm::vec2(1,23)));
+            TEST(l1.isPointOver(glm::vec2(1.9,-5)));
+            TEST(!l1.isPointOver(glm::vec2(-1,1)));
+            TEST(!l1.isPointOver(glm::vec2(3,-1)));
         }
         {
-            DPoint pt0(DPoint(1,1));
-            DPoint pt1(DPoint(1,3));
-            DPoint pt2(DPoint(1,-2));
-            vector<DPoint> poly;
-            poly.push_back(DPoint(0,0));
-            poly.push_back(DPoint(2,0));
-            poly.push_back(DPoint(2,2));
-            poly.push_back(DPoint(0,2));
+            glm::vec2 pt0(glm::vec2(1,1));
+            glm::vec2 pt1(glm::vec2(1,3));
+            glm::vec2 pt2(glm::vec2(1,-2));
+            vector<glm::vec2> poly;
+            poly.push_back(glm::vec2(0,0));
+            poly.push_back(glm::vec2(2,0));
+            poly.push_back(glm::vec2(2,2));
+            poly.push_back(glm::vec2(0,2));
             TEST(pointInPolygon(pt0, poly));
             TEST(!pointInPolygon(pt1, poly));
             TEST(!pointInPolygon(pt2, poly));
-            poly.push_back(DPoint(2,1));
+            poly.push_back(glm::vec2(2,1));
             TEST(!pointInPolygon(pt0, poly));
         }
         {
-            DPoint p1(DPoint(0,0));
-            DPoint v1(DPoint(1,1));
-            DPoint p2(DPoint(2,1));
-            DPoint v2(DPoint(1,0));
-            TEST(getLineLineIntersection(p1, v1, p2, v2) == DPoint(1,1));
+            glm::vec2 p1(glm::vec2(0,0));
+            glm::vec2 v1(glm::vec2(1,1));
+            glm::vec2 p2(glm::vec2(2,1));
+            glm::vec2 v2(glm::vec2(1,0));
+            TEST(getLineLineIntersection(p1, v1, p2, v2) == glm::vec2(1,1));
         }
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI, DPoint(15,5)), DPoint(20,10)));
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI*0.5, DPoint(15,5)),
-                DPoint(20,0)));
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI*1.5, DPoint(15,5)), 
-                DPoint(10,10)));
-        TEST(almostEqual(DPoint(10,0).getRotatedPivot(M_PI*2, DPoint(15,5)), 
-                DPoint(10,0)));
-        TEST(almostEqual(DPoint(23,0).getRotatedPivot(M_PI*0.5), DPoint(0,23)));
+        TEST(almostEqual(getRotatedPivot(glm::vec2(10,0), M_PI, glm::vec2(15,5)), 
+                glm::vec2(20,10)));
+        TEST(almostEqual(getRotatedPivot(glm::vec2(10,0), M_PI*0.5, glm::vec2(15,5)),
+                glm::vec2(20,0)));
+        TEST(almostEqual(getRotatedPivot(glm::vec2(10,0), M_PI*1.5, glm::vec2(15,5)), 
+                glm::vec2(10,10)));
+        TEST(almostEqual(getRotatedPivot(glm::vec2(10,0), M_PI*2, glm::vec2(15,5)), 
+                glm::vec2(10,0)));
+        TEST(almostEqual(getRotatedPivot(glm::vec2(23,0), M_PI*0.5), glm::vec2(0,23)));
 
-        TEST(almostEqual(DPoint(10,0), DPoint::fromPolar(0, 10)));
-        TEST(almostEqual(DPoint(0,10), DPoint::fromPolar(M_PI*0.5, 10)));
-        TEST(almostEqual(DPoint(0,-1), DPoint::fromPolar(M_PI*1.5, 1)));
-        TEST(almostEqual(vecAngle(DPoint(0,1),DPoint(1,0)), M_PI*0.5));
-        TEST(almostEqual(vecAngle(DPoint(0,-1),DPoint(1,0)), M_PI*1.5));
-        TEST(almostEqual(vecAngle(DPoint(0,2),DPoint(1,0)), M_PI*0.5));
+        {
+            // TODO: More tests
+            FRect(0,0,10,10);
+        }
     }
 };
 #ifdef _MSC_VER
@@ -353,32 +456,6 @@ public:
 #endif
 
 
-class Matrix3x4Test: public Test
-{
-public:
-    Matrix3x4Test()
-        : Test("Matrix3x4Test", 2)
-    {
-    }
-
-    void runTests()
-    {
-        Matrix3x4 mat1;
-        Matrix3x4 mat2;
-        mat1 *= mat2;
-        TEST(almostEqual(mat1, Matrix3x4()));
-
-        mat2 = Matrix3x4::createTranslate(0,0,0);
-        mat1 *= mat2;
-        TEST(almostEqual(mat1, Matrix3x4()));
-
-        mat2 = Matrix3x4::createScale(1,1,1);
-        mat1 *= mat2;
-        TEST(almostEqual(mat1, Matrix3x4()));
-    }
-};
-
-
 class TriangleTest: public Test
 {
 public:
@@ -389,31 +466,30 @@ public:
 
     void runTests()
     {
-        Triangle tri(DPoint(0,0), DPoint(4,4), DPoint(4,8));
-        TEST(tri.isInside(DPoint(3,4)));
-        TEST(!tri.isInside(DPoint(1,4)));
-        TEST(!tri.isInside(DPoint(2,1)));
-        TEST(!tri.isInside(DPoint(-2,5)));
-        TEST(!tri.isInside(DPoint(5,5)));
-        tri = Triangle(DPoint(0,0), DPoint(4,8), DPoint(4,4));
-        TEST(tri.isInside(DPoint(3,4)));
+        Triangle tri(glm::vec2(0,0), glm::vec2(4,4), glm::vec2(4,8));
+        TEST(tri.isInside(glm::vec2(3,4)));
+        TEST(!tri.isInside(glm::vec2(1,4)));
+        TEST(!tri.isInside(glm::vec2(2,1)));
+        TEST(!tri.isInside(glm::vec2(-2,5)));
+        TEST(!tri.isInside(glm::vec2(5,5)));
+        tri = Triangle(glm::vec2(0,0), glm::vec2(4,8), glm::vec2(4,4));
+        TEST(tri.isInside(glm::vec2(3,4)));
 
-        DPoint polyArray[] = {DPoint(0,0), DPoint(8,2), DPoint(9,0), DPoint(9,3), 
-                DPoint(1,1), DPoint(0,3)}; 
+        glm::vec2 polyArray[] = {glm::vec2(0,0), glm::vec2(8,2), glm::vec2(9,0), glm::vec2(9,3), 
+                glm::vec2(1,1), glm::vec2(0,3)}; 
 
-        DPointVector poly = vectorFromCArray(6, polyArray);
-
-        vector<int> triangulation;
-        triangulatePolygon(poly, triangulation);
+        Vec2Vector poly = vectorFromCArray(6, polyArray);
+        vector<unsigned int> triangulation;
+        triangulatePolygon(triangulation, poly);
 
         TEST(triangulation.size() == 4*3);
-        int baselineIndexes[] = {1,2,3, 4,5,0, 0,1,3, 3,4,0};
+        unsigned int baselineIndexes[] = {5,0,4, 1,4,0, 4,1,3, 1,2,3};
         TEST(triangulation == vectorFromCArray(12, baselineIndexes));
-/*        
+/*
         for (unsigned int i=0; i<triangulation.size(); i++) {
             cerr << i << ":" << triangulation[i] << endl;
         }
-*/        
+*/
     }
 
 };
@@ -445,9 +521,8 @@ public:
 
     void runTests()
     {
-#if defined(__APPLE__) || defined(_WIN32)
+        cerr << getAvgLibPath() << endl;
         TEST(getAvgLibPath() != "");
-#endif
 #ifdef __APPLE__
         TEST(getMemoryUsage() != 0);
 #endif
@@ -466,7 +541,7 @@ public:
     void runTests()
     {
         TEST(stringToInt("5") == 5);
-        TEST(almostEqual(stringToDouble("5.5"), 5.5));
+        TEST(almostEqual(stringToFloat("5.5"), 5.5f));
         TEST(stringToBool("False") == false);
         bool bExceptionThrown = false;
         try {
@@ -477,8 +552,8 @@ public:
             }
         }
         TEST(bExceptionThrown);
-        TEST(stringToDPoint(" ( 3.4 , 2.1 ) ") == DPoint(3.4, 2.1));
-        vector<double> v;
+        TEST(stringToVec2(" ( 3.4 , 2.1 ) ") == glm::vec2(3.4f, 2.1f));
+        vector<float> v;
         fromString("(1,2,3,4,5)", v);
         TEST(v.size() == 5 && v[0] == 1 && v[4] == 5);
         v.clear();
@@ -499,10 +574,10 @@ public:
     void runTests()
     {
         {
-            double xd[] = {0,1,2,3};
-            vector<double> x = vectorFromCArray(4, xd);
-            double yd[] = {3,2,1,0};
-            vector<double> y = vectorFromCArray(4, yd);
+            float xd[] = {0,1,2,3};
+            vector<float> x = vectorFromCArray(4, xd);
+            float yd[] = {3,2,1,0};
+            vector<float> y = vectorFromCArray(4, yd);
             CubicSpline spline(x, y);
             TEST(almostEqual(spline.interpolate(-1), 4));
             TEST(almostEqual(spline.interpolate(0), 3));
@@ -511,19 +586,46 @@ public:
             TEST(almostEqual(spline.interpolate(3.5), -0.5));
         }
         {
-            double xd[] = {2,4,6,8};
-            vector<double> x = vectorFromCArray(4, xd);
-            double yd[] = {0,1,3,6};
-            vector<double> y = vectorFromCArray(4, yd);
+            float xd[] = {2,4,6,8};
+            vector<float> x = vectorFromCArray(4, xd);
+            float yd[] = {0,1,3,6};
+            vector<float> y = vectorFromCArray(4, yd);
             CubicSpline spline(x, y);
             TEST(almostEqual(spline.interpolate(0), -1));
-            TEST(almostEqual(spline.interpolate(1), -0.5));
             TEST(almostEqual(spline.interpolate(2), 0));
             TEST(spline.interpolate(3) < 0.5);
+            TEST(spline.interpolate(3) > 0);
+            TEST(spline.interpolate(7) > 4);
+            TEST(spline.interpolate(7) < 5);
             TEST(almostEqual(spline.interpolate(8), 6));
-            TEST(almostEqual(spline.interpolate(9), 7.5));
             TEST(almostEqual(spline.interpolate(10), 9));
         }
+        {
+            float xd[] = {0,1,1};
+            vector<float> x = vectorFromCArray(3, xd);
+            float yd[] = {1,2,1};
+            vector<float> y = vectorFromCArray(3, yd);
+            bool bExceptionThrown = false;
+            try {
+                CubicSpline spline(x, y);
+            } catch (const Exception&) {
+                bExceptionThrown = true;
+            }
+            TEST(bExceptionThrown);
+        }
+/*
+        {
+            float xd[] = {0,1,2};
+            vector<float> x = vectorFromCArray(3, xd);
+            float yd[] = {1,2,1};
+            vector<float> y = vectorFromCArray(3, yd);
+            CubicSpline spline(x, y, true);
+            TEST(almostEqual(spline.interpolate(0), 1));
+            TEST(almostEqual(spline.interpolate(0.5), 1.5));
+            TEST(almostEqual(spline.interpolate(2), 1));
+            TEST(almostEqual(spline.interpolate(3), 2));
+        }
+*/
     }
 };
 
@@ -538,12 +640,28 @@ public:
 
     void runTests()
     {
-        BezierCurve curve(DPoint(0,0), DPoint(1,0), DPoint(1,1), DPoint(0,1));
-        TEST(almostEqual(curve.interpolate(0), DPoint(0,0)));
-        TEST(almostEqual(curve.getDeriv(0), DPoint(3, 0)));
-        TEST(almostEqual(curve.interpolate(1), DPoint(0,1)));
-        TEST(almostEqual(curve.getDeriv(1), DPoint(-3, 0)));
-        TEST(almostEqual(curve.interpolate(0.5), DPoint(0.75,0.5)));
+        BezierCurve curve(glm::vec2(0,0), glm::vec2(1,0), glm::vec2(1,1), glm::vec2(0,1));
+        TEST(almostEqual(curve.interpolate(0), glm::vec2(0,0)));
+        TEST(almostEqual(curve.getDeriv(0), glm::vec2(3, 0)));
+        TEST(almostEqual(curve.interpolate(1), glm::vec2(0,1)));
+        TEST(almostEqual(curve.getDeriv(1), glm::vec2(-3, 0)));
+        TEST(almostEqual(curve.interpolate(0.5), glm::vec2(0.75,0.5)));
+    }
+};
+
+
+class WideLineTest: public Test
+{
+public:
+    WideLineTest()
+        : Test("WideLineTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        WideLine line(glm::vec2(0,0), glm::vec2(4,3), 2);
+        TEST(almostEqual(line.getLen(), 5));
     }
 };
 
@@ -731,17 +849,151 @@ public:
 };
 
 
+class PolygonTest: public Test
+{
+public:
+    PolygonTest()
+        : Test("PolygonTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        glm::vec2 polyArray[] = {glm::vec2(30,0), glm::vec2(40,20), glm::vec2(60,30),
+                glm::vec2(40,40), glm::vec2(30,60), glm::vec2(20,40), glm::vec2(0,30),
+                glm::vec2(20,20)}; 
+
+        Vec2Vector poly = vectorFromCArray(8, polyArray);
+        vector<unsigned int> triangulation;
+        triangulatePolygon(triangulation, poly);
+
+        TEST(triangulation.size() == 6*3);
+        unsigned int baselineIndexes[] = {6,7,5, 5,7,1, 7,0,1, 5,1,3, 3,1,2, 4,5,3};
+        TEST(triangulation == vectorFromCArray(18, baselineIndexes));
+/*     
+        for (unsigned int i=0; i<triangulation.size(); i++) {
+            cerr << i << ":" << triangulation[i] << endl;
+        }/
+*/
+    }
+
+};
+
+
+class XmlParserTest: public Test
+{
+public:
+    XmlParserTest()
+      : Test("XmlParserTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        string sXmlString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
+            "<shiporder orderid=\"889923\">"
+            "  <orderperson>John Smith</orderperson>"
+            "</shiporder>";
+
+        {
+            string sSchema = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
+                "<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
+                "<xs:element name=\"shiporder\">"
+                "  <xs:complexType>"
+                "    <xs:sequence>"
+                "      <xs:element name=\"orderperson\" type=\"xs:string\"/>"
+                "    </xs:sequence>"
+                "    <xs:attribute name=\"orderid\" type=\"xs:string\" use=\"required\"/>"
+                "  </xs:complexType>"
+                "</xs:element>"
+                "</xs:schema>";
+
+            XMLParser parser;
+            parser.setSchema(sSchema, "shiporder.xsd");
+            parser.parse(sXmlString, "shiporder.xml");
+        }
+        {
+            string sDTD =
+                "<!ELEMENT shiporder (orderperson)* >"
+                "<!ATTLIST shiporder"
+                "    orderid CDATA #IMPLIED>"
+                "<!ELEMENT orderperson (#PCDATA) >";
+            XMLParser parser;
+            parser.setDTD(sDTD, "shiporder.dtd");
+            parser.parse(sXmlString, "shiporder.xml");
+        }
+    }
+};
+
+
+class StandardLoggerTest: public Test
+{
+public:
+    StandardLoggerTest()
+      : Test("StandardLoggerTest", 2)
+    {
+    }
+
+    void runTests()
+    {
+        std::stringstream buffer;
+        std::streambuf *sbuf = std::cerr.rdbuf();
+        Logger *logger = Logger::get();
+        {
+            std::cerr.rdbuf(buffer.rdbuf());
+                string msg("Test log message");
+                AVG_TRACE(Logger::category::NONE, Logger::severity::WARNING, msg);
+            std::cerr.rdbuf(sbuf);
+                TEST(buffer.str().find(msg) != string::npos);
+            buffer.str(string());
+
+            std::cerr.rdbuf(buffer.rdbuf());
+                AVG_TRACE(Logger::category::NONE, Logger::severity::DEBUG, msg);
+            std::cerr.rdbuf(sbuf);
+            std::cout << buffer.str();
+                TEST(buffer.str().find(msg) == string::npos);
+            buffer.str(string());
+        }
+        {
+            std::cerr.rdbuf(buffer.rdbuf());
+                category_t CUSTOM_CAT = logger->configureCategory("CUSTOM_CAT 1");
+                string msg("CUSTOM_CAT LOG");
+                AVG_TRACE(CUSTOM_CAT, Logger::severity::WARNING, msg);
+            std::cerr.rdbuf(sbuf);
+                TEST(buffer.str().find(msg) != string::npos);
+            buffer.str(string());
+        }
+        {
+            std::cerr.rdbuf(buffer.rdbuf());
+                category_t CUSTOM_CAT = logger->configureCategory("CUSTOM_CAT 1",
+                        Logger::severity::CRITICAL);
+                string msg_info("CUSTOM_CAT LOG INFO");
+                AVG_TRACE(CUSTOM_CAT, Logger::severity::WARNING, msg_info);
+            std::cerr.rdbuf(sbuf);
+                TEST(buffer.str().find(msg_info) == string::npos);
+            buffer.str(string());
+
+            std::cerr.rdbuf(buffer.rdbuf());
+                string msg_critical("CUSTOM_CAT LOG CRITICAL");
+                AVG_TRACE(CUSTOM_CAT, Logger::severity::CRITICAL, msg_critical);
+            std::cerr.rdbuf(sbuf);
+                TEST(buffer.str().find(msg_critical) != string::npos);
+            buffer.str(string());
+        }
+    }
+};
+
 class BaseTestSuite: public TestSuite
 {
 public:
     BaseTestSuite() 
         : TestSuite("BaseTestSuite")
     {
+        addTest(TestPtr(new DAGTest));
         addTest(TestPtr(new QueueTest));
         addTest(TestPtr(new WorkerThreadTest));
         addTest(TestPtr(new ObjectCounterTest));
-        addTest(TestPtr(new PointTest));
-        addTest(TestPtr(new Matrix3x4Test));
+        addTest(TestPtr(new GeomTest));
         addTest(TestPtr(new TriangleTest));
         addTest(TestPtr(new FileTest));
         addTest(TestPtr(new OSTest));
@@ -750,6 +1002,9 @@ public:
         addTest(TestPtr(new BezierCurveTest));
         addTest(TestPtr(new SignalTest));
         addTest(TestPtr(new BacktraceTest));
+        addTest(TestPtr(new PolygonTest));
+        addTest(TestPtr(new XmlParserTest));
+        addTest(TestPtr(new StandardLoggerTest));
     }
 };
 
