@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,7 @@ namespace avg {
 // XXX: Warning, possibly buggy.
 class AVG_API FilterBandpass: public Filter {
 public:
-    FilterBandpass(double lowWidth, double highWidth);
+    FilterBandpass(float lowWidth, float highWidth);
     virtual ~FilterBandpass();
 
     virtual BitmapPtr apply(BitmapPtr pBmpSrc);
