@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -30,13 +30,13 @@ namespace avg {
 class AVG_API FilterIntensity : public Filter
 {
 public:
-    FilterIntensity(int offset, double factor);
+    FilterIntensity(int offset, float factor);
     virtual ~FilterIntensity();
     virtual void applyInPlace(BitmapPtr pBmp) ;
 
 private:
     int m_Offset;
-    double m_Factor;
+    float m_Factor;
 };
 
 }  // namespace
