@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,8 +19,8 @@
 //  Current versions can be found at www.libavg.de
 //
 
-#ifndef _Filterfliprgba_H
-#define _Filterfliprgba_H
+#ifndef _FilterUnmultiplyAlpha_H
+#define _FilterUnmultiplyAlpha_H
 
 #include "../api.h"
 #include "Filter.h"
