@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,22 +32,25 @@ namespace avg {
 class AVG_API GPUBandpassFilter: public GPUFilter
 {
 public:
-    GPUBandpassFilter(const IntPoint& size, PixelFormat pfSrc, double min, double max, 
-            double postScale, bool bInvert, bool bStandalone=true);
+    GPUBandpassFilter(const IntPoint& size, PixelFormat pfSrc, float min, float max, 
+            float postScale, bool bInvert, bool bStandalone=true);
     virtual ~GPUBandpassFilter();
 
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
 
 private:
-    static void initShader();
-
-    double m_PostScale;
+    float m_PostScale;
     bool m_bInvert;
 
     GPUBlurFilter m_MinFilter;
     GPUBlurFilter m_MaxFilter;
+
+    IntGLShaderParamPtr m_pMinTexParam;
+    IntGLShaderParamPtr m_pMaxTexParam;
+    FloatGLShaderParamPtr m_pPostScaleParam;
+    IntGLShaderParamPtr m_pInvertParam;
 };
 
-} // namespace
+}
 #endif
 
