@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,26 +22,30 @@
 #include "GPUBrightnessFilter.h"
 #include "Bitmap.h"
 #include "ShaderRegistry.h"
+#include "OGLShader.h"
 
 #include "../base/ObjectCounter.h"
 #include "../base/Exception.h"
 
 #include <iostream>
 
-#define SHADERID "BRIGHTNESS"
+#define SHADERID "brightness"
 
 using namespace std;
 
 namespace avg {
 
-GPUBrightnessFilter::GPUBrightnessFilter(const IntPoint& size, PixelFormat pf, 
-        double alpha, bool bStandalone)
-    : GPUFilter(pf, B8G8R8A8, bStandalone),
+GPUBrightnessFilter::GPUBrightnessFilter(const IntPoint& size, bool bUseAlpha, 
+        float alpha, bool bStandalone)
+    : GPUFilter(SHADERID, bUseAlpha, bStandalone),
       m_Alpha(alpha)
 {
     ObjectCounter::get()->incRef(&typeid(*this));
     setDimensions(size);
-    initShader();
+
+    OGLShaderPtr pShader = getShader();
+    m_pTextureParam = pShader->getParam<int>("u_Texture");
+    m_pAlphaParam = pShader->getParam<float>("u_Alpha");
 }
 
 GPUBrightnessFilter::~GPUBrightnessFilter()
@@ -51,30 +55,10 @@ GPUBrightnessFilter::~GPUBrightnessFilter()
 
 void GPUBrightnessFilter::applyOnGPU(GLTexturePtr pSrcTex)
 {
-    OGLShaderPtr pShader = getShader(SHADERID);
-    pShader->activate();
-    pShader->setUniformIntParam("Texture", 0);
-    pShader->setUniformFloatParam("alpha", GLfloat(m_Alpha));
+    getShader()->activate();
+    m_pTextureParam->set(0);
+    m_pAlphaParam->set(m_Alpha);
     draw(pSrcTex);
-
-    glproc::UseProgramObject(0);
 }
 
-void GPUBrightnessFilter::initShader()
-{
-    string sProgram =
-        "uniform float alpha;\n"
-        "uniform sampler2D Texture;\n"
-
-        "void main(void)\n"
-        "{\n"
-        "    vec4 tex =texture2D(Texture, gl_TexCoord[0].st);\n" 
-        "    gl_FragColor.rgb = tex.rgb*alpha;\n"
-        "    gl_FragColor.a = tex.a;\n"
-        "}\n"
-        ;
-
-    getOrCreateShader(SHADERID, sProgram);
 }
-
-} // namespace
