@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,22 +26,24 @@
 #include "../api.h"
 
 #include "GPUFilter.h"
+#include "GLShaderParam.h"
 
 namespace avg {
 
 class AVG_API GPUInvertFilter : public GPUFilter
 {
 public:
-    GPUInvertFilter(const IntPoint& size, PixelFormat pf,
-            bool bStandalone=true);
+    GPUInvertFilter(const IntPoint& size, bool bUseAlpha, bool bStandalone=true);
     virtual ~GPUInvertFilter();
 
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
     void initShader();
 
+private:
+    IntGLShaderParamPtr m_pTextureParam;
 };
 
 typedef boost::shared_ptr<GPUInvertFilter> GPUInvertFilterPtr;
 
-} //end namespace avg
+}
 #endif
