@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,38 +24,53 @@
 
 #include "../api.h"
 #include "GPUFilter.h"
+#include "GLShaderParam.h"
 #include "GLTexture.h"
 
+#include "../base/GLMHelper.h"
+
 namespace avg {
 
 class AVG_API GPUShadowFilter: public GPUFilter
 {
 public:
-    GPUShadowFilter(const IntPoint& size, const DPoint& offset, double stdDev,
-            double opacity, const Pixel32& color);
+    GPUShadowFilter(const IntPoint& size, const glm::vec2& offset, float stdDev,
+            float opacity, const Pixel32& color);
     virtual ~GPUShadowFilter();
     
-    void setParams(const DPoint& offset, double stdDev, double opacity, 
+    void setParams(const glm::vec2& offset, float stdDev, float opacity, 
             const Pixel32& color);
     virtual void applyOnGPU(GLTexturePtr pSrcTex);
 
 private:
-    void initShaders();
-    void dumpKernel();
-    void calcKernel();
-    void setDimensions(IntPoint size, double stdDev, const DPoint& offset);
+    void setDimensions(IntPoint size, float stdDev, const glm::vec2& offset);
 
-    DPoint m_Offset;
-    double m_StdDev;
-    double m_Opacity;
+    glm::vec2 m_Offset;
+    float m_StdDev;
+    float m_Opacity;
     Pixel32 m_Color;
 
     GLTexturePtr m_pGaussCurveTex;
     ImagingProjectionPtr m_pProjection2;
+
+    FloatGLShaderParamPtr m_pHorizWidthParam;
+    IntGLShaderParamPtr m_pHorizRadiusParam;
+    IntGLShaderParamPtr m_pHorizTextureParam;
+    IntGLShaderParamPtr m_pHorizKernelTexParam;
+    Vec2fGLShaderParamPtr m_pHorizOffsetParam;
+
+    FloatGLShaderParamPtr m_pVertWidthParam;
+    IntGLShaderParamPtr m_pVertRadiusParam;
+    IntGLShaderParamPtr m_pVertTextureParam;
+    IntGLShaderParamPtr m_pVertKernelTexParam;
+    ColorGLShaderParamPtr m_pVertColorParam;
+    IntGLShaderParamPtr m_pVertOrigTexParam;
+    Vec2fGLShaderParamPtr m_pVertDestPosParam;
+    Vec2fGLShaderParamPtr m_pVertDestSizeParam;
 };
 
 typedef boost::shared_ptr<GPUShadowFilter> GPUShadowFilterPtr;
 
-} // namespace
+}
 #endif
 
