@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -43,10 +43,9 @@ PBO::PBO(const IntPoint& size, PixelFormat pf, unsigned usage)
     
     unsigned target = getTarget();
     glproc::BindBuffer(target, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO: BindBuffer()");
-    int memNeeded = size.x*size.y*getBytesPerPixel(pf);
-    glproc::BufferData(target, memNeeded, 0, usage);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO: BufferData()");
+    GLContext::checkError("PBO: BindBuffer()");
+    glproc::BufferData(target, getMemNeeded(), 0, usage);
+    GLContext::checkError("PBO: BufferData()");
     glproc::BindBuffer(target, 0);
 }
 
@@ -59,34 +58,36 @@ PBO::~PBO()
         pContext->getPBOCache().returnBuffer(m_PBOID);
     }
     glproc::BindBuffer(getTarget(), 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO: DeleteBuffers()");
+    GLContext::checkError("PBO: DeleteBuffers()");
     ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 void PBO::activate()
 {
     glproc::BindBuffer(getTarget(), m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::activate()");
-    
+    GLContext::checkError("PBO::activate()");  
+}
+
+int PBO::getID() const
+{
+    return m_PBOID;
 }
 
 void PBO::moveBmpToTexture(BitmapPtr pBmp, GLTexture& tex)
 {
-
     AVG_ASSERT(pBmp->getSize() == tex.getSize());
     AVG_ASSERT(getSize() == pBmp->getSize());
     AVG_ASSERT(pBmp->getPixelFormat() == getPF());
     AVG_ASSERT(tex.getPF() == getPF());
     AVG_ASSERT(!isReadPBO());
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::moveBmpToTexture BindBuffer()");
+    GLContext::checkError("PBO::moveBmpToTexture BindBuffer()");
     void * pPBOPixels = glproc::MapBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, GL_WRITE_ONLY);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::moveBmpToTexture MapBuffer()");
-    Bitmap PBOBitmap(getSize(), getPF(), (unsigned char *)pPBOPixels, 
-            getSize().x*getBytesPerPixel(getPF()), false); 
+    GLContext::checkError("PBO::moveBmpToTexture MapBuffer()");
+    Bitmap PBOBitmap(getSize(), getPF(), (unsigned char *)pPBOPixels, getStride(), false);
     PBOBitmap.copyPixels(*pBmp);
     glproc::UnmapBuffer(GL_PIXEL_UNPACK_BUFFER_EXT);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::setImage: UnmapBuffer()");
+    GLContext::checkError("PBO::setImage: UnmapBuffer()");
 
     tex.setDirty();
     moveToTexture(tex);
@@ -104,13 +105,13 @@ void PBO::moveTextureToPBO(GLTexture& tex, int mipmapLevel)
     AVG_ASSERT(getSize() == tex.getGLSize());
     AVG_ASSERT(getPF() == tex.getPF());
     glproc::BindBuffer(GL_PIXEL_PACK_BUFFER_EXT, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::getImage BindBuffer()");
+    GLContext::checkError("PBO::getImage BindBuffer()");
 
     tex.activate(GL_TEXTURE0);
 
     glGetTexImage(GL_TEXTURE_2D, mipmapLevel, GLTexture::getGLFormat(getPF()), 
             GLTexture::getGLType(getPF()), 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::getImage: glGetTexImage()");
+    GLContext::checkError("PBO::getImage: glGetTexImage()");
     if (mipmapLevel == 0) {
         m_ActiveSize = tex.getSize();
         m_BufferStride = tex.getGLSize().x;
@@ -124,15 +125,15 @@ BitmapPtr PBO::movePBOToBmp() const
 {
     AVG_ASSERT(isReadPBO());
     glproc::BindBuffer(GL_PIXEL_PACK_BUFFER_EXT, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::getImage BindBuffer()");
+    GLContext::checkError("PBO::getImage BindBuffer()");
     void * pPBOPixels = glproc::MapBuffer(GL_PIXEL_PACK_BUFFER_EXT, GL_READ_ONLY);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::getImage MapBuffer()");
+    GLContext::checkError("PBO::getImage MapBuffer()");
     Bitmap PBOBitmap(m_ActiveSize, getPF(), (unsigned char *)pPBOPixels, 
             m_BufferStride*getBytesPerPixel(getPF()), false);
     BitmapPtr pBmp(new Bitmap(m_ActiveSize, getPF()));
     pBmp->copyPixels(PBOBitmap);
     glproc::UnmapBuffer(GL_PIXEL_PACK_BUFFER_EXT);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::getImage: UnmapBuffer()");
+    GLContext::checkError("PBO::getImage: UnmapBuffer()");
     glproc::BindBuffer(GL_PIXEL_PACK_BUFFER_EXT, 0);
     
     return pBmp;
@@ -143,18 +144,16 @@ BitmapPtr PBO::lock()
     AVG_ASSERT(!isReadPBO());
     BitmapPtr pBmp;
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::lockBmp: glBindBuffer()");
-    glproc::BufferData(GL_PIXEL_UNPACK_BUFFER_EXT, 
-            getSize().x*getSize().y*getBytesPerPixel(getPF()), 0, m_Usage);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::lockBmp: glBufferData()");
+    GLContext::checkError("PBOTexture::lockBmp: glBindBuffer()");
+    glproc::BufferData(GL_PIXEL_UNPACK_BUFFER_EXT, getMemNeeded(), 0, m_Usage);
+    GLContext::checkError("PBOTexture::lockBmp: glBufferData()");
     unsigned char * pBuffer = (unsigned char *)
         glproc::MapBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, GL_WRITE_ONLY);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::lockBmp: glMapBuffer()");
+    GLContext::checkError("PBOTexture::lockBmp: glMapBuffer()");
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::lockBmp: glBindBuffer(0)");
+    GLContext::checkError("PBOTexture::lockBmp: glBindBuffer(0)");
 
-    pBmp = BitmapPtr(new Bitmap(getSize(), getPF(), pBuffer, 
-                getSize().x*getBytesPerPixel(getPF()), false));
+    pBmp = BitmapPtr(new Bitmap(getSize(), getPF(), pBuffer, getStride(), false));
     return pBmp;
 }
 
@@ -162,11 +161,11 @@ void PBO::unlock()
 {
     AVG_ASSERT(!isReadPBO());
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::unlockBmp: glBindBuffer()");
+    GLContext::checkError("PBOTexture::unlockBmp: glBindBuffer()");
     glproc::UnmapBuffer(GL_PIXEL_UNPACK_BUFFER_EXT);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::unlockBmp: glUnmapBuffer()");
+    GLContext::checkError("PBOTexture::unlockBmp: glUnmapBuffer()");
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::unlockBmp: glBindBuffer(0)");
+    GLContext::checkError("PBOTexture::unlockBmp: glBindBuffer(0)");
 }
 
 void PBO::moveToTexture(GLTexture& tex)
@@ -180,11 +179,19 @@ void PBO::moveToTexture(GLTexture& tex)
         size.y = getSize().y;
     } 
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_PBOID);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBOTexture::lockBmp: glBindBuffer()");
+    GLContext::checkError("PBOTexture::lockBmp: glBindBuffer()");
     tex.activate(GL_TEXTURE0);
+#ifdef __APPLE__
+    // See getStride()
+    if (getPF() == A8) {
+        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+    } else {
+        glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
+    }
+#endif
     glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, size.x, size.y,
             GLTexture::getGLFormat(getPF()), GLTexture::getGLType(getPF()), 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "PBO::setImage: glTexSubImage2D()");
+    GLContext::checkError("PBO::setImage: glTexSubImage2D()");
     glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, 0);
     tex.setDirty();
     tex.generateMipmaps();
@@ -207,6 +214,25 @@ bool PBO::isReadPBO() const
     }
 }
 
+unsigned PBO::getMemNeeded() const
+{
+    return getStride()*getSize().y;
+}
+
+unsigned PBO::getStride() const
+{
+    IntPoint size = getSize();
+    unsigned stride = Bitmap::getPreferredStride(size.x, getPF());
+#ifdef __APPLE__
+    if (getPF() == A8) {
+        // Workaround for apparent bug in Apple/NVidia drivers (Verified on OS X 10.6.8,
+        // MBP early 2011): GL_UNPACK_ALIGNMENT != 1 causes broken A8 textures.
+        stride = size.x;
+    }
+#endif
+    return stride;
+}
+
 unsigned PBO::getTarget() const
 {
     if (isReadPBO()) {
