@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 
 #include "../api.h"
 #include <string>
+#include <vector>
 
 namespace avg {
 
@@ -52,12 +53,14 @@ AVG_API std::ostream& operator <<(std::ostream& os, PixelFormat pf);
 
 std::string AVG_API getPixelFormatString(PixelFormat pf);
 PixelFormat AVG_API stringToPixelFormat(const std::string& s);
+std::vector<std::string> AVG_API getSupportedPixelFormats();
 bool AVG_API pixelFormatIsColored(PixelFormat pf);
 bool AVG_API pixelFormatIsBayer(PixelFormat pf);
 bool AVG_API pixelFormatHasAlpha(PixelFormat pf);
 bool AVG_API pixelFormatIsPlanar(PixelFormat pf);
+bool AVG_API pixelFormatIsBlueFirst(PixelFormat pf);
 unsigned AVG_API getNumPixelFormatPlanes(PixelFormat pf);
-unsigned getBytesPerPixel(PixelFormat pf);
+unsigned AVG_API getBytesPerPixel(PixelFormat pf);
 
 }
 #endif
