@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@
 #include "../graphics/Bitmap.h"
 #include "../graphics/Pixel24.h"
 
-#include "../base/Point.h"
+#include "../base/GLMHelper.h"
 
 #ifdef AVG_ENABLE_1394_2
 #include <dc1394/control.h>
@@ -38,9 +38,6 @@
 #ifndef AVG_ENABLE_1394_2
 typedef unsigned int dc1394feature_t;
 #endif
-#ifndef uint64_t
-#define uint64_t unsigned long long
-#endif
 
 #include <string>
 #include <map>
@@ -49,17 +46,15 @@ namespace avg {
 
 class AVG_API FWCamera: public Camera {
 public:
-    FWCamera(uint64_t guid, int unit, bool bFW800, IntPoint size, PixelFormat camPF, 
-            PixelFormat destPF, double frameRate);
+    FWCamera(unsigned long long guid, int unit, bool bFW800, IntPoint size,
+            PixelFormat camPF, PixelFormat destPF, float frameRate);
     virtual ~FWCamera();
     virtual void startCapture();
 
-    virtual IntPoint getImgSize();
     virtual BitmapPtr getImage(bool bWait);
 
     virtual const std::string& getDevice() const; 
     virtual const std::string& getDriverName() const; 
-    virtual double getFrameRate() const;
 
     virtual int getFeature(CameraFeature feature) const;
     bool hasFeature(CameraFeature feature);
@@ -69,7 +64,8 @@ public:
     virtual int getWhitebalanceV() const;
     virtual void setWhitebalance(int u, int v, bool bIgnoreOldValue=false);
 
-    static void dumpCameras();
+    static int countCameras();
+    static CameraInfo* getCameraInfos(int deviceNumber);
     static void resetBus();
 
 private:
@@ -78,15 +74,14 @@ private:
     void getWhitebalance(int* pU, int* pV) const;
     void enablePtGreyBayer();
 
-    IntPoint m_Size;
-    double m_FrameRate;
-
 #ifdef AVG_ENABLE_1394_2
+    static void getCameraImageFormats(dc1394camera_t* pCamera, CameraInfo* camInfo);
+    static void getCameraControls(dc1394camera_t* pCamera, CameraInfo* camInfo);
+
     dc1394_t * m_pDC1394;
     dc1394camera_t * m_pCamera;
     dc1394framerate_t m_FrameRateConstant; 
-    dc1394video_mode_t m_Mode;            
-    static void dumpCameraInfo(dc1394camera_t * pCamera);
+    dc1394video_mode_t m_Mode;
 #endif
 
     FeatureMap m_Features;
