@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 #include "../graphics/Filterfill.h"
 #include "../graphics/Filterfillrect.h"
 #include "../graphics/Filtergrayscale.h"
+#include "../graphics/BitmapLoader.h"
 
 #include "../base/TimeSource.h"
 #include "../base/Exception.h"
@@ -36,27 +37,25 @@ using namespace std;
 namespace avg {
 
 FakeCamera::FakeCamera(PixelFormat camPF, PixelFormat destPF)
-    : Camera(camPF, destPF),
+    : Camera(camPF, destPF, IntPoint(640, 480), 60),
       m_pBmpQ(new std::queue<BitmapPtr>()),
       m_bIsOpen(false)
 {
-    m_ImgSize = IntPoint(640, 480);
 }
 
 FakeCamera::FakeCamera(std::vector<std::string>& pictures)
-    : Camera(I8, I8),
+    : Camera(I8, I8, IntPoint(640,480), 60),
       m_pBmpQ(new std::queue<BitmapPtr>()),
       m_bIsOpen(false)
 {
-    m_ImgSize = IntPoint(640, 480);
     for (vector<string>::iterator it = pictures.begin(); it != pictures.end(); ++it) {
         try {
-            BitmapPtr pBmp (new Bitmap(*it));
+            BitmapPtr pBmp = loadBitmap(*it);
             FilterGrayscale().applyInPlace(pBmp);
-            m_ImgSize = pBmp->getSize();
+            setImgSize(pBmp->getSize());
             m_pBmpQ->push(pBmp);
         } catch (Exception& ex) {
-            AVG_TRACE(Logger::ERROR, ex.getStr());
+            AVG_LOG_ERROR(ex.getStr());
             throw;
         }
     }
@@ -77,11 +76,6 @@ void FakeCamera::close()
 }
 
 
-IntPoint FakeCamera::getImgSize()
-{
-    return m_ImgSize;
-}
-
 BitmapPtr FakeCamera::getImage(bool bWait)
 {
     if (bWait) {
@@ -114,11 +108,6 @@ const std::string& FakeCamera::getDriverName() const
     return sDriverName;
 }
 
-double FakeCamera::getFrameRate() const
-{
-    return 60;
-}
-
 const string& FakeCamera::getMode() const
 {
     static string sMode = "FakeCamera";
