@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -43,13 +43,11 @@ public:
     virtual void open();
     virtual void close();
 
-    virtual IntPoint getImgSize();
     virtual BitmapPtr getImage(bool bWait);
     virtual bool isCameraAvailable();
 
     virtual const std::string& getDevice() const; 
     virtual const std::string& getDriverName() const; 
-    virtual double getFrameRate() const;
     virtual const std::string& getMode() const;
 
     virtual int getFeature(CameraFeature feature) const;
@@ -60,7 +58,6 @@ public:
     virtual void setWhitebalance(int u, int v, bool bIgnoreOldValue=false);
 
 private:
-    IntPoint m_ImgSize;
     BitmapQueuePtr m_pBmpQ;
     bool m_bIsOpen;
 };
