@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,8 @@
 //
 
 #include "Arg.h"
+#include "FontStyle.h"
+
 #include <string>
 
 using namespace std;
@@ -31,14 +33,15 @@ namespace avg {
 #ifndef _WIN32
 template class Arg<int>;
 template class Arg<bool>;
-template class Arg<double>;
 template class Arg<float>;
 template class Arg<string>;
-template class Arg<DPoint>;
-template class Arg<IntTriple>;
-template class Arg<DTriple>;
-template class Arg<vector<double> >;
-template class Arg<vector<DPoint> >;
-template class Arg<vector<IntTriple> >;
+template class Arg<glm::vec2>;
+template class Arg<glm::vec3>;
+template class Arg<glm::ivec3>;
+template class Arg<std::vector<float> >;
+template class Arg<std::vector<int> >;
+template class Arg<vector<glm::vec2> >;
+template class Arg<vector<glm::ivec3> >;
+template class Arg<FontStyle>;
 #endif
 }
