@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,8 +25,7 @@
 #define _Arg_H_
 
 #include "../api.h"
-#include "../base/Point.h"
-#include "../base/Triple.h"
+#include "../base/GLMHelper.h"
 
 #include "ArgBase.h"
 
@@ -35,7 +34,7 @@
 
 namespace avg {
 
-class Node;
+class ExportedObject;
 
 template<class T>
 class AVG_TEMPLATE_API Arg: public ArgBase
@@ -47,7 +46,7 @@ public:
 
     void setValue(const T& Value);
     const T& getValue() const;
-    virtual void setMember(Node * pNode) const;
+    virtual void setMember(ExportedObject * pObj) const;
     virtual ArgBase* createCopy() const;
 
 private:
@@ -81,10 +80,10 @@ void Arg<T>::setValue(const T& Value)
 }
 
 template<class T>
-void Arg<T>::setMember(Node * pNode) const
+void Arg<T>::setMember(ExportedObject * pObj) const
 {
     if (getMemberOffset() != -1) {
-        T* pMember = (T*)((char*)pNode+getMemberOffset());
+        T* pMember = (T*)((char*)pObj+getMemberOffset());
         *pMember = m_Value;
     }
 }
@@ -103,14 +102,14 @@ ArgBase* Arg<T>::createCopy() const
 extern template class Arg<int>;
 extern template class Arg<bool>;
 extern template class Arg<float>;
-extern template class Arg<double>;
 extern template class Arg<std::string>;
-extern template class Arg<DPoint>;
-extern template class Arg<IntTriple>;
-extern template class Arg<DTriple>;
-extern template class Arg<std::vector<double> >;
-extern template class Arg<std::vector<DPoint> >;
-extern template class Arg<std::vector<IntTriple> >;
+extern template class Arg<glm::vec2>;
+extern template class Arg<glm::vec3>;
+extern template class Arg<glm::ivec3>;
+extern template class Arg<std::vector<float> >;
+extern template class Arg<std::vector<int> >;
+extern template class Arg<std::vector<glm::vec2> >;
+extern template class Arg<std::vector<glm::ivec2> >;
 #endif
 #endif
 
