@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,7 +32,7 @@
 
 namespace avg {
 
-class Node;
+class ExportedObject;
 
 class AVG_API ArgBase
 {
@@ -44,7 +44,7 @@ public:
     bool isDefault() const;
     bool isRequired() const;
     
-    virtual void setMember(Node * pNode) const = 0;
+    virtual void setMember(ExportedObject * pObj) const = 0;
    
     virtual ArgBase* createCopy() const = 0;
 
