@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,7 @@
 #include "RasterNode.h"
 
 #include "../imaging/Camera.h"
+#include "../imaging/CameraInfo.h"
 
 #include <boost/thread/thread.hpp>
 
@@ -38,11 +39,12 @@ namespace avg {
 
 class TextureMover;
 typedef boost::shared_ptr<TextureMover> TextureMoverPtr;
+typedef std::vector<CameraInfo> CamerasInfosVector;
 
 class AVG_API CameraNode : public RasterNode
 {
     public:
-        static NodeDefinition createDefinition();
+        static void registerType();
         
         CameraNode(const ArgList& args);
         virtual ~CameraNode();
@@ -65,7 +67,7 @@ class AVG_API CameraNode : public RasterNode
             return m_pCamera->getDriverName();
         }
 
-        double getFrameRate() const
+        float getFrameRate() const
         {
             return m_pCamera->getFrameRate();
         }
@@ -94,21 +96,22 @@ class AVG_API CameraNode : public RasterNode
         void setAutoUpdateCameraImage(bool bVal);
         bool isImageAvailable() const;
 
-        virtual void preRender();
-        virtual void render(const DRect& Rect);
+        virtual void preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+                float parentEffectiveOpacity);
+        virtual void render();
 
         int getFrameNum() const;
         IntPoint getMediaSize();
         virtual BitmapPtr getBitmap();
 
-        static void dumpCameras();
+        static CamerasInfosVector getCamerasInfos();
         static void resetFirewireBus();
 
     private:
         int getFeature (CameraFeature feature) const;
         void setFeature (CameraFeature feature, int value);
 
-        virtual double getFPS() const;
+        virtual float getFPS() const;
         virtual void open();
         virtual PixelFormat getPixelFormat();
         void setFeature(int FeatureID);
