@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,6 @@
 #include "ChromaKeyFXNode.h"
 
 #include "../base/ObjectCounter.h"
-#include "../graphics/ShaderRegistry.h"
 
 #include <string>
 
@@ -31,7 +30,7 @@ using namespace std;
 namespace avg {
 
 ChromaKeyFXNode::ChromaKeyFXNode() 
-    : FXNode(),
+    : FXNode(false),
       m_sColorName("00FF00"),
       m_Color(0, 255, 0),
       m_HTolerance(0.0),
@@ -68,46 +67,46 @@ const std::string& ChromaKeyFXNode::getColor() const
     return m_sColorName;
 }
 
-void ChromaKeyFXNode::setHTolerance(double tolerance)
+void ChromaKeyFXNode::setHTolerance(float tolerance)
 {
     m_HTolerance = tolerance;
     updateFilter();
 }
 
-double ChromaKeyFXNode::getHTolerance() const
+float ChromaKeyFXNode::getHTolerance() const
 {
     return m_HTolerance;
 }
 
-void ChromaKeyFXNode::setSTolerance(double tolerance)
+void ChromaKeyFXNode::setSTolerance(float tolerance)
 {
     m_STolerance = tolerance;
     updateFilter();
 }
 
-double ChromaKeyFXNode::getSTolerance() const
+float ChromaKeyFXNode::getSTolerance() const
 {
     return m_STolerance;
 }
 
-void ChromaKeyFXNode::setLTolerance(double tolerance)
+void ChromaKeyFXNode::setLTolerance(float tolerance)
 {
     m_LTolerance = tolerance;
     updateFilter();
 }
 
-double ChromaKeyFXNode::getLTolerance() const
+float ChromaKeyFXNode::getLTolerance() const
 {
     return m_LTolerance;
 }
 
-void ChromaKeyFXNode::setSoftness(double softness)
+void ChromaKeyFXNode::setSoftness(float softness)
 {
     m_Softness = softness;
     updateFilter();
 }
 
-double ChromaKeyFXNode::getSoftness() const
+float ChromaKeyFXNode::getSoftness() const
 {
     return m_Softness;
 }
@@ -123,20 +122,20 @@ int ChromaKeyFXNode::getErosion() const
     return m_Erosion;
 }
 
-void ChromaKeyFXNode::setSpillThreshold(double spillThreshold)
+void ChromaKeyFXNode::setSpillThreshold(float spillThreshold)
 {
     m_SpillThreshold = spillThreshold;
     updateFilter();
 }
 
-double ChromaKeyFXNode::getSpillThreshold() const
+float ChromaKeyFXNode::getSpillThreshold() const
 {
     return m_SpillThreshold;
 }
 
 GPUFilterPtr ChromaKeyFXNode::createFilter(const IntPoint& size)
 {
-    m_pFilter = GPUChromaKeyFilterPtr(new GPUChromaKeyFilter(size, B8G8R8A8, false));
+    m_pFilter = GPUChromaKeyFilterPtr(new GPUChromaKeyFilter(size, false));
     m_pFilter->setParams(m_Color, m_HTolerance, m_STolerance, m_LTolerance, m_Softness,
             m_Erosion, m_SpillThreshold);
     setDirty();
