@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -106,24 +106,22 @@ string Event::typeStr() const
 string Event::typeStr(Event::Type type)
 {
     switch(type) {
-        case KEYUP:
-            return "KEYUP";
-        case KEYDOWN:
-            return "KEYDOWN";
-        case CURSORMOTION:
-            return "CURSORMOTION";
-        case CURSORUP:
-            return "CURSORUP";
-        case CURSORDOWN:
-            return "CURSORDOWN";
-        case CURSOROVER:
-            return "CURSOROVER";
-        case CURSOROUT:
-            return "CURSOROUT";
-        case CUSTOMEVENT:
-            return "CUSTOMEVENT";
-        case RESIZE:
-            return "RESIZE";
+        case KEY_UP:
+            return "KEY_UP";
+        case KEY_DOWN:
+            return "KEY_DOWN";
+        case CURSOR_MOTION:
+            return "CURSOR_MOTION";
+        case CURSOR_UP:
+            return "CURSOR_UP";
+        case CURSOR_DOWN:
+            return "CURSOR_DOWN";
+        case CURSOR_OVER:
+            return "CURSOR_OVER";
+        case CURSOR_OUT:
+            return "CURSOR_OUT";
+        case CUSTOM_EVENT:
+            return "CUSTOM_EVENT";
         case QUIT:
             return "QUIT";
         default:
@@ -135,7 +133,7 @@ string Event::typeStr(Event::Type type)
 void Event::trace()
 {
     string sType = typeStr();
-    AVG_TRACE(Logger::EVENTS, sType); 
+    AVG_TRACE(Logger::category::EVENTS,Logger::severity::INFO, sType); 
 }
 
 }
