@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -39,15 +39,15 @@ typedef boost::weak_ptr<class IInputDevice> IInputDeviceWeakPtr;
 class AVG_API Event {
     public:
         enum Type {
-            KEYUP,
-            KEYDOWN,
-            CURSORMOTION,
-            CURSORUP,
-            CURSORDOWN,
-            CURSOROVER,  
-            CURSOROUT,
-            CUSTOMEVENT,
-            RESIZE,
+            // XXX: Hack to make sure this enum can't be passed to Node.subscribe.
+            KEY_UP = 10000,
+            KEY_DOWN,
+            CURSOR_MOTION,
+            CURSOR_UP,
+            CURSOR_DOWN,
+            CURSOR_OVER,  
+            CURSOR_OUT,
+            CUSTOM_EVENT,
             QUIT 
         };
         enum Source {MOUSE=1, TOUCH=2, TRACK=4, CUSTOM=8, NONE=16};
