@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,7 +32,7 @@ namespace avg {
 
 class AVG_API FXNode {
 public:
-    FXNode();
+    FXNode(bool bSupportsGLES=true);
     virtual ~FXNode();
 
     virtual void connect();
@@ -43,7 +43,7 @@ public:
 
     GLTexturePtr getTex();
     BitmapPtr getImage();
-    DRect getRelDestRect() const;
+    FRect getRelDestRect() const;
 
     bool isDirty() const;
     void resetDirty();
@@ -54,10 +54,12 @@ protected:
 
 private:
     virtual GPUFilterPtr createFilter(const IntPoint& size) = 0;
+    void checkGLES() const;
 
     IntPoint m_Size;
     GPUFilterPtr m_pFilter;
     
+    bool m_bSupportsGLES;
     bool m_bDirty;
 };
 
