@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,11 +21,13 @@
 
 #include "RasterNode.h"
 
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 #include "OGLSurface.h"
 #include "FXNode.h"
 
 #include "../graphics/ImagingProjection.h"
+#include "../graphics/ShaderRegistry.h"
+#include "../graphics/BitmapLoader.h"
 
 #include "../base/MathHelper.h"
 #include "../base/Logger.h"
@@ -37,10 +39,9 @@ using namespace std;
 
 namespace avg {
 
-NodeDefinition RasterNode::createDefinition()
+void RasterNode::registerType()
 {
-    return NodeDefinition("rasternode")
-        .extendDefinition(AreaNode::createDefinition())
+    TypeDefinition def = TypeDefinition("rasternode", "areanode")
         .addArg(Arg<int>("maxtilewidth", -1, false, 
                 offsetof(RasterNode, m_MaxTileSize.x)))
         .addArg(Arg<int>("maxtileheight", -1, false, 
@@ -49,24 +50,23 @@ NodeDefinition RasterNode::createDefinition()
                 offsetof(RasterNode, m_sBlendMode)))
         .addArg(Arg<bool>("mipmap", false))
         .addArg(Arg<UTF8String>("maskhref", "", false, offsetof(RasterNode, m_sMaskHref)))
-        .addArg(Arg<DPoint>("maskpos", DPoint(0,0), false,
+        .addArg(Arg<glm::vec2>("maskpos", glm::vec2(0,0), false,
                 offsetof(RasterNode, m_MaskPos)))
-        .addArg(Arg<DPoint>("masksize", DPoint(0,0), false,
+        .addArg(Arg<glm::vec2>("masksize", glm::vec2(0,0), false,
                 offsetof(RasterNode, m_MaskSize)))
-        .addArg(Arg<DTriple>("gamma", DTriple(1.0,1.0,1.0), false,
+        .addArg(Arg<glm::vec3>("gamma", glm::vec3(1.0f,1.0f,1.0f), false,
                 offsetof(RasterNode, m_Gamma)))
-        .addArg(Arg<DTriple>("contrast", DTriple(1.0,1.0,1.0), false,
+        .addArg(Arg<glm::vec3>("contrast", glm::vec3(1.0f,1.0f,1.0f), false,
                 offsetof(RasterNode, m_Contrast)))
-        .addArg(Arg<DTriple>("intensity", DTriple(1.0,1.0,1.0), false,
+        .addArg(Arg<glm::vec3>("intensity", glm::vec3(1.0f,1.0f,1.0f), false,
                 offsetof(RasterNode, m_Intensity)));
+    TypeRegistry::get()->registerType(def);
 }
 
 RasterNode::RasterNode()
     : m_pSurface(0),
       m_Material(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE, false),
-      m_bBound(false),
       m_TileSize(-1,-1),
-      m_pVertexes(0),
       m_bFXDirty(true)
 {
 }
@@ -95,15 +95,12 @@ void RasterNode::setArgs(const ArgList& args)
 
 void RasterNode::connectDisplay()
 {
-    checkMaskSupport(m_sMaskHref);
     AreaNode::connectDisplay();
 
-    m_pSurface->attach();
-    m_bBound = false;
     if (m_MaxTileSize != IntPoint(-1, -1)) {
         m_TileSize = m_MaxTileSize;
     }
-    calcVertexGrid(m_TileVertices);
+    newSurface();
     setBlendModeStr(m_sBlendMode);
     if (m_pMaskBmp) {
         downloadMask();
@@ -115,10 +112,6 @@ void RasterNode::connectDisplay()
 
 void RasterNode::disconnect(bool bKill)
 {
-    if (m_pVertexes) {
-        delete m_pVertexes;
-        m_pVertexes = 0;
-    }
     if (m_pSurface) {
         m_pSurface->destroy();
     }
@@ -143,13 +136,9 @@ void RasterNode::checkReload()
         m_sMaskFilename = sMaskFilename;
         try {
             if (m_sMaskFilename != "") {
-                AVG_TRACE(Logger::MEMORY, "Loading " << m_sMaskFilename);
-                m_pMaskBmp = BitmapPtr(new Bitmap(m_sMaskFilename));
-                if (m_pMaskBmp->getPixelFormat() != I8) {
-                    BitmapPtr pTempBmp = m_pMaskBmp;
-                    m_pMaskBmp = BitmapPtr(new Bitmap(m_pMaskBmp->getSize(), I8));
-                    m_pMaskBmp->copyPixels(*pTempBmp);
-                }
+                AVG_TRACE(Logger::category::MEMORY, Logger::severity::INFO,
+                        "Loading " << m_sMaskFilename);
+                m_pMaskBmp = loadBitmap(m_sMaskFilename, I8);
                 setMaskCoords();
             }
         } catch (Exception & ex) {
@@ -158,9 +147,9 @@ void RasterNode::checkReload()
             }
             m_sMaskFilename = "";
             if (getState() != Node::NS_UNCONNECTED) {
-                AVG_TRACE(Logger::ERROR, ex.getStr());
+                AVG_LOG_ERROR(ex.getStr());
             } else {
-                AVG_TRACE(Logger::MEMORY, ex.getStr());
+                AVG_TRACE(Logger::category::MEMORY, Logger::severity::ERROR, ex.getStr());
             }
         }
         if (m_sMaskFilename == "") {
@@ -178,9 +167,6 @@ void RasterNode::checkReload()
 VertexGrid RasterNode::getOrigVertexCoords()
 {
     checkDisplayAvailable("getOrigVertexCoords");
-    if (!m_bBound) {
-        bind();
-    }
     VertexGrid grid;
     calcVertexGrid(grid);
     return grid;
@@ -189,18 +175,12 @@ VertexGrid RasterNode::getOrigVertexCoords()
 VertexGrid RasterNode::getWarpedVertexCoords() 
 {
     checkDisplayAvailable("getWarpedVertexCoords");
-    if (!m_bBound) {
-        bind();
-    }
     return m_TileVertices;
 }
 
 void RasterNode::setWarpedVertexCoords(const VertexGrid& grid)
 {
     checkDisplayAvailable("setWarpedVertexCoords");
-    if (!m_bBound) {
-        bind();
-    }
     bool bGridOK = true;
     IntPoint numTiles = getNumTiles();
     if (grid.size() != (unsigned)(numTiles.y+1)) {
@@ -216,7 +196,6 @@ void RasterNode::setWarpedVertexCoords(const VertexGrid& grid)
                 "setWarpedVertexCoords() called with incorrect grid size.");
     }
     m_TileVertices = grid;
-    m_bVertexArrayDirty = true;
 }
 
 int RasterNode::getMaxTileWidth() const
@@ -241,8 +220,15 @@ const std::string& RasterNode::getBlendModeStr() const
 
 void RasterNode::setBlendModeStr(const string& sBlendMode)
 {
+    GLContext::BlendMode blendMode = GLContext::stringToBlendMode(sBlendMode);
+    if (!GLContext::getMain()->isBlendModeSupported(blendMode)) {
+        m_sBlendMode = "blend";
+        m_BlendMode = GLContext::BLEND_BLEND;
+        throw Exception(AVG_ERR_UNSUPPORTED, 
+              "Min and max blend modes are not supported in this OpenGL configuration.");
+    }
     m_sBlendMode = sBlendMode;
-    m_BlendMode = GLContext::stringToBlendMode(sBlendMode);
+    m_BlendMode = blendMode;
 }
 
 const UTF8String& RasterNode::getMaskHRef() const
@@ -252,36 +238,33 @@ const UTF8String& RasterNode::getMaskHRef() const
 
 void RasterNode::setMaskHRef(const UTF8String& sHref)
 {
-    if (GLContext::getCurrent()) {
-        checkMaskSupport(sHref);
-    }
     m_sMaskHref = sHref;
     checkReload();
 }
 
-const DPoint& RasterNode::getMaskPos() const
+const glm::vec2& RasterNode::getMaskPos() const
 {
     return m_MaskPos;
 }
 
-void RasterNode::setMaskPos(const DPoint& pos)
+void RasterNode::setMaskPos(const glm::vec2& pos)
 {
     m_MaskPos = pos;
     setMaskCoords();
 }
 
-const DPoint& RasterNode::getMaskSize() const
+const glm::vec2& RasterNode::getMaskSize() const
 {
     return m_MaskSize;
 }
 
-void RasterNode::setMaskSize(const DPoint& size)
+void RasterNode::setMaskSize(const glm::vec2& size)
 {
     m_MaskSize = size;
     setMaskCoords();
 }
 
-void RasterNode::getElementsByPos(const DPoint& pos, vector<NodeWeakPtr>& pElements)
+void RasterNode::getElementsByPos(const glm::vec2& pos, vector<NodePtr>& pElements)
 {
     // Node isn't pickable if it's warped.
     if (m_MaxTileSize == IntPoint(-1, -1)) {
@@ -289,12 +272,12 @@ void RasterNode::getElementsByPos(const DPoint& pos, vector<NodeWeakPtr>& pEleme
     }
 }
 
-DTriple RasterNode::getGamma() const
+glm::vec3 RasterNode::getGamma() const
 {
     return m_Gamma;
 }
 
-void RasterNode::setGamma(const DTriple& gamma)
+void RasterNode::setGamma(const glm::vec3& gamma)
 {
     m_Gamma = gamma;
     if (getState() == Node::NS_CANRENDER) {
@@ -302,12 +285,12 @@ void RasterNode::setGamma(const DTriple& gamma)
     }
 }
 
-DTriple RasterNode::getIntensity() const
+glm::vec3 RasterNode::getIntensity() const
 {
     return m_Intensity;
 }
 
-void RasterNode::setIntensity(const DTriple& intensity)
+void RasterNode::setIntensity(const glm::vec3& intensity)
 {
     m_Intensity = intensity;
     if (getState() == Node::NS_CANRENDER) {
@@ -315,12 +298,12 @@ void RasterNode::setIntensity(const DTriple& intensity)
     }
 }
 
-DTriple RasterNode::getContrast() const
+glm::vec3 RasterNode::getContrast() const
 {
     return m_Contrast;
 }
 
-void RasterNode::setContrast(const DTriple& contrast)
+void RasterNode::setContrast(const glm::vec3& contrast)
 {
     m_Contrast = contrast;
     if (getState() == Node::NS_CANRENDER) {
@@ -342,16 +325,35 @@ void RasterNode::setEffect(FXNodePtr pFXNode)
     }
 }
 
-void RasterNode::blt32(const DPoint& destSize, double opacity, 
-        GLContext::BlendMode mode, bool bPremultipliedAlpha)
+void RasterNode::calcVertexArray(const VertexArrayPtr& pVA, const Pixel32& color)
 {
-    blt(destSize, mode, opacity, Pixel32(255, 255, 255, 255), bPremultipliedAlpha);
+    if (isVisible() && m_pSurface->isCreated()) {
+        pVA->startSubVA(m_SubVA);
+        for (unsigned y = 0; y < m_TileVertices.size()-1; y++) {
+            for (unsigned x = 0; x < m_TileVertices[0].size()-1; x++) {
+                int curVertex = m_SubVA.getNumVerts();
+                m_SubVA.appendPos(m_TileVertices[y][x], m_TexCoords[y][x], color); 
+                m_SubVA.appendPos(m_TileVertices[y][x+1], m_TexCoords[y][x+1], color); 
+                m_SubVA.appendPos(m_TileVertices[y+1][x+1], m_TexCoords[y+1][x+1], color);
+                m_SubVA.appendPos(m_TileVertices[y+1][x], m_TexCoords[y+1][x], color); 
+                m_SubVA.appendQuadIndexes(
+                        curVertex+1, curVertex, curVertex+2, curVertex+3);
+            }
+        }
+    }
 }
 
-void RasterNode::blta8(const DPoint& destSize, double opacity, 
-        const Pixel32& color, GLContext::BlendMode mode)
+void RasterNode::blt32(const glm::mat4& transform, const glm::vec2& destSize, 
+        float opacity, GLContext::BlendMode mode, bool bPremultipliedAlpha)
 {
-    blt(destSize, mode, opacity, color, false);
+    blt(transform, destSize, mode, opacity, Pixel32(255, 255, 255, 255),
+            bPremultipliedAlpha);
+}
+
+void RasterNode::blta8(const glm::mat4& transform, const glm::vec2& destSize, 
+        float opacity, const Pixel32& color, GLContext::BlendMode mode)
+{
+    blt(transform, destSize, mode, opacity, color, false);
 }
 
 GLContext::BlendMode RasterNode::getBlendMode() const
@@ -383,25 +385,17 @@ void RasterNode::setMaskCoords()
 
 void RasterNode::calcMaskCoords()
 {
-    DPoint maskSize;
-    DPoint mediaSize = DPoint(getMediaSize());
-    if (m_MaskSize == DPoint(0,0)) {
-        maskSize = DPoint(1,1);
+    glm::vec2 maskSize;
+    glm::vec2 mediaSize = glm::vec2(getMediaSize());
+    if (m_MaskSize == glm::vec2(0,0)) {
+        maskSize = glm::vec2(1,1);
     } else {
-        maskSize = DPoint(m_MaskSize.x/mediaSize.x, m_MaskSize.y/mediaSize.y);
+        maskSize = glm::vec2(m_MaskSize.x/mediaSize.x, m_MaskSize.y/mediaSize.y);
     }
-    DPoint maskPos = DPoint(m_MaskPos.x/mediaSize.x, m_MaskPos.y/mediaSize.y);
+    glm::vec2 maskPos = glm::vec2(m_MaskPos.x/mediaSize.x, m_MaskPos.y/mediaSize.y);
     m_pSurface->setMaskCoords(maskPos, maskSize);
 }
 
-void RasterNode::checkMaskSupport(const string& sHref)
-{
-    if (!(GLContext::getCurrent()->isUsingShaders()) && sHref != "") {
-        throw Exception(AVG_ERR_UNSUPPORTED,
-                "Can't use masks - unsupported on this hardware/driver combination.");
-    }
-}
-
 void RasterNode::downloadMask()
 {
     GLTexturePtr pTex(new GLTexture(m_pMaskBmp->getSize(), I8, 
@@ -410,64 +404,45 @@ void RasterNode::downloadMask()
     m_pSurface->setMask(pTex);
 }
 
-void RasterNode::bind() 
-{
-    if (!m_bBound) {
-        calcTexCoords();
-    }
-    m_bBound = true;
-}
-
 static ProfilingZoneID FXProfilingZone("RasterNode::renderFX");
 
-void RasterNode::renderFX(const DPoint& destSize, const Pixel32& color, 
-        bool bPremultipliedAlpha)
+void RasterNode::renderFX(const glm::vec2& destSize, const Pixel32& color, 
+        bool bPremultipliedAlpha, bool bForceRender)
 {
-    ScopeTimer Timer(FXProfilingZone);
     setupFX(false);
-    GLContext* pContext = GLContext::getCurrent();
-    pContext->enableGLColorArray(false);
-    pContext->enableTexture(true);
-    if (m_pFXNode && (m_bFXDirty || m_pSurface->isDirty() || m_pFXNode->isDirty())) {
-        if (!m_bBound) {
-            bind();
-        }
+    if (m_pFXNode && (m_bFXDirty || m_pSurface->isDirty() || m_pFXNode->isDirty() ||
+            bForceRender))
+    {
+        ScopeTimer Timer(FXProfilingZone);
+        GLContext* pContext = GLContext::getMain();
+        StandardShader::get()->setAlpha(1.0f);
         m_pSurface->activate(getMediaSize());
 
         m_pFBO->activate();
-        clearGLBuffers(GL_COLOR_BUFFER_BIT);
+        clearGLBuffers(GL_COLOR_BUFFER_BIT, false);
 
-        glColor4d(double(color.getR())/256, double(color.getG())/256, 
-                double(color.getB())/256, 1);
         if (bPremultipliedAlpha) {
             glproc::BlendColor(1.0f, 1.0f, 1.0f, 1.0f);
         }
         pContext->setBlendMode(GLContext::BLEND_BLEND, bPremultipliedAlpha);
-
-        glMatrixMode(GL_MODELVIEW);
-        glPushMatrix();
-
-        m_pImagingProjection->activate();
-        m_pImagingProjection->draw();
+        m_pImagingProjection->setColor(color);
+        m_pImagingProjection->draw(StandardShader::get()->getShader());
 
 /*
         static int i=0;
         stringstream ss;
-        ss << "foo" << i << ".png";
+        ss << "node" << i << ".png";
         BitmapPtr pBmp = m_pFBO->getImage(0);
         pBmp->save(ss.str());
-  */  
+*/    
         m_pFXNode->apply(m_pFBO->getTex());
         
-        glPopMatrix();
-        OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "RasterNode::renderFX(): glPopMatrix");
 /*        
         stringstream ss1;
-        ss1 << "bar" << ".png";
+        ss1 << "nodefx" << i << ".png";
         i++;
         m_pFXNode->getImage()->save(ss1.str());
 */
-        glproc::UseProgramObject(0);
         m_bFXDirty = false;
         m_pSurface->resetDirty();
         m_pFXNode->resetDirty();
@@ -486,83 +461,71 @@ void RasterNode::checkDisplayAvailable(std::string sMsg)
     }
 }
 
+void RasterNode::newSurface()
+{
+    if (m_pSurface->isCreated()) {
+        calcVertexGrid(m_TileVertices);
+        calcTexCoords();
+    }
+}
+
 void RasterNode::setupFX(bool bNewFX)
 {
     if (m_pSurface && m_pSurface->getSize() != IntPoint(-1, -1) && m_pFXNode) {
-        if (!GLContext::getCurrent()->isUsingShaders()) {
-            throw Exception(AVG_ERR_UNSUPPORTED,
-                    "Can't use FX - unsupported on this hardware/driver combination.");
-        }
         if (bNewFX || !m_pFBO || m_pFBO->getSize() != m_pSurface->getSize()) {
             m_pFXNode->setSize(m_pSurface->getSize());
             m_pFXNode->connect();
             m_bFXDirty = true;
         }
         if (!m_pFBO || m_pFBO->getSize() != m_pSurface->getSize()) {
-            m_pFBO = FBOPtr(new FBO(IntPoint(m_pSurface->getSize()), B8G8R8A8, 1, 1,
-                    false, getMipmap()));
+            PixelFormat pf = BitmapLoader::get()->getDefaultPixelFormat(true);
+            m_pFBO = FBOPtr(new FBO(IntPoint(m_pSurface->getSize()), pf, 1, 1, false, 
+                    getMipmap()));
             GLTexturePtr pTex = m_pFBO->getTex();
+            #ifndef AVG_ENABLE_EGL
             pTex->setWrapMode(GL_CLAMP_TO_BORDER, GL_CLAMP_TO_BORDER);
+            #endif
             m_pImagingProjection = ImagingProjectionPtr(new ImagingProjection(
                     m_pSurface->getSize()));
         }
     }
 }
 
-void RasterNode::blt(const DPoint& destSize, GLContext::BlendMode mode,
-        double opacity, const Pixel32& color, bool bPremultipliedAlpha)
+void RasterNode::blt(const glm::mat4& transform, const glm::vec2& destSize, 
+        GLContext::BlendMode mode, float opacity, const Pixel32& color,
+        bool bPremultipliedAlpha)
 {
-    if (!m_bBound) {
-        bind();
-    }
-    GLContext* pContext = GLContext::getCurrent();
-    pContext->enableGLColorArray(false);
-    pContext->enableTexture(true);
-    DRect destRect;
+    GLContext* pContext = GLContext::getMain();
+    FRect destRect;
+    
+    StandardShaderPtr pShader = pContext->getStandardShader();
+    pContext->setBlendColor(glm::vec4(1.0f, 1.0f, 1.0f, opacity));
+    pShader->setAlpha(opacity);
     if (m_pFXNode) {
-        m_pFXNode->getTex()->activate(GL_TEXTURE0);
-
         pContext->setBlendMode(mode, true);
-        glColor4d(1.0, 1.0, 1.0, opacity);
-        DRect relDestRect = m_pFXNode->getRelDestRect();
-        destRect = DRect(relDestRect.tl.x*destSize.x, relDestRect.tl.y*destSize.y,
+        m_pFXNode->getTex()->activate(GL_TEXTURE0);
+        pShader->setColorModel(0);
+        pShader->disableColorspaceMatrix();
+        pShader->setGamma(glm::vec4(1.0f, 1.0f, 1.0f, 1.0f));
+        pShader->setPremultipliedAlpha(true);
+        pShader->setMask(false);
+
+        FRect relDestRect = m_pFXNode->getRelDestRect();
+        destRect = FRect(relDestRect.tl.x*destSize.x, relDestRect.tl.y*destSize.y,
                 relDestRect.br.x*destSize.x, relDestRect.br.y*destSize.y);
     } else {
         m_pSurface->activate(getMediaSize(), bPremultipliedAlpha);
         pContext->setBlendMode(mode, bPremultipliedAlpha);
-        glColor4d(double(color.getR())/256, double(color.getG())/256, 
-                double(color.getB())/256, opacity);
-        destRect = DRect(DPoint(0,0), destSize);
+        destRect = FRect(glm::vec2(0,0), destSize);
     }
-    glproc::BlendColor(1.0f, 1.0f, 1.0f, float(opacity));
-    glPushMatrix();
-    glTranslated(destRect.tl.x, destRect.tl.y, 1);
-    glScaled(destRect.size().x, destRect.size().y, 1);
+    glm::vec3 pos(destRect.tl.x, destRect.tl.y, 0);
+    glm::vec3 scaleVec(destRect.size().x, destRect.size().y, 1);
+    glm::mat4 localTransform = glm::translate(transform, pos);
+    localTransform = glm::scale(localTransform, scaleVec);
+    pShader->setTransform(localTransform);
+    pShader->activate();
 
-    if (m_bVertexArrayDirty) {
-        m_pVertexes->reset();
-        for (unsigned y = 0; y < m_TileVertices.size()-1; y++) {
-            for (unsigned x = 0; x < m_TileVertices[0].size()-1; x++) {
-                int curVertex = m_pVertexes->getCurVert();
-                m_pVertexes->appendPos(m_TileVertices[y][x], m_TexCoords[y][x]); 
-                m_pVertexes->appendPos(m_TileVertices[y][x+1], m_TexCoords[y][x+1]); 
-                m_pVertexes->appendPos(m_TileVertices[y+1][x+1], m_TexCoords[y+1][x+1]); 
-                m_pVertexes->appendPos(m_TileVertices[y+1][x], m_TexCoords[y+1][x]); 
-                m_pVertexes->appendQuadIndexes(
-                        curVertex+1, curVertex, curVertex+2, curVertex+3);
-            }
-        }
-        m_bVertexArrayDirty = false;
-    }
-
-    m_pVertexes->draw();
-
-    glPopMatrix();
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "RasterNode::blt(): glPopMatrix 2");
-
-    PixelFormat pf = m_pSurface->getPixelFormat();
-    AVG_TRACE(Logger::BLTS, "(" << destSize.x << ", " << destSize.y << ")" 
-            << ", m_pf: " << pf);
+    m_SubVA.draw();
 }
 
 IntPoint RasterNode::getNumTiles()
@@ -571,38 +534,33 @@ IntPoint RasterNode::getNumTiles()
     if (m_TileSize.x == -1) {
         return IntPoint(1,1);
     } else {
-        return IntPoint(safeCeil(double(size.x)/m_TileSize.x),
-                safeCeil(double(size.y)/m_TileSize.y));
+        return IntPoint(safeCeil(float(size.x)/m_TileSize.x),
+                safeCeil(float(size.y)/m_TileSize.y));
     }
 }
 
 void RasterNode::calcVertexGrid(VertexGrid& grid)
 {
     IntPoint numTiles = getNumTiles();
-    std::vector<DPoint> TileVerticesLine(numTiles.x+1);
-    grid = std::vector<std::vector<DPoint> > (numTiles.y+1, TileVerticesLine);
+    std::vector<glm::vec2> TileVerticesLine(numTiles.x+1);
+    grid = std::vector<std::vector<glm::vec2> > (numTiles.y+1, TileVerticesLine);
     for (unsigned y = 0; y < grid.size(); y++) {
         for (unsigned x = 0; x < grid[y].size(); x++) {
             calcTileVertex(x, y, grid[y][x]);
         }
     }
-    if (m_pVertexes) {
-        delete m_pVertexes;
-    }
-    m_bVertexArrayDirty = true;
-    m_pVertexes = new VertexArray(numTiles.x*numTiles.y*4, numTiles.x*numTiles.y*6);
 }
 
-void RasterNode::calcTileVertex(int x, int y, DPoint& Vertex) 
+void RasterNode::calcTileVertex(int x, int y, glm::vec2& Vertex) 
 {
     IntPoint numTiles = getNumTiles();
     if (x < numTiles.x) {
-        Vertex.x = double(m_TileSize.x*x) / m_pSurface->getSize().x;
+        Vertex.x = float(m_TileSize.x*x) / m_pSurface->getSize().x;
     } else {
         Vertex.x = 1;
     }
     if (y < numTiles.y) {
-        Vertex.y = double(m_TileSize.y*y) / m_pSurface->getSize().y;
+        Vertex.y = float(m_TileSize.y*y) / m_pSurface->getSize().y;
     } else {
         Vertex.y = 1;
     }
@@ -610,22 +568,22 @@ void RasterNode::calcTileVertex(int x, int y, DPoint& Vertex)
 
 void RasterNode::calcTexCoords()
 {
-    DPoint textureSize = DPoint(m_pSurface->getTextureSize());
-    DPoint imageSize = DPoint(m_pSurface->getSize());
-    DPoint texCoordExtents = DPoint(imageSize.x/textureSize.x,
+    glm::vec2 textureSize = glm::vec2(m_pSurface->getTextureSize());
+    glm::vec2 imageSize = glm::vec2(m_pSurface->getSize());
+    glm::vec2 texCoordExtents = glm::vec2(imageSize.x/textureSize.x,
             imageSize.y/textureSize.y);
 
-    DPoint texSizePerTile;
+    glm::vec2 texSizePerTile;
     if (m_TileSize.x == -1) {
         texSizePerTile = texCoordExtents;
     } else {
-        texSizePerTile = DPoint(double(m_TileSize.x)/imageSize.x*texCoordExtents.x,
-                double(m_TileSize.y)/imageSize.y*texCoordExtents.y);
+        texSizePerTile = glm::vec2(float(m_TileSize.x)/imageSize.x*texCoordExtents.x,
+                float(m_TileSize.y)/imageSize.y*texCoordExtents.y);
     }
 
     IntPoint numTiles = getNumTiles();
-    vector<DPoint> texCoordLine(numTiles.x+1);
-    m_TexCoords = std::vector<std::vector<DPoint> > 
+    vector<glm::vec2> texCoordLine(numTiles.x+1);
+    m_TexCoords = std::vector<std::vector<glm::vec2> > 
             (numTiles.y+1, texCoordLine);
     for (unsigned y = 0; y < m_TexCoords.size(); y++) {
         for (unsigned x = 0; x < m_TexCoords[y].size(); x++) {
@@ -641,6 +599,5 @@ void RasterNode::calcTexCoords()
             }
         }
     }
-    m_bVertexArrayDirty = true;
 }
 }
