@@ -1,6 +1,6 @@
 
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -45,20 +45,19 @@
 #include "../graphics/GLContext.h"
 #include "../graphics/Filterflip.h"
 #include "../graphics/Filterfliprgb.h"
-#include "../graphics/ShaderRegistry.h"
+#include "../graphics/Display.h"
+
+#include "../video/VideoDecoder.h"
 
 #include "OGLSurface.h"
 #include "OffscreenCanvas.h"
 
-#include <SDL/SDL.h>
-
 #ifdef __APPLE__
 #include <ApplicationServices/ApplicationServices.h>
 #endif
-#ifdef linux
+
+#include <SDL/SDL.h>
 #include <SDL/SDL_syswm.h>
-#include <X11/extensions/xf86vmode.h>
-#endif
 
 #ifdef __APPLE__
 #include <OpenGL/OpenGL.h>
@@ -68,6 +67,9 @@
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
 #endif
+#ifdef AVG_ENABLE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif
 
 #include <signal.h>
 #include <iostream>
@@ -84,27 +86,7 @@ using namespace std;
 
 namespace avg {
 
-double SDLDisplayEngine::s_RefreshRate = 0.0;
-
-void safeSetAttribute(SDL_GLattr attr, int value) 
-{
-    int err = SDL_GL_SetAttribute(attr, value);
-    if (err == -1) {
-        throw Exception(AVG_ERR_VIDEO_GENERAL, SDL_GetError());
-    }
-}
-
-SDLDisplayEngine::SDLDisplayEngine()
-    : IInputDevice(EXTRACT_INPUTDEVICE_CLASSNAME(SDLDisplayEngine)),
-      m_WindowSize(0,0),
-      m_PPMM(0),
-      m_pScreen(0),
-      m_VBMethod(VB_NONE),
-      m_VBMod(0),
-      m_bMouseOverApp(true),
-      m_pLastMouseEvent(new MouseEvent(Event::CURSORMOTION, false, false, false, 
-            IntPoint(-1, -1), MouseEvent::NO_BUTTON, DPoint(-1, -1), 0)),
-      m_NumMouseButtonsDown(0)
+void SDLDisplayEngine::initSDL()
 {
 #ifdef __APPLE__
     static bool bSDLInitialized = false;
@@ -113,10 +95,34 @@ SDLDisplayEngine::SDLDisplayEngine()
         bSDLInitialized = true;
     }
 #endif
-    if (SDL_InitSubSystem(SDL_INIT_VIDEO)==-1) {
-        AVG_TRACE(Logger::ERROR, "Can't init SDL display subsystem.");
-        exit(-1);
+#ifdef linux
+    // Disable all other video drivers (DirectFB, libcaca, ...) to avoid confusing
+    // error messages.
+    SDL_putenv((char*)"SDL_VIDEODRIVER=x11");
+#endif
+    int err = SDL_InitSubSystem(SDL_INIT_VIDEO);
+    if (err == -1) {
+        throw Exception(AVG_ERR_VIDEO_INIT_FAILED, SDL_GetError());
     }
+}
+
+void SDLDisplayEngine::quitSDL()
+{
+#ifndef _WIN32
+    SDL_QuitSubSystem(SDL_INIT_VIDEO);
+#endif
+}
+
+SDLDisplayEngine::SDLDisplayEngine()
+    : IInputDevice(EXTRACT_INPUTDEVICE_CLASSNAME(SDLDisplayEngine)),
+      m_WindowSize(0,0),
+      m_pScreen(0),
+      m_pLastMouseEvent(new MouseEvent(Event::CURSOR_MOTION, false, false, false, 
+            IntPoint(-1, -1), MouseEvent::NO_BUTTON, glm::vec2(-1, -1), 0)),
+      m_pGLContext(0)
+{
+    initSDL();
+
     m_Gamma[0] = 1.0;
     m_Gamma[1] = 1.0;
     m_Gamma[2] = 1.0;
@@ -125,68 +131,26 @@ SDLDisplayEngine::SDLDisplayEngine()
 
 SDLDisplayEngine::~SDLDisplayEngine()
 {
-#ifndef _WIN32
-    SDL_QuitSubSystem(SDL_INIT_VIDEO);
-#endif
 }
 
 void SDLDisplayEngine::init(const DisplayParams& dp, GLConfig glConfig) 
 {
-    calcScreenDimensions(dp.m_DotsPerMM);
+    // This "fixes" the default behaviour of SDL under x11, avoiding it
+    // to report relative mouse coordinates when going fullscreen and
+    // the mouse cursor is hidden (grabbed). So far libavg and apps based
+    // on it don't use relative coordinates.
+    setEnv("SDL_MOUSE_RELATIVE", "0");
+
+    if (m_Gamma[0] != 1.0f || m_Gamma[1] != 1.0f || m_Gamma[2] != 1.0f) {
+        internalSetGamma(1.0f, 1.0f, 1.0f);
+    }
     stringstream ss;
     if (dp.m_Pos.x != -1) {
         ss << dp.m_Pos.x << "," << dp.m_Pos.y;
         setEnv("SDL_VIDEO_WINDOW_POS", ss.str().c_str());
     }
-#ifdef linux
-    IntPoint oldWindowSize = m_WindowSize;
-#endif
-    double aspectRatio = double(dp.m_Size.x)/double(dp.m_Size.y);
-    if (dp.m_WindowSize == IntPoint(0, 0)) {
-        m_WindowSize = dp.m_Size;
-    } else if (dp.m_WindowSize.x == 0) {
-        m_WindowSize.x = int(dp.m_WindowSize.y*aspectRatio);
-        m_WindowSize.y = dp.m_WindowSize.y;
-    } else {
-        m_WindowSize.x = dp.m_WindowSize.x;
-        m_WindowSize.y = int(dp.m_WindowSize.x/aspectRatio);
-    }
-
-    switch (dp.m_BPP) {
-        case 32:
-            safeSetAttribute(SDL_GL_RED_SIZE, 8);
-            safeSetAttribute(SDL_GL_GREEN_SIZE, 8);
-            safeSetAttribute(SDL_GL_BLUE_SIZE, 8);
-            safeSetAttribute(SDL_GL_BUFFER_SIZE, 32);
-            break;
-        case 24:
-            safeSetAttribute(SDL_GL_RED_SIZE, 8);
-            safeSetAttribute(SDL_GL_GREEN_SIZE, 8);
-            safeSetAttribute(SDL_GL_BLUE_SIZE, 8);
-            safeSetAttribute(SDL_GL_BUFFER_SIZE, 24);
-            break;
-        case 16:
-            safeSetAttribute(SDL_GL_RED_SIZE, 5);
-            safeSetAttribute(SDL_GL_GREEN_SIZE, 6);
-            safeSetAttribute(SDL_GL_BLUE_SIZE, 5);
-            safeSetAttribute(SDL_GL_BUFFER_SIZE, 16);
-            break;
-        case 15:
-            safeSetAttribute(SDL_GL_RED_SIZE, 5);
-            safeSetAttribute(SDL_GL_GREEN_SIZE, 5);
-            safeSetAttribute(SDL_GL_BLUE_SIZE, 5);
-            safeSetAttribute(SDL_GL_BUFFER_SIZE, 15);
-            break;
-        default:
-            AVG_TRACE(Logger::ERROR, "Unsupported bpp " << dp.m_BPP <<
-                    "in SDLDisplayEngine::init()");
-            exit(-1);
-    }
-    safeSetAttribute(SDL_GL_DEPTH_SIZE, 24);
-    safeSetAttribute(SDL_GL_STENCIL_SIZE, 8);
-    safeSetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-
-    unsigned int Flags = SDL_OPENGL;
+    m_WindowSize = dp.m_WindowSize;
+    unsigned int Flags = 0;
     if (dp.m_bFullscreen) {
         Flags |= SDL_FULLSCREEN;
     }
@@ -196,73 +160,73 @@ void SDLDisplayEngine::init(const DisplayParams& dp, GLConfig glConfig)
         Flags |= SDL_NOFRAME;
     }
 
-    bool bAllMultisampleValuesTested = false;
+#ifndef linux
+    if (glConfig.m_bUseDebugContext) {
+        glConfig.m_bUseDebugContext = false;
+    }
+    switch (dp.m_BPP) {
+        case 24:
+            SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
+            SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
+            SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
+            SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, 24);
+            break;
+        case 16:
+            SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
+            SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 6);
+            SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
+            SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, 16);
+            break;
+        default:
+            AVG_LOG_ERROR("Unsupported bpp " << dp.m_BPP <<
+                    "in SDLDisplayEngine::init()");
+            exit(-1);
+    }
+    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 0);
+    SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);
+    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
+    SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL , 0); 
+    Flags |= SDL_OPENGL;
+
     m_pScreen = 0;
-    while (!bAllMultisampleValuesTested && !m_pScreen) {
+    while (glConfig.m_MultiSampleSamples && !m_pScreen) {
         if (glConfig.m_MultiSampleSamples > 1) {
-            safeSetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
-            safeSetAttribute(SDL_GL_MULTISAMPLESAMPLES, glConfig.m_MultiSampleSamples);
+            SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
+            SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES,
+                    glConfig.m_MultiSampleSamples);
         } else {
-            safeSetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 0);
-            safeSetAttribute(SDL_GL_MULTISAMPLESAMPLES, 0);
+            SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 0);
+            SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 0);
         }
         m_pScreen = SDL_SetVideoMode(m_WindowSize.x, m_WindowSize.y, dp.m_BPP, Flags);
         if (!m_pScreen) {
-            switch (glConfig.m_MultiSampleSamples) {
-                case 1:
-                    bAllMultisampleValuesTested = true;
-                    break;
-                case 2:  
-                    glConfig.m_MultiSampleSamples = 1;
-                    break;
-                case 4:  
-                    glConfig.m_MultiSampleSamples = 2;
-                    break;
-                case 8:  
-                    glConfig.m_MultiSampleSamples = 4;
-                    break;
-                default:
-                    glConfig.m_MultiSampleSamples = 8;
-                    break;
-            }
+            glConfig.m_MultiSampleSamples = GLContext::nextMultiSampleValue(
+                    glConfig.m_MultiSampleSamples);
         }
     }
+#else
+    // Linux version: Context created manually, not by SDL
+    m_pScreen = SDL_SetVideoMode(m_WindowSize.x, m_WindowSize.y, dp.m_BPP, Flags);
+#endif
     if (!m_pScreen) {
         throw Exception(AVG_ERR_UNSUPPORTED, string("Setting SDL video mode failed: ")
                 + SDL_GetError() + ". (size=" + toString(m_WindowSize) + ", bpp=" + 
-                toString(dp.m_BPP) + ", multisamplesamples=" + 
-                toString(glConfig.m_MultiSampleSamples) + ").");
+                toString(dp.m_BPP) + ").");
     }
-    m_pGLContext = GLContextPtr(new GLContext(true, glConfig));
+    SDL_SysWMinfo info;
+    SDL_VERSION(&info.version);
+    int rc = SDL_GetWMInfo(&info);
+    AVG_ASSERT(rc != -1);
+    m_pGLContext = GLContext::create(glConfig, m_WindowSize, &info);
+    GLContext::setMain(m_pGLContext);
 
 #if defined(HAVE_XI2_1) || defined(HAVE_XI2_2) 
     SDL_EventState(SDL_SYSWMEVENT, SDL_ENABLE);
     m_pXIMTInputDevice = 0;
 #endif
     SDL_WM_SetCaption("libavg", 0);
-    calcRefreshRate();
+    Display::get()->getRefreshRate();
 
-    glEnable(GL_BLEND);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_BLEND)");
-    glShadeModel(GL_FLAT);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glShadeModel(GL_FLAT)");
-    glDisable(GL_DEPTH_TEST);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_DEPTH_TEST)");
-    glEnable(GL_STENCIL_TEST);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glEnable(GL_STENCIL_TEST)");
-    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glTexEnvf()");
-    glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
-    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-    glPixelStorei(GL_PACK_ROW_LENGTH, 0);
-    glPixelStorei(GL_PACK_ALIGNMENT, 1);
-    if (!queryOGLExtension("GL_ARB_vertex_buffer_object")) {
-        throw Exception(AVG_ERR_UNSUPPORTED,
-            "Graphics driver lacks vertex buffer support, unable to initialize graphics.");
-    }
-
-    glEnableClientState(GL_VERTEX_ARRAY);
-    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     setGamma(dp.m_Gamma[0], dp.m_Gamma[1], dp.m_Gamma[2]);
     showCursor(dp.m_bShowCursor);
     if (dp.m_Framerate == 0) {
@@ -275,11 +239,31 @@ void SDLDisplayEngine::init(const DisplayParams& dp, GLConfig glConfig)
     // SDL sets up a signal handler we really don't want.
     signal(SIGSEGV, SIG_DFL);
     m_pGLContext->logConfig();
+    VideoDecoder::logConfig();
 
     SDL_EnableUNICODE(1);
-    if (m_pGLContext->isUsingShaders()) {
-        OGLSurface::createShader();
+}
+
+IntPoint SDLDisplayEngine::calcWindowSize(const DisplayParams& dp) const
+{
+    float aspectRatio = float(dp.m_Size.x)/float(dp.m_Size.y);
+    IntPoint windowSize;
+    if (dp.m_WindowSize == IntPoint(0, 0)) {
+        windowSize = dp.m_Size;
+    } else if (dp.m_WindowSize.x == 0) {
+        windowSize.x = int(dp.m_WindowSize.y*aspectRatio);
+        windowSize.y = dp.m_WindowSize.y;
+    } else {
+        windowSize.x = dp.m_WindowSize.x;
+        windowSize.y = int(dp.m_WindowSize.x/aspectRatio);
     }
+    AVG_ASSERT(windowSize.x != 0 && windowSize.y != 0);
+    return windowSize;
+}
+        
+void SDLDisplayEngine::setWindowTitle(const string& sTitle)
+{
+    SDL_WM_SetCaption(sTitle.c_str(), 0);
 }
 
 #ifdef _WIN32
@@ -288,37 +272,28 @@ void SDLDisplayEngine::init(const DisplayParams& dp, GLConfig glConfig)
 void SDLDisplayEngine::teardown()
 {
     if (m_pScreen) {
-        if (m_Gamma[0] != 1.0 || m_Gamma[1] != 1.0 || m_Gamma[2] != 1.0) {
-            SDL_SetGamma(1.0, 1.0, 1.0);
-        }
 #ifdef linux
         // Workaround for broken mouse cursor on exit under Ubuntu 8.04.
         SDL_ShowCursor(SDL_ENABLE);
-//        SDL_SetVideoMode(m_WindowWidth, m_WindowHeight, 24, 0);
 #endif
         m_pScreen = 0;
-        m_pGLContext = GLContextPtr();
+        if (m_pGLContext) {
+            delete m_pGLContext;
+            m_pGLContext = 0;
+        }
+        GLContext::setMain(0);
     }
 }
 
-double SDLDisplayEngine::getRefreshRate() 
-{
-    if (s_RefreshRate == 0.0) {
-        calcRefreshRate();
-    }
-    return s_RefreshRate;
-}
-
-void SDLDisplayEngine::setGamma(double red, double green, double blue)
+void SDLDisplayEngine::setGamma(float red, float green, float blue)
 {
     if (red > 0) {
-        AVG_TRACE(Logger::CONFIG, "Setting gamma to " << red << ", " << green << ", " << blue);
-        int err = SDL_SetGamma(float(red), float(green), float(blue));
+        bool bOk = internalSetGamma(red, green, blue);
         m_Gamma[0] = red;
         m_Gamma[1] = green;
         m_Gamma[2] = blue;
-        if (err == -1) {
-            AVG_TRACE(Logger::WARNING, "Unable to set display gamma.");
+        if (!bOk) {
+            AVG_LOG_WARNING("Unable to set display gamma.");
         }
     }
 }
@@ -333,32 +308,17 @@ int SDLDisplayEngine::getKeyModifierState() const
     return SDL_GetModState();
 }
 
-void SDLDisplayEngine::calcScreenDimensions(double dotsPerMM)
+bool SDLDisplayEngine::internalSetGamma(float red, float green, float blue)
 {
-    if (dotsPerMM != 0) {
-        const SDL_VideoInfo* pInfo = SDL_GetVideoInfo();
-        m_ScreenResolution = IntPoint(pInfo->current_w, pInfo->current_h);
-        m_PPMM = dotsPerMM;
-    }
-
-    if (m_PPMM == 0) {
-        const SDL_VideoInfo* pInfo = SDL_GetVideoInfo();
-        m_ScreenResolution = IntPoint(pInfo->current_w, pInfo->current_h);
-#ifdef WIN32
-        HDC hdc = CreateDC("DISPLAY", NULL, NULL, NULL);
-        m_PPMM = GetDeviceCaps(hdc, LOGPIXELSX)/25.4;
+#ifdef __APPLE__
+    // Workaround for broken SDL_SetGamma for libSDL 1.2.15 under Lion
+    CGError err = CGSetDisplayTransferByFormula(kCGDirectMainDisplay, 0, 1, 1/red,
+            0, 1, 1/green, 0, 1, 1/blue);
+    return (err == CGDisplayNoErr);
 #else
-    #ifdef linux
-        Display * pDisplay = XOpenDisplay(0);
-        DPoint displayMM(DisplayWidthMM(pDisplay,0), DisplayHeightMM(pDisplay,0));
-    #elif defined __APPLE__
-        CGSize size = CGDisplayScreenSize(CGMainDisplayID());
-        DPoint displayMM(size.width, size.height);
-    #endif
-        // Non-Square pixels cause errors here. We'll fix that when it happens.
-        m_PPMM = m_ScreenResolution.x/displayMM.x;
+    int err = SDL_SetGamma(float(red), float(green), float(blue));
+    return (err != -1);
 #endif
-    }
 }
 
 static ProfilingZoneID SwapBufferProfilingZone("Render - swap buffers");
@@ -366,9 +326,12 @@ static ProfilingZoneID SwapBufferProfilingZone("Render - swap buffers");
 void SDLDisplayEngine::swapBuffers()
 {
     ScopeTimer timer(SwapBufferProfilingZone);
+#ifdef linux    
+    m_pGLContext->swapBuffers();
+#else
     SDL_GL_SwapBuffers();
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "swapBuffers()");
-    AVG_TRACE(Logger::BLTS, "GL SwapBuffers");
+#endif
+    GLContext::checkError("swapBuffers()");
 }
 
 void SDLDisplayEngine::showCursor(bool bShow)
@@ -391,24 +354,35 @@ void SDLDisplayEngine::showCursor(bool bShow)
 
 BitmapPtr SDLDisplayEngine::screenshot(int buffer)
 {
-    BitmapPtr pBmp (new Bitmap(m_WindowSize, B8G8R8X8, "screenshot"));
-    string sTmp;
-    bool bBroken = getEnv("AVG_BROKEN_READBUFFER", sTmp);
-    GLenum buf = buffer;
-    if (!buffer) {
-        if (bBroken) {
-            // Workaround for buggy GL_FRONT on some machines.
-            buf = GL_BACK;
-        } else {
-            buf = GL_FRONT;
+    BitmapPtr pBmp;
+    glproc::BindFramebuffer(GL_FRAMEBUFFER, 0);
+    if (m_pGLContext->isGLES()) {
+        pBmp = BitmapPtr(new Bitmap(m_WindowSize, R8G8B8X8, "screenshot"));
+        glReadPixels(0, 0, m_WindowSize.x, m_WindowSize.y, GL_RGBA, GL_UNSIGNED_BYTE, 
+                pBmp->getPixels());
+        GLContext::checkError("SDLDisplayEngine::screenshot:glReadPixels()");
+    } else {
+#ifndef AVG_ENABLE_EGL
+        pBmp = BitmapPtr(new Bitmap(m_WindowSize, B8G8R8X8, "screenshot"));
+        string sTmp;
+        bool bBroken = getEnv("AVG_BROKEN_READBUFFER", sTmp);
+        GLenum buf = buffer;
+        if (!buffer) {
+            if (bBroken) {
+                // Workaround for buggy GL_FRONT on some machines.
+                buf = GL_BACK;
+            } else {
+                buf = GL_FRONT;
+            }
         }
+        glReadBuffer(buf);
+        GLContext::checkError("SDLDisplayEngine::screenshot:glReadBuffer()");
+        glproc::BindBuffer(GL_PIXEL_PACK_BUFFER, 0);
+        glReadPixels(0, 0, m_WindowSize.x, m_WindowSize.y, GL_BGRA, GL_UNSIGNED_BYTE, 
+                pBmp->getPixels());
+        GLContext::checkError("SDLDisplayEngine::screenshot:glReadPixels()");
+#endif
     }
-    glReadBuffer(buf);
-    glproc::BindBuffer(GL_PIXEL_PACK_BUFFER_EXT, 0);
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "SDLDisplayEngine::screenshot:glReadBuffer()");
-    glReadPixels(0, 0, m_WindowSize.x, m_WindowSize.y, GL_BGRA, GL_UNSIGNED_BYTE, 
-            pBmp->getPixels());
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "SDLDisplayEngine::screenshot:glReadPixels()");
     FilterFlip().applyInPlace(pBmp);
     return pBmp;
 }
@@ -418,172 +392,6 @@ IntPoint SDLDisplayEngine::getSize()
     return m_Size;
 }
 
-void SDLDisplayEngine::initMacVBlank(int rate)
-{
-#ifdef __APPLE__
-    CGLContextObj context = CGLGetCurrentContext();
-    AVG_ASSERT (context);
-#if MAC_OS_X_VERSION_10_5
-    GLint l = rate;
-#else
-    long l = rate;
-#endif
-    if (rate > 1) {
-        AVG_TRACE(Logger::WARNING,
-                "VBlank rate set to " << rate 
-                << " but Mac OS X only supports 1. Assuming 1.");
-        l = 1;
-    }
-    CGLError err = CGLSetParameter(context, kCGLCPSwapInterval, &l);
-    AVG_ASSERT(!err);
-#endif
-}
-
-bool SDLDisplayEngine::initVBlank(int rate) 
-{
-    if (rate > 0) {
-#ifdef __APPLE__
-        initMacVBlank(rate);
-        m_VBMethod = VB_APPLE;
-#elif defined _WIN32
-        if (queryOGLExtension("WGL_EXT_swap_control")) {
-            glproc::SwapIntervalEXT(rate);
-            m_VBMethod = VB_WIN;
-        } else {
-            AVG_TRACE(Logger::WARNING,
-                    "Windows VBlank setup failed: OpenGL Extension not supported.");
-            m_VBMethod = VB_NONE;
-        }
-#else
-        if (getenv("__GL_SYNC_TO_VBLANK") != 0) {
-            AVG_TRACE(Logger::WARNING, 
-                    "__GL_SYNC_TO_VBLANK set. This interferes with libavg vblank handling.");
-            m_VBMethod = VB_NONE;
-        } else {
-            if (queryGLXExtension("GLX_SGI_swap_control")) {
-                m_VBMethod = VB_SGI;
-                glproc::SwapIntervalSGI(rate);
-
-            } else {
-                AVG_TRACE(Logger::WARNING,
-                        "Linux VBlank setup failed: OpenGL Extension not supported.");
-                m_VBMethod = VB_NONE;
-            }
-        }
-#endif
-    } else {
-        switch (m_VBMethod) {
-            case VB_APPLE:
-                initMacVBlank(0);
-                break;
-            case VB_WIN:
-#ifdef _WIN32
-                glproc::SwapIntervalEXT(0);
-#endif
-                break;
-            case VB_SGI:
-#ifdef linux            
-                if (queryGLXExtension("GLX_SGI_swap_control")) {
-                    glproc::SwapIntervalSGI(rate);
-                }
-#endif
-                break;
-            default:
-                break;
-        }
-        m_VBMethod = VB_NONE;
-    }
-    switch(m_VBMethod) {
-        case VB_SGI:
-            AVG_TRACE(Logger::CONFIG, 
-                    "  Using SGI OpenGL extension for vertical blank support.");
-            break;
-        case VB_APPLE:
-            AVG_TRACE(Logger::CONFIG, "  Using Apple GL vertical blank support.");
-            break;
-        case VB_WIN:
-            AVG_TRACE(Logger::CONFIG, "  Using Windows GL vertical blank support.");
-            break;
-        case VB_NONE:
-            AVG_TRACE(Logger::CONFIG, "  Vertical blank support disabled.");
-            break;
-        default:
-            AVG_TRACE(Logger::WARNING, "  Illegal vblank enum value.");
-    }
-    return m_VBMethod != VB_NONE;
-}
-
-bool SDLDisplayEngine::vbWait(int rate)
-{
-    switch(m_VBMethod) {
-        case VB_SGI:
-        case VB_APPLE:
-        case VB_WIN:
-            return true;
-        case VB_NONE:
-        default:
-            AVG_ASSERT(false);
-            return false;
-    }
-}
-
-void SDLDisplayEngine::calcRefreshRate()
-{
-    double lastRefreshRate = s_RefreshRate;
-    s_RefreshRate = 0;
-#ifdef __APPLE__
-    CFDictionaryRef modeInfo = CGDisplayCurrentMode(CGMainDisplayID());
-    if (modeInfo) {
-        CFNumberRef value = (CFNumberRef) CFDictionaryGetValue(modeInfo, 
-                kCGDisplayRefreshRate);
-        if (value) {
-            CFNumberGetValue(value, kCFNumberIntType, &s_RefreshRate);
-            if (s_RefreshRate < 1.0) {
-                AVG_TRACE(Logger::CONFIG, 
-                        "This seems to be a TFT screen, assuming 60 Hz refresh rate.");
-                s_RefreshRate = 60;
-            }
-        } else {
-            AVG_TRACE(Logger::WARNING, 
-                    "Apple refresh rate calculation (CFDictionaryGetValue) failed");
-        }
-    } else {
-        AVG_TRACE(Logger::WARNING, 
-                "Apple refresh rate calculation (CGDisplayCurrentMode) failed");
-    }
-#elif defined _WIN32
-    // This isn't correct for multi-monitor systems.
-    HDC hDC = CreateDC("DISPLAY", NULL, NULL, NULL);
-    s_RefreshRate = GetDeviceCaps(hDC, VREFRESH);
-    if (s_RefreshRate < 2) {
-        s_RefreshRate = 60;
-    }
-    DeleteDC(hDC);
-#else 
-    Display * pDisplay = XOpenDisplay(0);
-    int pixelClock;
-    XF86VidModeModeLine modeLine;
-    bool bOK = XF86VidModeGetModeLine (pDisplay, DefaultScreen(pDisplay), 
-            &pixelClock, &modeLine);
-    if (!bOK) {
-        AVG_TRACE (Logger::WARNING, 
-                "Could not get current refresh rate (XF86VidModeGetModeLine failed).");
-        AVG_TRACE (Logger::WARNING, 
-                "Defaulting to 60 Hz refresh rate.");
-    }
-    double HSyncRate = pixelClock*1000.0/modeLine.htotal;
-    s_RefreshRate = HSyncRate/modeLine.vtotal;
-    XCloseDisplay(pDisplay);
-#endif
-    if (s_RefreshRate == 0 || isnan(s_RefreshRate)) {
-        s_RefreshRate = 60;
-    }
-    if (lastRefreshRate != s_RefreshRate) {
-        AVG_TRACE(Logger::CONFIG, "Vertical Refresh Rate: " << s_RefreshRate);
-    }
-
-}
-
 vector<long> SDLDisplayEngine::KeyCodeTranslationTable(SDLK_LAST, key::KEY_UNKNOWN);
 
 const char * getEventTypeName(unsigned char type) 
@@ -627,12 +435,14 @@ vector<EventPtr> SDLDisplayEngine::pollEvents()
     SDL_Event sdlEvent;
     vector<EventPtr> events;
 
+    int numEvents = 0;
     while (SDL_PollEvent(&sdlEvent)) {
+        numEvents++;
         EventPtr pNewEvent;
         switch (sdlEvent.type) {
             case SDL_MOUSEMOTION:
-                if (m_bMouseOverApp) {
-                    pNewEvent = createMouseEvent(Event::CURSORMOTION, sdlEvent, 
+                {
+                    pNewEvent = createMouseEvent(Event::CURSOR_MOTION, sdlEvent, 
                             MouseEvent::NO_BUTTON);
                     CursorEventPtr pNewCursorEvent = 
                             boost::dynamic_pointer_cast<CursorEvent>(pNewEvent);
@@ -646,10 +456,10 @@ vector<EventPtr> SDLDisplayEngine::pollEvents()
                 }
                 break;
             case SDL_MOUSEBUTTONDOWN:
-                pNewEvent = createMouseButtonEvent(Event::CURSORDOWN, sdlEvent);
+                pNewEvent = createMouseButtonEvent(Event::CURSOR_DOWN, sdlEvent);
                 break;
             case SDL_MOUSEBUTTONUP:
-                pNewEvent = createMouseButtonEvent(Event::CURSORUP, sdlEvent);
+                pNewEvent = createMouseButtonEvent(Event::CURSOR_UP, sdlEvent);
                 break;
             case SDL_JOYAXISMOTION:
 //                pNewEvent = createAxisEvent(sdlEvent));
@@ -661,10 +471,10 @@ vector<EventPtr> SDLDisplayEngine::pollEvents()
 //                pNewEvent = createButtonEvent(Event::BUTTON_UP, sdlEvent));
                 break;
             case SDL_KEYDOWN:
-                pNewEvent = createKeyEvent(Event::KEYDOWN, sdlEvent);
+                pNewEvent = createKeyEvent(Event::KEY_DOWN, sdlEvent);
                 break;
             case SDL_KEYUP:
-                pNewEvent = createKeyEvent(Event::KEYUP, sdlEvent);
+                pNewEvent = createKeyEvent(Event::KEY_UP, sdlEvent);
                 break;
             case SDL_QUIT:
                 pNewEvent = EventPtr(new Event(Event::QUIT, Event::NONE));
@@ -690,6 +500,10 @@ vector<EventPtr> SDLDisplayEngine::pollEvents()
             events.push_back(pNewEvent);
         }
     }
+    if (numEvents > 124) {
+        AVG_TRACE(Logger::category::EVENTS, Logger::severity::WARNING, 
+                "SDL Event queue full, dropping events.");
+    }
     return events;
 }
 
@@ -706,13 +520,13 @@ EventPtr SDLDisplayEngine::createMouseEvent(Event::Type type, const SDL_Event& s
     Uint8 buttonState = SDL_GetMouseState(&x, &y);
     x = int((x*m_Size.x)/m_WindowSize.x);
     y = int((y*m_Size.y)/m_WindowSize.y);
-    DPoint lastMousePos = m_pLastMouseEvent->getPos();
-    DPoint speed;
+    glm::vec2 lastMousePos = m_pLastMouseEvent->getPos();
+    glm::vec2 speed;
     if (lastMousePos.x == -1) {
-        speed = DPoint(0,0);
+        speed = glm::vec2(0,0);
     } else {
-        double lastFrameTime = 1000/getEffectiveFramerate();
-        speed = DPoint(x-lastMousePos.x, y-lastMousePos.y)/lastFrameTime;
+        float lastFrameTime = 1000/getEffectiveFramerate();
+        speed = glm::vec2(x-lastMousePos.x, y-lastMousePos.y)/lastFrameTime;
     }
     MouseEventPtr pEvent(new MouseEvent(type, (buttonState & SDL_BUTTON(1)) != 0,
             (buttonState & SDL_BUTTON(2)) != 0, (buttonState & SDL_BUTTON(3)) != 0,
@@ -795,7 +609,8 @@ EventPtr SDLDisplayEngine::createKeyEvent(Event::Type type, const SDL_Event& sdl
 
     KeyEventPtr pEvent(new KeyEvent(type,
             sdlEvent.key.keysym.scancode, keyCode,
-            SDL_GetKeyName(sdlEvent.key.keysym.sym), sdlEvent.key.keysym.unicode, modifiers));
+            SDL_GetKeyName(sdlEvent.key.keysym.sym), sdlEvent.key.keysym.unicode,
+                    modifiers));
     return pEvent;
 }
 
@@ -1047,32 +862,4 @@ bool SDLDisplayEngine::isFullscreen() const
     return m_bIsFullscreen;
 }
 
-IntPoint SDLDisplayEngine::getScreenResolution()
-{
-    calcScreenDimensions();
-    return m_ScreenResolution;
-}
-
-double SDLDisplayEngine::getPixelsPerMM()
-{
-    calcScreenDimensions();
-
-    return m_PPMM;
-}
-
-DPoint SDLDisplayEngine::getPhysicalScreenDimensions()
-{
-    calcScreenDimensions();
-    DPoint size;
-    DPoint screenRes = DPoint(getScreenResolution());
-    size.x = screenRes.x/m_PPMM;
-    size.y = screenRes.y/m_PPMM;
-    return size;
-}
-
-void SDLDisplayEngine::assumePixelsPerMM(double ppmm)
-{
-    m_PPMM = ppmm;
-}
-
 }
