@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -44,19 +44,22 @@ class XInputMTInputDevice;
 class MouseEvent;
 typedef boost::shared_ptr<class MouseEvent> MouseEventPtr;
 class GLContext;
-typedef boost::shared_ptr<class GLContext> GLContextPtr;
 
 class AVG_API SDLDisplayEngine: public DisplayEngine, public IInputDevice
 {
     public:
+        static void initSDL();
+        static void quitSDL();
         SDLDisplayEngine();
         virtual ~SDLDisplayEngine();
         virtual void init(const DisplayParams& dp, GLConfig glConfig);
+        IntPoint calcWindowSize(const DisplayParams& dp) const;
+
+        void setWindowTitle(const std::string& sTitle);
 
         // From DisplayEngine
         virtual void teardown();
-        virtual double getRefreshRate();
-        virtual void setGamma(double red, double green, double blue);
+        virtual void setGamma(float red, float green, float blue);
         virtual void setMousePos(const IntPoint& pos);
         virtual int getKeyModifierState() const;
 
@@ -71,54 +74,32 @@ class AVG_API SDLDisplayEngine: public DisplayEngine, public IInputDevice
 
         const IntPoint& getWindowSize() const;
         bool isFullscreen() const;
-        IntPoint getScreenResolution();
-        double getPixelsPerMM();
-        DPoint getPhysicalScreenDimensions();
-        void assumePixelsPerMM(double ppmm);
         virtual void swapBuffers();
 
     private:
-        void initSDL(int width, int height, bool isFullscreen, int bpp);
         void initTranslationTable();
-        void calcScreenDimensions(double dotsPerMM=0);
+
+        bool internalSetGamma(float red, float green, float blue);
 
         EventPtr createMouseEvent
                 (Event::Type Type, const SDL_Event & SDLEvent, long Button);
-        EventPtr createMouseButtonEvent
-                (Event::Type Type, const SDL_Event & SDLEvent);
-        EventPtr createKeyEvent
-                (Event::Type Type, const SDL_Event & SDLEvent);
+        EventPtr createMouseButtonEvent(Event::Type Type, const SDL_Event & SDLEvent);
+        EventPtr createKeyEvent(Event::Type Type, const SDL_Event & SDLEvent);
         
         IntPoint m_Size;
         bool m_bIsFullscreen;
         IntPoint m_WindowSize;
-        IntPoint m_ScreenResolution;
-        double m_PPMM;
 
         SDL_Surface * m_pScreen;
 
-        // Vertical blank stuff.
-        virtual bool initVBlank(int rate);
-        void initMacVBlank(int rate);
-        bool vbWait(int rate);
-        enum VBMethod {VB_SGI, VB_APPLE, VB_WIN, VB_NONE};
-        VBMethod m_VBMethod;
-        int m_VBMod;
-        int m_LastVBCount;
-
-        static void calcRefreshRate();
-        static double s_RefreshRate;
-
         // Event handling.
-        bool m_bMouseOverApp;
         MouseEventPtr m_pLastMouseEvent;
-        int m_NumMouseButtonsDown;
         static std::vector<long> KeyCodeTranslationTable;
         XInputMTInputDevice * m_pXIMTInputDevice;
 
-        GLContextPtr m_pGLContext;
+        GLContext* m_pGLContext;
 
-        double m_Gamma[3];
+        float m_Gamma[3];
 };
 
 typedef boost::shared_ptr<SDLDisplayEngine> SDLDisplayEnginePtr;
