@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -28,16 +28,15 @@
 
 #include "../base/IFrameEndListener.h"
 #include "../base/UTF8String.h"
-#include "../audio/IAudioSource.h"
 
 namespace avg {
 
-class VideoDecoder;
+class AsyncVideoDecoder;
 
-class AVG_API SoundNode : public AreaNode, IFrameEndListener, IAudioSource
+class AVG_API SoundNode : public AreaNode, IFrameEndListener
 {
     public:
-        static NodeDefinition createDefinition();
+        static void registerType();
 
         SoundNode(const ArgList& args);
         virtual ~SoundNode();
@@ -52,8 +51,8 @@ class AVG_API SoundNode : public AreaNode, IFrameEndListener, IAudioSource
 
         const UTF8String& getHRef() const;
         void setHRef(const UTF8String& href);
-        double getVolume();
-        void setVolume(double volume);
+        float getVolume();
+        void setVolume(float volume);
         void checkReload();
 
         long long getDuration() const;
@@ -68,8 +67,6 @@ class AVG_API SoundNode : public AreaNode, IFrameEndListener, IAudioSource
 
         virtual void onFrameEnd();
 
-        virtual int fillAudioBuffer(AudioBufferPtr pBuffer);
-
     private:
         void seek(long long destTime);
         void onEOF();
@@ -85,15 +82,16 @@ class AVG_API SoundNode : public AreaNode, IFrameEndListener, IAudioSource
         std::string m_Filename;
         bool m_bLoop;
         PyObject * m_pEOFCallback;
-        bool m_bAudioEnabled;
+        long long m_SeekBeforeCanRenderTime;
 
         long long m_StartTime;
         long long m_PauseTime;
         long long m_PauseStartTime;
 
-        VideoDecoder * m_pDecoder;
-        double m_Volume;
+        AsyncVideoDecoder* m_pDecoder;
+        float m_Volume;
         SoundState m_State;
+        int m_AudioID;
 };
 
 }
