@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -21,22 +21,26 @@
 #include "VideoNode.h"
 #include "Player.h"
 #include "OGLSurface.h"
-#include "NodeDefinition.h"
+#include "TypeDefinition.h"
 #include "Canvas.h"
 
 #include "../base/Exception.h"
 #include "../base/Logger.h"
 #include "../base/ScopeTimer.h"
 #include "../base/XMLHelper.h"
+#include "../base/ObjectCounter.h"
 
 #include "../graphics/Filterfill.h"
 #include "../graphics/GLTexture.h"
 #include "../graphics/TextureMover.h"
 
-#include "../audio/SDLAudioEngine.h"
+#include "../audio/AudioEngine.h"
 
 #include "../video/AsyncVideoDecoder.h"
-#include "../video/FFMpegDecoder.h"
+#include "../video/SyncVideoDecoder.h"
+#ifdef AVG_ENABLE_VDPAU
+#include "../video/VDPAUDecoder.h"
+#endif
 
 #include <iostream>
 #include <sstream>
@@ -45,25 +49,28 @@
 #include <unistd.h>
 #endif
 
-using namespace boost::python;
+using namespace boost;
 using namespace std;
 
 namespace avg {
 
-NodeDefinition VideoNode::createDefinition()
+void VideoNode::registerType()
 {
-    return NodeDefinition("video", Node::buildNode<VideoNode>)
-        .extendDefinition(RasterNode::createDefinition())
+    TypeDefinition def = TypeDefinition("video", "rasternode", 
+            ExportedObject::buildObject<VideoNode>)
         .addArg(Arg<UTF8String>("href", "", false, offsetof(VideoNode, m_href)))
         .addArg(Arg<bool>("loop", false, false, offsetof(VideoNode, m_bLoop)))
         .addArg(Arg<bool>("threaded", true, false, offsetof(VideoNode, m_bThreaded)))
-        .addArg(Arg<double>("fps", 0.0, false, offsetof(VideoNode, m_FPS)))
+        .addArg(Arg<float>("fps", 0.0, false, offsetof(VideoNode, m_FPS)))
         .addArg(Arg<int>("queuelength", 8, false, 
                 offsetof(VideoNode, m_QueueLength)))
-        .addArg(Arg<double>("volume", 1.0, false, offsetof(VideoNode, m_Volume)))
+        .addArg(Arg<float>("volume", 1.0, false, offsetof(VideoNode, m_Volume)))
         .addArg(Arg<bool>("accelerated", false, false,
                 offsetof(VideoNode, m_bUsesHardwareAcceleration)))
+        .addArg(Arg<bool>("enablesound", true, false,
+                offsetof(VideoNode, m_bEnableSound)))
         ;
+    TypeRegistry::get()->registerType(def);
 }
 
 VideoNode::VideoNode(const ArgList& args)
@@ -78,7 +85,9 @@ VideoNode::VideoNode(const ArgList& args)
       m_SeekBeforeCanRenderTime(0),
       m_pDecoder(0),
       m_Volume(1.0),
-      m_bUsesHardwareAcceleration(false)
+      m_bUsesHardwareAcceleration(false),
+      m_bEnableSound(true),
+      m_AudioID(-1)
 {
     args.setMembers(this);
     m_Filename = m_href;
@@ -88,11 +97,11 @@ VideoNode::VideoNode(const ArgList& args)
                 "Can't set queue length for unthreaded videos because there is no decoder queue in this case.");
     }
     if (m_bThreaded) {
-        VideoDecoderPtr pSyncDecoder = VideoDecoderPtr(new FFMpegDecoder());
-        m_pDecoder = new AsyncVideoDecoder(pSyncDecoder, m_QueueLength);
+        m_pDecoder = new AsyncVideoDecoder(m_QueueLength);
     } else {
-        m_pDecoder = new FFMpegDecoder();
+        m_pDecoder = new SyncVideoDecoder();
     }
+
     ObjectCounter::get()->incRef(&typeid(*this));
 }
 
@@ -124,6 +133,7 @@ void VideoNode::connectDisplay()
 void VideoNode::connect(CanvasPtr pCanvas)
 {
     pCanvas->registerFrameEndListener(this);
+    checkReload();
     RasterNode::connect(pCanvas);
 }
 
@@ -183,7 +193,7 @@ void VideoNode::seekToFrame(int frameNum)
     }
     exceptionIfUnloaded("seekToFrame");
     if (getCurFrame() != frameNum) {
-        long long destTime = (long long)(frameNum*1000.0/m_pDecoder->getNominalFPS());
+        long long destTime = (long long)(frameNum*1000.0/m_pDecoder->getStreamFPS());
         seek(destTime);
     }
 }
@@ -200,11 +210,32 @@ long long VideoNode::getDuration() const
     return (long long)(m_pDecoder->getVideoInfo().m_Duration*1000);
 }
 
+long long VideoNode::getVideoDuration() const
+{
+    exceptionIfUnloaded("getVideoDuration");
+    return (long long)(m_pDecoder->getVideoInfo().m_VideoDuration*1000);
+}
+
+long long VideoNode::getAudioDuration() const
+{
+    exceptionIfUnloaded("getAudioDuration");
+    if (!hasAudio()) {
+        throw Exception(AVG_ERR_INVALID_ARGS, "Video has no audio track.");
+    }
+
+    return (long long)(m_pDecoder->getVideoInfo().m_AudioDuration*1000);
+}
+
 int VideoNode::getBitrate() const
 {
     exceptionIfUnloaded("getBitrate");
     return m_pDecoder->getVideoInfo().m_Bitrate;
 }
+std::string VideoNode::getContainerFormat() const
+{
+    exceptionIfUnloaded("getContainerFormat");
+    return m_pDecoder->getVideoInfo().m_sContainerFormat;
+}
 
 string VideoNode::getVideoCodec() const
 {
@@ -286,6 +317,8 @@ void VideoNode::setEOFCallback(PyObject * pEOFCallback)
     if (pEOFCallback == Py_None) {
         m_pEOFCallback = 0;
     } else {
+        avgDeprecationWarning("1.8", "VideoNode.setEOFCallback()", 
+                "Node.subscribe(END_OF_FILE)");
         Py_INCREF(pEOFCallback);
         m_pEOFCallback = pEOFCallback;
     }
@@ -308,19 +341,19 @@ void VideoNode::setHRef(const UTF8String& href)
     checkReload();
 }
 
-double VideoNode::getVolume()
+float VideoNode::getVolume()
 {
     return m_Volume;
 }
 
-void VideoNode::setVolume(double Volume)
+void VideoNode::setVolume(float volume)
 {
-    if (Volume < 0) {
-        Volume = 0;
+    if (volume < 0) {
+        volume = 0;
     }
-    m_Volume = Volume;
-    if (m_VideoState != Unloaded && hasAudio()) {
-        m_pDecoder->setVolume(Volume);
+    m_Volume = volume;
+    if (m_AudioID != -1) {
+        AudioEngine::get()->setSourceVolume(m_AudioID, volume);
     }
 }
 
@@ -345,62 +378,65 @@ void VideoNode::checkReload()
 
 void VideoNode::onFrameEnd()
 {
+    AsyncVideoDecoder* pAsyncDecoder = dynamic_cast<AsyncVideoDecoder*>(m_pDecoder);
+    if (pAsyncDecoder && (m_VideoState == Playing || m_VideoState == Paused)) {
+        pAsyncDecoder->updateAudioStatus();
+    }
     if (m_bEOFPending) {
         // If the VideoNode is unlinked by python in onEOF, the following line prevents
         // the object from being deleted until we return from this function.
-        NodePtr pTempThis = shared_from_this();
+        NodePtr pTempThis = getSharedThis();
         m_bEOFPending = false;
         onEOF();
     }
 }
 
-int VideoNode::fillAudioBuffer(AudioBufferPtr pBuffer)
-{
-    AVG_ASSERT(m_bThreaded);
-    if (m_VideoState == Playing) {
-        return m_pDecoder->fillAudioBuffer(pBuffer);
-    } else {
-        return 0;
-    }
-}
-
-void VideoNode::changeVideoState(VideoState NewVideoState)
+void VideoNode::changeVideoState(VideoState newVideoState)
 {
     long long curTime = Player::get()->getFrameTime(); 
-    if (m_VideoState == NewVideoState) {
+    if (m_VideoState == newVideoState) {
         return;
     }
     if (m_VideoState == Unloaded) {
         m_PauseStartTime = curTime;
         open();
     }
-    if (NewVideoState == Unloaded) {
+    if (newVideoState == Unloaded) {
         close();
     }
     if (getState() == NS_CANRENDER) {
         if (m_VideoState == Unloaded) {
             startDecoding();
         }
-        if (NewVideoState == Paused) {
+        if (newVideoState == Paused) {
             m_PauseStartTime = curTime;
-        } else if (NewVideoState == Playing && m_VideoState == Paused) {
+            if (m_AudioID != -1) {
+                AudioEngine::get()->pauseSource(m_AudioID);
+            }
+        } else if (newVideoState == Playing && m_VideoState == Paused) {
 /*            
             cerr << "Play after pause:" << endl;
             cerr << "  getFrameTime()=" << curTime << endl;
             cerr << "  m_PauseStartTime=" << m_PauseStartTime << endl;
             cerr << "  offset=" << (1000.0/m_pDecoder->getFPS()) << endl;
 */
+            if (m_AudioID != -1) {
+                AudioEngine::get()->playSource(m_AudioID);
+            }
             m_PauseTime += (curTime-m_PauseStartTime
                     - (long long)(1000.0/m_pDecoder->getFPS()));
         }
     }
-    m_VideoState = NewVideoState;
+    m_VideoState = newVideoState;
 }
 
 void VideoNode::seek(long long destTime) 
 {
     if (getState() == NS_CANRENDER) {    
-        m_pDecoder->seek(double(destTime)/1000.0);
+        if (m_AudioID != -1) {
+            AudioEngine::get()->notifySeek(m_AudioID);
+        }
+        m_pDecoder->seek(float(destTime)/1000.0f);
         m_StartTime = Player::get()->getFrameTime() - destTime;
         m_JitterCompensation = 0.5;
         m_PauseTime = 0;
@@ -419,8 +455,7 @@ void VideoNode::open()
     m_FramesTooLate = 0;
     m_FramesInRowTooLate = 0;
     m_FramesPlayed = 0;
-    m_pDecoder->open(m_Filename, m_bThreaded, m_bUsesHardwareAcceleration);
-    m_pDecoder->setVolume(m_Volume);
+    m_pDecoder->open(m_Filename, m_bUsesHardwareAcceleration, m_bEnableSound);
     VideoInfo videoInfo = m_pDecoder->getVideoInfo();
     if (!videoInfo.m_bHasVideo) {
         m_pDecoder->close();
@@ -431,34 +466,38 @@ void VideoNode::open()
     m_JitterCompensation = 0.5;
     m_PauseTime = 0;
 
+    m_bSeekPending = false;
     m_bFirstFrameDecoded = false;
     m_bFrameAvailable = false;
     m_bUsesHardwareAcceleration = videoInfo.m_bUsesVDPAU;
+    setViewport(-32767, -32767, -32767, -32767);
 }
 
 void VideoNode::startDecoding()
 {
     const AudioParams * pAP = 0;
-    SDLAudioEngine* pAudioEngine = SDLAudioEngine::get();
+    AudioEngine* pAudioEngine = AudioEngine::get();
     if (pAudioEngine) {
         pAP = pAudioEngine->getParams();
     }
-    m_pDecoder->startDecoding(GLContext::getCurrent()->isUsingShaders(), pAP);
+    m_pDecoder->startDecoding(GLContext::getMain()->useGPUYUVConversion(), pAP);
     VideoInfo videoInfo = m_pDecoder->getVideoInfo();
     if (m_FPS != 0.0) {
         if (videoInfo.m_bHasAudio) {
-            AVG_TRACE(Logger::WARNING, 
-                    getID() + ": Can't set FPS if video contains audio. Ignored.");
+            AVG_LOG_WARNING(getID() + ": Can't set FPS if video contains audio. Ignored.");
         } else {
             m_pDecoder->setFPS(m_FPS);
         }
     }
     if (videoInfo.m_bHasAudio && pAudioEngine) {
-        pAudioEngine->addSource(this);
+        AsyncVideoDecoder* pAsyncDecoder = 
+                dynamic_cast<AsyncVideoDecoder*>(m_pDecoder);
+        m_AudioID = pAudioEngine->addSource(*pAsyncDecoder->getAudioMsgQ(), 
+                *pAsyncDecoder->getAudioStatusQ());
+        pAudioEngine->setSourceVolume(m_AudioID, m_Volume);
     }
     m_bSeekPending = true;
     
-    setViewport(-32767, -32767, -32767, -32767);
     createTextures(videoInfo.m_Size);
    
     if (m_SeekBeforeCanRenderTime != 0) {
@@ -474,8 +513,8 @@ void VideoNode::createTextures(IntPoint size)
     if (pixelFormatIsPlanar(pf)) {
         m_pTextures[0] = GLTexturePtr(new GLTexture(size, I8, bMipmap));
         IntPoint halfSize(size.x/2, size.y/2);
-        m_pTextures[1] = GLTexturePtr(new GLTexture(halfSize, I8, bMipmap));
-        m_pTextures[2] = GLTexturePtr(new GLTexture(halfSize, I8, bMipmap));
+        m_pTextures[1] = GLTexturePtr(new GLTexture(halfSize, I8, bMipmap, 128));
+        m_pTextures[2] = GLTexturePtr(new GLTexture(halfSize, I8, bMipmap, 128));
         if (pixelFormatHasAlpha(pf)) {
             m_pTextures[3] = GLTexturePtr(new GLTexture(size, I8, bMipmap));
         }
@@ -501,13 +540,15 @@ void VideoNode::createTextures(IntPoint size)
     } else {
         getSurface()->create(pf, m_pTextures[0]);
     }
+    newSurface();
 }
 
 void VideoNode::close()
 {
-    SDLAudioEngine* pAudioEngine = SDLAudioEngine::get();
-    if (hasAudio() && pAudioEngine) {
-        pAudioEngine->removeSource(this);
+    AudioEngine* pAudioEngine = AudioEngine::get();
+    if (m_AudioID != -1) {
+        pAudioEngine->removeSource(m_AudioID);
+        m_AudioID = -1;
     }
     m_pDecoder->close();
     if (m_FramesTooLate > 0) {
@@ -517,8 +558,9 @@ void VideoNode::close()
         } else {
             sID = getID();
         }
-        AVG_TRACE(Logger::PROFILE_VIDEO, "Missed video frames for '" << sID << "': " 
-                << m_FramesTooLate << " of " << m_FramesPlayed);
+        AVG_TRACE(Logger::category::PROFILE_VIDEO, Logger::severity::INFO,
+                "Missed video frames for '" << sID << "': " << m_FramesTooLate <<
+                " of " << m_FramesPlayed);
         m_FramesTooLate = 0;
     }
 }
@@ -537,7 +579,7 @@ IntPoint VideoNode::getMediaSize()
     }
 }
 
-double VideoNode::getFPS() const
+float VideoNode::getFPS() const
 {
     return m_pDecoder->getFPS();
 }
@@ -553,6 +595,7 @@ long long VideoNode::getNextFrameTime() const
         case Unloaded:
             return 0;
         case Paused:
+            AVG_ASSERT(m_PauseStartTime-m_StartTime >= 0);
             return m_PauseStartTime-m_StartTime;
         case Playing:
             {
@@ -596,10 +639,11 @@ void VideoNode::exceptionIfUnloaded(const std::string& sFuncName) const
 
 static ProfilingZoneID PrerenderProfilingZone("VideoNode::prerender");
 
-void VideoNode::preRender()
+void VideoNode::preRender(const VertexArrayPtr& pVA, bool bIsParentActive, 
+        float parentEffectiveOpacity)
 {
     ScopeTimer timer(PrerenderProfilingZone);
-    Node::preRender();
+    Node::preRender(pVA, bIsParentActive, parentEffectiveOpacity);
     if (isVisible()) {
         if (m_VideoState != Unloaded) {
             if (m_VideoState == Playing) {
@@ -616,35 +660,36 @@ void VideoNode::preRender()
             }
         }
     } else {
-        if (m_bSeekPending && m_bFirstFrameDecoded && m_VideoState != Unloaded) {
+        if (m_VideoState != Unloaded && m_bSeekPending && m_bFirstFrameDecoded) {
             renderFrame();
         }
         if (m_VideoState == Playing) {
             // Throw away frames that are not visible to make sure the video 
             // stays in sync.
-            m_pDecoder->throwAwayFrame(getNextFrameTime()/1000.0);
+            m_pDecoder->throwAwayFrame(getNextFrameTime()/1000.0f);
 
             if (m_pDecoder->isEOF()) {
                 updateStatusDueToDecoderEOF();
             }
         }
     }
+    calcVertexArray(pVA);
 }
 
 static ProfilingZoneID RenderProfilingZone("VideoNode::render");
 
-void VideoNode::render(const DRect& rect)
+void VideoNode::render()
 {
     ScopeTimer timer(RenderProfilingZone);
     if (m_VideoState != Unloaded && m_bFirstFrameDecoded) {
-        blt32(getSize(), getEffectiveOpacity(), getBlendMode());
+        blt32(getTransform(), getSize(), getEffectiveOpacity(), getBlendMode());
     }
 }
 
 VideoNode::VideoAccelType VideoNode::getVideoAccelConfig()
 {
 #ifdef AVG_ENABLE_VDPAU
-    if (VDPAU::isAvailable()) {
+    if (VDPAUDecoder::isAvailable()) {
         return VDPAU;
     }
 #endif
@@ -653,12 +698,26 @@ VideoNode::VideoAccelType VideoNode::getVideoAccelConfig()
 
 bool VideoNode::renderFrame()
 {
-    FrameAvailableCode frameAvailable = renderToSurface();
+    FrameAvailableCode frameAvailable =
+            m_pDecoder->renderToTexture(m_pTextures, getNextFrameTime()/1000.0f);
+
+    // Even with vsync, frame duration has a bit of jitter. If the video frames rendered
+    // are at the border of a frame's time, this can cause irregular display times.
+    // So, if we detect this condition, we adjust the frame time by a small fraction
+    // to move it towards the center of the time slot.
+    long long jitter = (long long)(getNextFrameTime()-m_pDecoder->getCurTime()*1000);
+    if (jitter > (long long)(0.4*(1000/m_pDecoder->getFPS()))) {
+        m_JitterCompensation += 0.05;
+        if (m_JitterCompensation > 1) {
+            m_JitterCompensation -= 1;
+        }
+    }
+
     if (m_pDecoder->isEOF()) {
-//        AVG_TRACE(Logger::PROFILE, "------------------ EOF -----------------");
         updateStatusDueToDecoderEOF();
         if (m_bLoop) {
-            frameAvailable = renderToSurface();
+            frameAvailable = 
+                    m_pDecoder->renderToTexture(m_pTextures, getNextFrameTime()/1000.0f);
         }
     }
 
@@ -666,17 +725,16 @@ bool VideoNode::renderFrame()
         case FA_NEW_FRAME:
             m_FramesPlayed++;
             m_FramesInRowTooLate = 0;
-            bind();
             m_bSeekPending = false;
             setMaskCoords();
-//            AVG_TRACE(Logger::PROFILE, "New frame.");
+//            AVG_TRACE(Logger::category::PROFILE, "New frame.");
             break;
         case FA_STILL_DECODING:
             {
                 m_FramesPlayed++;
                 m_FramesTooLate++;
                 m_FramesInRowTooLate++;
-                double framerate = Player::get()->getEffectiveFramerate();
+                float framerate = Player::get()->getEffectiveFramerate();
                 long long frameTime = Player::get()->getFrameTime();
                 if (m_VideoState == Playing) {
                     if (m_FramesInRowTooLate > 3 && framerate != 0) {
@@ -702,50 +760,18 @@ bool VideoNode::renderFrame()
                     }
                 }
             }
-//            AVG_TRACE(Logger::PROFILE, "Missed video frame.");
+//            AVG_TRACE(Logger::category::PROFILE, "Missed video frame.");
             break;
         case FA_USE_LAST_FRAME:
             m_FramesInRowTooLate = 0;
             m_bSeekPending = false;
-//            AVG_TRACE(Logger::PROFILE, "Video frame reused.");
+//            AVG_TRACE(Logger::category::PROFILE, "Video frame reused.");
             break;
         default:
             AVG_ASSERT(false);
     }
 
     return (frameAvailable == FA_NEW_FRAME);
-    return false;
-}
-
-FrameAvailableCode VideoNode::renderToSurface()
-{
-    FrameAvailableCode frameAvailable;
-    PixelFormat pf = m_pDecoder->getPixelFormat();
-    std::vector<BitmapPtr> pBmps;
-    for (unsigned i=0; i<getNumPixelFormatPlanes(pf); ++i) {
-        pBmps.push_back(m_pTextures[i]->lockStreamingBmp());
-    }
-    if (pixelFormatIsPlanar(pf)) {
-        frameAvailable = m_pDecoder->renderToBmps(pBmps, getNextFrameTime()/1000.0);
-    } else {
-        frameAvailable = m_pDecoder->renderToBmp(pBmps[0], getNextFrameTime()/1000.0);
-    }
-    for (unsigned i=0; i<getNumPixelFormatPlanes(pf); ++i) {
-        m_pTextures[i]->unlockStreamingBmp(frameAvailable == FA_NEW_FRAME);
-    }
-
-    // Even with vsync, frame duration has a bit of jitter. If the video frames rendered
-    // are at the border of a frame's time, this can cause irregular display times.
-    // So, if we detect this condition, we adjust the frame time by a small fraction
-    // to move it towards the center of the time slot.
-    long long jitter = (long long)(getNextFrameTime()-m_pDecoder->getCurTime()*1000);
-    if (jitter > (long long)(0.4*(1000/m_pDecoder->getFPS()))) {
-        m_JitterCompensation += 0.05;
-        if (m_JitterCompensation > 1) {
-            m_JitterCompensation -= 1;
-        }
-    }
-    return frameAvailable;
 }
 
 void VideoNode::onEOF()
@@ -755,22 +781,26 @@ void VideoNode::onEOF()
         PyObject * result = PyEval_CallObject(m_pEOFCallback, arglist);
         Py_DECREF(arglist);    
         if (!result) {
-            throw error_already_set();
+            throw py::error_already_set();
         }
         Py_DECREF(result);
     }
+    notifySubscribers("END_OF_FILE");
 }
 
-
 void VideoNode::updateStatusDueToDecoderEOF()
 {
     m_bEOFPending = true;
     if (m_bLoop) {
         m_StartTime = Player::get()->getFrameTime();
+        m_PauseStartTime = Player::get()->getFrameTime();
         m_JitterCompensation = 0.5;
         m_PauseTime = 0;
         m_FramesInRowTooLate = 0;
         m_bFrameAvailable = false;
+        if (m_AudioID != -1) {
+            AudioEngine::get()->notifySeek(m_AudioID);
+        }
         m_pDecoder->loop();
     } else {
         changeVideoState(Paused);
