@@ -1,5 +1,5 @@
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -19,17 +19,15 @@
 #
 
 from libavg import avg
-g_Player = None
 
 class RoundedRect(avg.PolygonNode):
     def __init__(self, size, radius, pos=(0,0), parent=None, **kwargs):
-        avg.PolygonNode.__init__(self, **kwargs)
+        super(RoundedRect, self).__init__(**kwargs)
         self.__pos = avg.Point2D(pos)
         self.__size = avg.Point2D(size)
         self.__radius = radius
         self.__calcPolygon()
-        if parent:
-            parent.appendChild(self)
+        self.registerInstance(self, parent)
         
     def getPos(self):
         return self.__pos
@@ -65,14 +63,14 @@ class RoundedRect(avg.PolygonNode):
                 pos.append(p)
             return pos
 
-        if self.__size.x < self.__radius*2:
-            self.__radius = self.__size.x/2
-        if self.__size.y < self.__radius*2:
-            self.__radius = self.__size.y/2
-        if self.__radius == 0:
-            self.__radius = 0.01
-        pos = []
         r = self.__radius
+        if self.__size.x < r*2:
+            r = self.__size.x/2
+        if self.__size.y < r*2:
+            r = self.__size.y/2
+        if r == 0:
+            r = 0.01
+        pos = []
         size = self.__size
         pos.extend(calcQuarterCircle(self.pos+(size.x-r,r), r, -1.57))
         pos.extend(calcQuarterCircle(self.pos+(size.x-r,size.y-r), r, 0))
@@ -84,14 +82,13 @@ class RoundedRect(avg.PolygonNode):
 class PieSlice(avg.PolygonNode):
     def __init__(self, radius, startangle, endangle, pos=(0,0), parent=None,
             **kwargs):
-        avg.PolygonNode.__init__(self, **kwargs)
+        super(PieSlice, self).__init__(**kwargs)
         self.__pos = avg.Point2D(pos)
         self.__radius = radius
         self.__startangle = startangle
         self.__endangle = endangle
         self.__calcPolygon()
-        if parent:
-            parent.appendChild(self)
+        self.registerInstance(self, parent)
 
     def getPos(self):
         return self.__pos
@@ -148,14 +145,13 @@ class Arc(avg.PolyLineNode):
     # TODO: Code duplication with PieSlice
     def __init__(self, radius, startangle, endangle, pos=(0,0), parent=None,
             **kwargs):
-        avg.PolyLineNode.__init__(self, **kwargs)
+        super(Arc, self).__init__(**kwargs)
         self.__pos = avg.Point2D(pos)
         self.__radius = radius
         self.__startangle = startangle
         self.__endangle = endangle
         self.__calcPolygon()
-        if parent:
-            parent.appendChild(self)
+        self.registerInstance(self, parent)
 
     def getPos(self):
         return self.__pos
