@@ -6,10 +6,10 @@ from libavg import *
 def startAnim():
     animObj.start()
 
-player = avg.Player.get()
-player.loadFile("text.avg")
+canvas = player.createMainCanvas(size=(640,480))
+rootNode = canvas.getRootNode()
+node = avg.WordsNode(pos=(10,10), text="Hello World", parent=rootNode)
 
-node = player.getElementByID("HelloText")
 animObj = LinearAnim(node, "x", 2000, 0, 200)
 player.setTimeout(0, startAnim)
 
