@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 #  libavg - Media Playback Engine. 
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -20,23 +20,16 @@
 #
 #  Current versions can be found at www.libavg.de
 
-from libavg import avg, AVGApp
-from libavg.avg import ImageNode
+from libavg import avg, app
 
-g_Player = avg.Player.get()
-
-class INVERT(AVGApp):
-
-    multiTouch = False
-
-    def init(self):
-        rootNode = g_Player.getRootNode()
-
-        orig = ImageNode(parent = rootNode, href='../graphics/testfiles/hsl.png')
-        invert = ImageNode(parent = rootNode, href='../graphics/testfiles/hsl.png',
+class Invert(app.MainDiv):
+    def onInit(self):
+        orig = avg.ImageNode(parent=self, href='../test/media/hsl.png')
+        invert = avg.ImageNode(parent=self, href='../test/media/hsl.png',
                 pos=(orig.size.x+10, 0))
         invert.setEffect(avg.InvertFXNode())
 
+
 if __name__ == '__main__':
-    INVERT.start(resolution=(200,200),debugWindowSize=(200,200))
+    app.App().run(Invert(), app_resolution='200x200')
 
