@@ -1,11 +1,11 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from libavg import avg
+from libavg import avg, player
 
-player = avg.Player.get()
-player.loadString("""<avg size="(160,120)"/>""")
+canvas = player.createMainCanvas(size=(160,120))
+rootNode = canvas.getRootNode()
 videoNode = avg.VideoNode(href="mpeg1-48x48-sound.avi", pos=(10,10), 
-        parent=player.getRootNode())
+        parent=rootNode)
 videoNode.play()
 player.play()
