@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,7 @@
 # Current versions can be found at www.libavg.de
 #
 
-import unittest
-
-from libavg import avg
+from libavg import avg, player
 from testcase import *
 
 class AnimTestCase(AVGTestCase):
@@ -33,7 +31,7 @@ class AnimTestCase(AVGTestCase):
         root = self.loadEmptyScene()
         self.__node = avg.ImageNode(id="test", pos=(64,30), href="rgb24-65x65.png",
                 parent=root)
-        Player.setFakeFPS(10)
+        player.setFakeFPS(10)
 
     def testAnimType(self, curAnim, imgBaseName):
         def onStop():
@@ -54,27 +52,27 @@ class AnimTestCase(AVGTestCase):
         self.__anim.setStopCallback(onStop)
         self.__onStopCalled = False
         self.assertException(lambda: self.__anim.start())
-        self.start((
-                 startAnim,
-                 lambda: self.compareImage(imgBaseName+"1", False),
-                 lambda: self.compareImage(imgBaseName+"2", False),
-                 lambda: self.compareImage(imgBaseName+"3", False),
+        self.start(False,
+                (startAnim,
+                 lambda: self.compareImage(imgBaseName+"1"),
+                 lambda: self.compareImage(imgBaseName+"2"),
+                 lambda: self.compareImage(imgBaseName+"3"),
                  lambda: self.assert_(self.__onStopCalled),
                  lambda: self.assert_(not(self.__anim.isRunning())),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0),
-                 lambda: self.compareImage(imgBaseName+"4", False),
+                 lambda: self.compareImage(imgBaseName+"4"),
                  lambda: self.assertEqual(self.__node.x, 100),
                  startAnim,
-                 lambda: self.compareImage(imgBaseName+"1", False),
+                 lambda: self.compareImage(imgBaseName+"1"),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 1),
                  abortAnim,
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0),
-                 lambda: self.compareImage(imgBaseName+"5", False),
+                 lambda: self.compareImage(imgBaseName+"5"),
                  lambda: self.assert_(not(self.__anim.isRunning())),
                  None,
                  lambda: self.assert_(self.__onStopCalled),
                  startKeepAttr,
-                 lambda: self.compareImage(imgBaseName+"6", False),
+                 lambda: self.compareImage(imgBaseName+"6"),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 1)
                 ))
         self.__anim = None
@@ -92,15 +90,15 @@ class AnimTestCase(AVGTestCase):
             self.__onStopBeforeOnStart = self.__onStopCalled
         
         self.initScene()
-        sameNode = Player.getElementByID("test")
+        sameNode = player.getElementByID("test")
         anim1 = avg.LinearAnim(self.__node, "x", 500, 0, 100,
                                False, None, on1Stop)
         anim2 = avg.LinearAnim(sameNode, "x", 300, 0, 100,
                                False, on2Start)
         self.__onStopCalled = False
         self.__onStopBeforeOnStart = False
-        self.start((
-                 lambda: anim1.start(),
+        self.start(False,
+                (lambda: anim1.start(),
                  lambda: self.assert_(not(self.__onStopCalled)),
                  lambda: anim2.start(),
                  lambda: self.assert_(self.__onStopBeforeOnStart),
@@ -116,11 +114,11 @@ class AnimTestCase(AVGTestCase):
         self.initScene()
         self.__node.opacity=0.5
         self.__onStopCalled = False
-        self.start((
-                 lambda: avg.fadeIn(self.__node, 200, 1, onStop),
-                 lambda: self.compareImage("testFadeIn1", False),
-                 lambda: self.compareImage("testFadeIn2", False),
-                 lambda: self.compareImage("testFadeIn3", False),
+        self.start(False,
+                (lambda: avg.fadeIn(self.__node, 200, 1, onStop),
+                 lambda: self.compareImage("testFadeIn1"),
+                 lambda: self.compareImage("testFadeIn2"),
+                 lambda: self.compareImage("testFadeIn3"),
                  lambda: self.assert_(self.__onStopCalled),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0)
                 ))
@@ -133,11 +131,11 @@ class AnimTestCase(AVGTestCase):
         self.initScene()
         self.__node.opacity=0.5
         self.__onStopCalled = False
-        self.start((
-                 lambda: avg.fadeOut(self.__node, 200, onStop),
-                 lambda: self.compareImage("testFadeOut1", False),
-                 lambda: self.compareImage("testFadeOut2", False),
-                 lambda: self.compareImage("testFadeOut3", False),
+        self.start(False,
+                (lambda: avg.fadeOut(self.__node, 200, onStop),
+                 lambda: self.compareImage("testFadeOut1"),
+                 lambda: self.compareImage("testFadeOut2"),
+                 lambda: self.compareImage("testFadeOut3"),
                  lambda: self.assert_(self.__onStopCalled),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0)
                 ))
@@ -159,9 +157,9 @@ class AnimTestCase(AVGTestCase):
         self.initScene()
         self.__anim = avg.LinearAnim(self.__node, "x", 0, 0, 100, False, None, onStop)
         self.__onStopCalled = False
-        self.start((
-                 startAnim,
-                 lambda: self.compareImage("testLinearAnimZeroDurationC1", False),
+        self.start(False,
+                (startAnim,
+                 lambda: self.compareImage("testLinearAnimZeroDurationC1"),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0),
                  lambda: self.assert_(self.__onStopCalled),
                  lambda: self.assert_(not(self.__anim.isRunning()))
@@ -180,10 +178,10 @@ class AnimTestCase(AVGTestCase):
             anim.start()
 
         self.initScene()
-        self.start((
-             forth,
-             lambda: self.delay(300),
-            ))
+        self.start(False,
+                (forth,
+                 lambda: self.delay(300),
+                ))
 
     def testPointAnim(self):
         def startAnim():
@@ -196,32 +194,29 @@ class AnimTestCase(AVGTestCase):
         self.initScene()
         self.__anim = avg.LinearAnim(self.__node, "pos", 200, avg.Point2D(0,0), 
                 avg.Point2D(100,40), False)
-        self.start((
-                 startAnim,
-                 lambda: self.compareImage("testPointAnim1", False),
-                 lambda: self.compareImage("testPointAnim2", False),
+        self.start(False,
+                (startAnim,
+                 lambda: self.compareImage("testPointAnim1"),
+                 lambda: self.compareImage("testPointAnim2"),
                  None,
                  lambda: self.assert_(not(self.__anim.isRunning())),
                  startKeepAttr,
-                 lambda: self.compareImage("testPointAnim3", False),
+                 lambda: self.compareImage("testPointAnim3"),
                 ))
 
     def testIntAnim(self):
-        def startAnim():
-            self.__anim.start()
-
         self.initScene()
         self.__doubleAnim = avg.LinearAnim(self.__node, "x", 300, 0, 100, True)
         self.__pointAnim = avg.LinearAnim(self.__node, "pos", 200, avg.Point2D(0,0), 
                 avg.Point2D(100,40), True)
-        self.start((
-                 self.__doubleAnim.start,
+        self.start(False,
+                (self.__doubleAnim.start,
                  lambda: self.delay(100),
-                 lambda: self.compareImage("testIntAnim1", False),
+                 lambda: self.compareImage("testIntAnim1"),
                  self.__doubleAnim.abort,
                  self.__pointAnim.start,
                  lambda: self.delay(100),
-                 lambda: self.compareImage("testIntAnim2", False),
+                 lambda: self.compareImage("testIntAnim2"),
                 ))
 
     def testEaseInOutAnim(self):
@@ -247,17 +242,17 @@ class AnimTestCase(AVGTestCase):
 
         self.__animStarted = False
         self.__animStopped = False
-        self.start((
-                 self.__anim.start,
+        self.start(False,
+                (self.__anim.start,
                  lambda: self.assert_(self.__animStarted),
-                 lambda: self.compareImage("testContinuousAnim1", False),
+                 lambda: self.compareImage("testContinuousAnim1"),
                  self.__anim.abort,
                  lambda: self.assert_(self.__animStopped),
                  reset,
                  self.__anim.start,
                  self.__linearAnim.start,
                  lambda: self.assert_(self.__animStopped),
-                 lambda: self.compareImage("testContinuousAnim2", False),
+                 lambda: self.compareImage("testContinuousAnim2"),
                  self.__linearAnim.abort,
                 ))
 
@@ -271,8 +266,8 @@ class AnimTestCase(AVGTestCase):
 
         self.initScene()
         self.__endCalled = False
-        self.start((
-                 startAnim, 
+        self.start(False,
+                (startAnim, 
                  lambda: self.assert_(self.anim.isRunning()),
                  lambda: self.delay(200),
                  lambda: self.assert_(not(self.anim.isRunning())),
@@ -318,24 +313,24 @@ class AnimTestCase(AVGTestCase):
             node = avg.ImageNode(id=str(i), pos=(64, i*20), href="rgb24-64x64.png")
             root.appendChild(node)
             self.nodes.append(node)
-        Player.setFakeFPS(10)
+        player.setFakeFPS(10)
         self.__endCalled = False
-        self.start((
-                 startFireForgetAnim,
+        self.start(False,
+                (startFireForgetAnim,
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 2),
                  None,
                  startAnim,
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 3),
-                 lambda: self.compareImage("testParallelAnimC1", False),
+                 lambda: self.compareImage("testParallelAnimC1"),
                  lambda: self.assert_(self.anim.isRunning()),
                  lambda: self.delay(200),
                  lambda: self.assert_(not(self.anim.isRunning())),
-                 lambda: self.compareImage("testParallelAnimC2", False),
+                 lambda: self.compareImage("testParallelAnimC2"),
                  lambda: self.assert_(self.__endCalled),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0),
                  startAnim,
                  abortAnim,
-                 lambda: self.compareImage("testParallelAnimC3", False),
+                 lambda: self.compareImage("testParallelAnimC3"),
                  lambda: self.assert_(self.__endCalled),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 0),
                  startTimedAnim,
@@ -356,8 +351,8 @@ class AnimTestCase(AVGTestCase):
             avg.LinearAnim(self.__node, "x", 300, 0, 100, False, None).start()
         
         self.initScene()
-        self.start((
-                 makeAnims,
+        self.start(False,
+                (makeAnims,
                  None
                 ))
 
@@ -387,26 +382,81 @@ class AnimTestCase(AVGTestCase):
         self.__state1StopCallbackCalled = False
         self.__state2StartCallbackCalled = False
         self.__stop1Start2CallbackOrder = False
-        self.start((
-                 makeAnim,
-                 lambda: self.compareImage("testStateAnimC1", False),
+        self.start(False,
+                (makeAnim,
+                 lambda: self.compareImage("testStateAnimC1"),
                  lambda: self.anim.setState("STATE1"),
                  None,
-                 lambda: self.compareImage("testStateAnimC2", False),
+                 lambda: self.compareImage("testStateAnimC2"),
                  lambda: self.assertEqual(self.anim.getState(), "STATE2"),
-                 lambda: self.compareImage("testStateAnimC3", False),
+                 lambda: self.compareImage("testStateAnimC3"),
                  lambda: self.assert_(self.__state1StopCallbackCalled),
                  lambda: self.assert_(self.__state2StartCallbackCalled),
                  lambda: self.assert_(self.__stop1Start2CallbackOrder),
                  lambda: self.assertEqual(self.anim.getState(), "STATE3"),
-                 lambda: self.compareImage("testStateAnimC4", False),
+                 lambda: self.compareImage("testStateAnimC4"),
                  lambda: self.anim.setState("STATE1"),
                  lambda: self.assertEqual(avg.getNumRunningAnims(), 1),
-                 lambda: self.compareImage("testStateAnimC5", False),
+                 lambda: self.compareImage("testStateAnimC5"),
                  killAnim,
-#                 lambda: Player.getTestHelper().dumpObjects()
+#                 lambda: player.getTestHelper().dumpObjects()
                 ))
 
+    def testNonNodeAttrAnim(self):
+        class GenericClass(object):
+            def __init__(self):
+                self.numberValue = 0
+                self.pointValue = avg.Point2D(0.0, 0.0)
+
+        def on1Stop():
+            self.__onStopCalled = True
+
+        def on2Start():
+            self.__onStopBeforeOnStart = self.__onStopCalled
+
+        self.loadEmptyScene()
+        player.setFakeFPS(10)
+        genericObject1 = GenericClass()
+        genericObject2 = genericObject1
+        genericObject3 = GenericClass()
+        anim1 = avg.LinearAnim(genericObject1, "numberValue", 1000, 0, 100,
+                False, None, on1Stop)
+        anim2 = avg.LinearAnim(genericObject2, "numberValue", 1200, 0, 200,
+                False, on2Start)
+        anim3 = avg.LinearAnim(genericObject1, "pointValue", 800, (0, 0), (100, 200))
+        anim4 = avg.LinearAnim(genericObject3, "numberValue", 400, 0, 42)
+        self.__onStopCalled = False
+        self.__onStopBeforeOnStart = False
+        self.start(False,
+                (lambda: anim1.start(),
+                 lambda: self.assert_(anim1.isRunning()),
+                 lambda: self.assert_(not(self.__onStopCalled)),
+                 lambda: anim2.start(),
+                 lambda: self.assert_(self.__onStopBeforeOnStart),
+                 lambda: self.assert_(not(anim1.isRunning())),
+                 lambda: self.assert_(anim2.isRunning()),
+                 lambda: self.assertEqual(avg.getNumRunningAnims(), 1),
+                 lambda: anim3.start(),
+                 lambda: self.assert_(anim2.isRunning()),
+                 lambda: self.assert_(anim3.isRunning()),
+                 lambda: self.assertEqual(avg.getNumRunningAnims(), 2),
+                 lambda: anim4.start(),
+                 lambda: self.assert_(anim4.isRunning()),
+                 lambda: self.assertEqual(avg.getNumRunningAnims(), 3),
+                 lambda: self.delay(200),
+                 lambda: self.assertEqual(avg.getNumRunningAnims(), 0),
+                 lambda: self.assert_(genericObject1.numberValue == 200),
+                 lambda: self.assert_(genericObject2.pointValue == (100, 200)),
+                 lambda: self.assert_(genericObject3.numberValue == 42)
+                ))
+        anim1 = None
+        anim2 = None
+        anim3 = None
+        anim4 = None
+        genericObject1 = None
+        genericObject2 = None
+        genericObject3 = None
+
 
 def animTestSuite(tests):
     availableTests = (
@@ -425,7 +475,7 @@ def animTestSuite(tests):
         "testParallelAnim",
         "testParallelAnimRegistry",
         "testStateAnim",
+        "testNonNodeAttrAnim"
         )
     return createAVGTestSuite(availableTests, AnimTestCase, tests)
 
-Player = avg.Player.get()
