@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,10 @@
 # Current versions can be found at www.libavg.de
 #
 
-from libavg import avg, utils
+from libavg import avg, utils, player
 from testcase import *
 
+
 class FXTestCase(AVGTestCase):
     def __init__(self, testFuncName):
         AVGTestCase.__init__(self, testFuncName)
@@ -44,6 +45,10 @@ class FXTestCase(AVGTestCase):
                     fillcolor="FFFFFF")
             root.insertChild(node, 0)
 
+        def emptyImageFX():
+            node = avg.ImageNode(parent=root, href="", pos=(64,0))
+            node.setEffect(avg.NullFXNode())
+
         # Initial setup is 3x2 images: 
         # rows: no alpha, alpha, alpha & opacity 0.6
         # cols: no FX, FX
@@ -67,27 +72,28 @@ class FXTestCase(AVGTestCase):
                 opacity=0.6)
             configureNode(node, fx)
 
-        self.start((
-                 lambda: self.compareImage("testImageNullFX1", False),
+        self.start(False,
+                (lambda: self.compareImage("testImageNullFX1"),
                  addBgNode,
-                 lambda: self.compareImage("testImageNullFX2", False),
+                 lambda: self.compareImage("testImageNullFX2"),
                  activateFX,
-                 lambda: self.compareImage("testImageNullFX2", False),
+                 lambda: self.compareImage("testImageNullFX2"),
                  newNode,
-                 lambda: self.compareImage("testImageNullFX3", False),
+                 lambda: self.compareImage("testImageNullFX3"),
                  newFX,
-                 lambda: self.compareImage("testImageNullFX3", False),
+                 lambda: self.compareImage("testImageNullFX3"),
+                 emptyImageFX,
                  lambda: utils.initFXCache(10),
                 ))
 
     def testVideoNullFX(self):
         root = self.loadEmptyScene()
-        Player.setFakeFPS(25)
-        node = avg.VideoNode(parent=root, href="../video/testfiles/mjpeg-48x48.avi",
+        player.setFakeFPS(25)
+        node = avg.VideoNode(parent=root, href="mjpeg-48x48.avi",
                 threaded=False)
         node.setEffect(avg.NullFXNode())
         node.play()
-        self.start((lambda: self.compareImage("testVideoNullFX", False),))
+        self.start(False, (lambda: self.compareImage("testVideoNullFX"),))
 
     def testWordsNullFX(self):
         root = self.loadEmptyScene()
@@ -95,22 +101,28 @@ class FXTestCase(AVGTestCase):
         node.setEffect(avg.NullFXNode())
         node = avg.WordsNode(parent=root, text="testtext", pos=(0,20),
                 font="Bitstream Vera Sans")
-        self.start((
-                 lambda: self.compareImage("testWordsNullFX", True),
+        self.start(True,
+                (lambda: self.compareImage("testWordsNullFX"),
                 ))
 
     def testCanvasNullFX(self):
-        def setOpacity():
+        def setOuterOpacity():
             node.opacity=0.6
 
+        def setInnerOpacity():
+            innerNode = canvas.getElementByID("test")
+            innerNode.opacity = 0.0
+
         root = self.loadEmptyScene()
-        self.__createOffscreenCanvas()
+        canvas = self.__createOffscreenCanvas()
         node = avg.ImageNode(parent=root, href="canvas:offscreen")
         node.setEffect(avg.NullFXNode())
-        self.start((
-                 lambda: self.compareImage("testCanvasNullFX1", False),
-                 setOpacity,
-                 lambda: self.compareImage("testCanvasNullFX2", False),
+        self.start(False,
+                (lambda: self.compareImage("testCanvasNullFX1"),
+                 setOuterOpacity,
+                 lambda: self.compareImage("testCanvasNullFX2"),
+                 setInnerOpacity,
+                 lambda: self.compareImage("testCanvasNullFX3"),
                 ))
 
     def testNodeInCanvasNullFX(self):
@@ -123,20 +135,22 @@ class FXTestCase(AVGTestCase):
                 fillopacity=1)
         canvas.getRootNode().insertChild(rect, 0)
         
-        self.start((
-                 lambda: self.compareImage("testNodeInCanvasNullFX1", False),
+        self.start(False,
+                (lambda: self.compareImage("testNodeInCanvasNullFX1"),
                 ))
 
     def testRenderPipeline(self):
-        print
+        sys.stderr.write("\n")
         for useSrcCanvas in (False, True):
             for useDestCanvas in (False, True):
                 for useFX in (False, True):
                     for useColorConv in (False, True):
-                        print "  ", useSrcCanvas, useDestCanvas, useFX, useColorConv
+                        sys.stderr.write("  "+str(useSrcCanvas)+" "+str(useDestCanvas)+
+                                " "+str(useFX)+" "+str(useColorConv)+"\n")
                         root = self.loadEmptyScene()
                         if useSrcCanvas:
-                            srcCanvas = Player.createCanvas(id="src", size=(160,120))
+                            srcCanvas = player.createCanvas(id="src", size=(160,120),
+                                    mediadir="media")
                             avg.ImageNode(href="rgb24alpha-64x64.png", 
                                     parent=srcCanvas.getRootNode())
                             srcImg = avg.ImageNode(href="canvas:src")
@@ -147,17 +161,18 @@ class FXTestCase(AVGTestCase):
                         if useColorConv:
                             srcImg.contrast = (1.01, 1.0, 1.0)
                         if useDestCanvas:
-                            destCanvas = Player.createCanvas(id="dest", size=(160,120))
+                            destCanvas = player.createCanvas(id="dest",
+                                    size=(160,120), mediadir="media")
                             destCanvas.getRootNode().appendChild(srcImg)
-                            destImg = avg.ImageNode(href="canvas:dest", parent=root)
+                            avg.ImageNode(href="canvas:dest", parent=root)
                         else:
                             root.appendChild(srcImg)
-                        self.start((
-                                lambda: self.compareImage("testRenderPipeline", False),
+                        self.start(False,
+                                (lambda: self.compareImage("testRenderPipeline"),
                                 ))
 
     def testBlurFX(self):
-        
+       
         def setRadius(radius):
             self.effect.radius = radius
         
@@ -168,25 +183,26 @@ class FXTestCase(AVGTestCase):
             self.node.setEffect(self.effect)
 
         def addNewFX():
-            effect = avg.BlurFXNode()
-            effect.radius = 8
+            effect = avg.BlurFXNode(8)
             self.node.setEffect(effect)
 
         root = self.loadEmptyScene()
         self.node = avg.ImageNode(parent=root, pos=(10,10), href="rgb24-64x64.png")
         self.effect = avg.BlurFXNode()
-        self.node.setEffect(self.effect)
-        self.start((
-                 lambda: self.compareImage("testBlurFX1", False),
+        self.start(False,
+                (self.skipIfMinimalShader,
+                 lambda: self.node.setEffect(self.effect),
+                 lambda: self.compareImage("testBlurFX1"),
                  lambda: setRadius(8),
-                 lambda: self.compareImage("testBlurFX2", False),
+                 lambda: self.compareImage("testBlurFX2"),
                  removeFX,
-                 lambda: self.compareImage("testBlurFX3", False),
+                 lambda: self.compareImage("testBlurFX3"),
                  reAddFX,
-                 lambda: self.compareImage("testBlurFX2", False),
+                 lambda: self.compareImage("testBlurFX2"),
                  removeFX,
                  addNewFX,
-                 lambda: self.compareImage("testBlurFX2", False),
+                 lambda: self.compareImage("testBlurFX2"),
+                 lambda: setRadius(300),
                 ))
 
     def testHueSatFX(self):
@@ -202,20 +218,21 @@ class FXTestCase(AVGTestCase):
         root = self.loadEmptyScene()
         self.node = avg.ImageNode(parent=root, pos=(10,10), href="rgb24alpha-64x64.png")
         resetFX()
-        self.start((
-                lambda: self.compareImage("testHueSatFX1", False),
-                lambda: setParam('saturation', -50),
-                lambda: self.compareImage("testHueSatFX2", False),
-                lambda: setParam('saturation', -100),
-                lambda: self.compareImage("testHueSatFX3", False),
-                lambda: setParam('saturation', -150),
-                lambda: self.compareImage("testHueSatFX3", False),
-                resetFX,
-                lambda: setParam('hue', 180),
-                lambda: self.compareImage("testHueSatFX4", False),
-                lambda: setParam('hue', -180),
-                lambda: self.compareImage("testHueSatFX4", False),
-        ))
+        self.start(False,
+                (self.skipIfMinimalShader,
+                 lambda: self.compareImage("testHueSatFX1"),
+                 lambda: setParam('saturation', -50),
+                 lambda: self.compareImage("testHueSatFX2"),
+                 lambda: setParam('saturation', -100),
+                 lambda: self.compareImage("testHueSatFX3"),
+                 lambda: setParam('saturation', -150),
+                 lambda: self.compareImage("testHueSatFX3"),
+                 resetFX,
+                 lambda: setParam('hue', 180),
+                 lambda: self.compareImage("testHueSatFX4"),
+                 lambda: setParam('hue', -180),
+                 lambda: self.compareImage("testHueSatFX4"),
+                ))
 
     def testInvertFX(self):
 
@@ -227,18 +244,18 @@ class FXTestCase(AVGTestCase):
             self.redRect = avg.RectNode(parent=self.root, pos=(5, 5), fillcolor='FF0000',
                     fillopacity=1, opacity=0, size=(72, 72))
             self.node = avg.ImageNode(parent=self.root, pos=(10,10),
-                    href="../graphics/testfiles/rgb24alpha-64x64.png")
+                    href="rgb24alpha-64x64.png")
             resetFX()
 
         self.root = self.loadEmptyScene()
-        self.node = avg.ImageNode(parent=self.root, pos=(10,10),
-                href="../graphics/testfiles/hsl.png")
+        self.node = avg.ImageNode(parent=self.root, pos=(10,10), href="hsl.png")
         resetFX()
-        self.start((
-                lambda: self.compareImage("testInvertFX1", False),
-                redAlphaScene,
-                lambda: self.compareImage("testInvertFX2", False),
-        ))
+        self.start(False,
+                (self.skipIfMinimalShader,
+                 lambda: self.compareImage("testInvertFX1"),
+                 redAlphaScene,
+                 lambda: self.compareImage("testInvertFX2"),
+                ))
 
     def testShadowFX(self):
         
@@ -252,33 +269,42 @@ class FXTestCase(AVGTestCase):
         rect = avg.RectNode(parent=root, pos=(9.5,9.5), color="0000FF")
         node = avg.ImageNode(parent=root, pos=(10,10), href="shadow.png")
         rect.size = node.size + (1, 1)
-        effect = avg.ShadowFXNode()
-        node.setEffect(effect)
-        self.start((
-                 lambda: self.compareImage("testShadowFX1", False),
+        effect = avg.ShadowFXNode((0,0), 1, 1, "FFFFFF")
+        self.start(False,
+                (self.skipIfMinimalShader,
+                 lambda: node.setEffect(effect),
+                 lambda: self.compareImage("testShadowFX1"),
                  lambda: setParams((0,0), 3, 2, "00FFFF"),
-                 lambda: self.compareImage("testShadowFX2", False),
+                 lambda: self.compareImage("testShadowFX2"),
                  lambda: setParams((2,2), 0.1, 1, "FFFFFF"),
-                 lambda: self.compareImage("testShadowFX3", False),
+                 lambda: self.compareImage("testShadowFX3"),
                  lambda: setParams((-2,-2), 0.1, 1, "FFFFFF"),
-                 lambda: self.compareImage("testShadowFX4", False),
+                 lambda: self.compareImage("testShadowFX4"),
                  lambda: setParams((-2,-2), 3, 1, "FFFFFF"),
-                 lambda: self.compareImage("testShadowFX5", False),
+                 lambda: self.compareImage("testShadowFX5"),
                  lambda: setParams((0,0), 0, 1, "FFFFFF"),
-                 lambda: self.compareImage("testShadowFX6", False),
+                 lambda: self.compareImage("testShadowFX6"),
                 ))
 
     def testWordsShadowFX(self):
+        
+        def setParams(offset, radius, opacity, color):
+            effect.offset = offset
+            effect.radius = radius
+            effect.opacity = opacity
+            effect.color =  color
+
         root = self.loadEmptyScene()
         node = avg.WordsNode(parent=root, pos=(10,10), text="testtext", 
                 font="Bitstream Vera Sans")
         effect = avg.ShadowFXNode()
-        effect.setParams((0,0), 1.5, 1.5, "FF0000")
-        node.setEffect(effect)
-        self.start((
-                 lambda: self.compareImage("testWordsShadowFX1", True),
-                 lambda: effect.setParams((2,2), 2, 2, "00FFFF"),
-                 lambda: self.compareImage("testWordsShadowFX2", True),
+        setParams((0,0), 1.5, 1.5, "FF0000")
+        self.start(True,
+                (self.skipIfMinimalShader,
+                 lambda: node.setEffect(effect),
+                 lambda: self.compareImage("testWordsShadowFX1"),
+                 lambda: setParams((2,2), 2, 2, "00FFFF"),
+                 lambda: self.compareImage("testWordsShadowFX2"),
                 ))
 
     def testGamma(self):
@@ -288,11 +314,11 @@ class FXTestCase(AVGTestCase):
         root = self.loadEmptyScene()
         node = avg.ImageNode(parent=root, href="colorramp.png", gamma=(0.5,0.5,0.5))
         self.assertEqual(node.gamma, (0.5,0.5,0.5))
-        self.start((
-                 lambda: self.compareImage("testGamma1", False),
+        self.start(False,
+                (lambda: self.compareImage("testGamma1"),
                  lambda: setGamma((1.5,2.0,2.5)),
                  lambda: self.assertEqual(node.gamma, (1.5,2.0,2.5)),
-                 lambda: self.compareImage("testGamma2", False),
+                 lambda: self.compareImage("testGamma2"),
                 ))
 
     def testIntensity(self):
@@ -302,32 +328,34 @@ class FXTestCase(AVGTestCase):
         def showVideo():
             node.unlink(True)
             self.videoNode = avg.VideoNode(parent=root, size=(96,96), threaded=False, 
-                    href="../video/testfiles/mpeg1-48x48.mpg", intensity=(0.5,0.5,0.5))
+                    href="mpeg1-48x48.mov", intensity=(0.5,0.5,0.5))
             self.videoNode.play()
 
-        def showText():
-            self.videoNode.unlink(True)
-            textNode = avg.WordsNode(parent=root, fontsize=24, font="Bitstream Vera Sans",
-                    intensity=(0.5,0.5,0.5), text="Half-brightness text.",
-                    width=140)
-
         root = self.loadEmptyScene()
         node = avg.ImageNode(parent=root, href="colorramp.png", intensity=(0.5,0.5,0.5))
         self.assertEqual(node.intensity, (0.5,0.5,0.5))
-        Player.setFakeFPS(10)
-        self.start((
-                 lambda: self.compareImage("testIntensity1", False),
+        player.setFakeFPS(10)
+        self.start(False,
+                (lambda: self.compareImage("testIntensity1"),
                  lambda: setIntensity((1.5,2.0,2.5)),
                  lambda: self.assertEqual(node.intensity, (1.5,2.0,2.5)),
-                 lambda: self.compareImage("testIntensity2", False),
+                 lambda: self.compareImage("testIntensity2"),
                  showVideo,
-                 lambda: self.compareImage("testIntensity3", False),
-                 showText,
-                 lambda: self.compareImage("testIntensity4", False),
+                 lambda: self.compareImage("testIntensity3"),
                 ))
-        Player.setFakeFPS(-1)
+        player.setFakeFPS(-1)
         self.videoNode = None
 
+    def testWordsIntensity(self):
+        root = self.loadEmptyScene()
+        avg.WordsNode(parent=root, fontsize=24, font="Bitstream Vera Sans",
+                intensity=(0.5,0.5,0.5), text="brightness",
+                width=140)
+        self.start(True,
+                (lambda: self.compareImage("testWordsIntensity"),
+                ))
+
+
     def testContrast(self):
         def setContrast(val):
             node.contrast = val
@@ -335,22 +363,22 @@ class FXTestCase(AVGTestCase):
         def showVideo():
             node.unlink(True)
             videoNode = avg.VideoNode(parent=root, size=(96,96), threaded=False, 
-                    href="../video/testfiles/mpeg1-48x48.mpg", contrast=(0.5,0.5,0.5))
+                    href="mpeg1-48x48.mov", contrast=(0.5,0.5,0.5))
             videoNode.play()
 
         root = self.loadEmptyScene()
         node = avg.ImageNode(parent=root, href="colorramp.png", contrast=(0.5,0.5,0.5))
         self.assertEqual(node.contrast, (0.5,0.5,0.5))
-        Player.setFakeFPS(10)
-        self.start((
-                 lambda: self.compareImage("testContrast1", False),
+        player.setFakeFPS(10)
+        self.start(False,
+                (lambda: self.compareImage("testContrast1"),
                  lambda: setContrast((1.5,2.0,2.5)),
                  lambda: self.assertEqual(node.contrast, (1.5,2.0,2.5)),
-                 lambda: self.compareImage("testContrast2", False),
+                 lambda: self.compareImage("testContrast2"),
                  showVideo,
-                 lambda: self.compareImage("testContrast3", False),
+                 lambda: self.compareImage("testContrast3"),
                 ))
-        Player.setFakeFPS(-1)
+        player.setFakeFPS(-1)
 
     def testFXUpdate(self):
         # This tests if the FX render-on-demand functionality doesn't forget updates.
@@ -372,7 +400,7 @@ class FXTestCase(AVGTestCase):
         def addVideo():
             node.unlink(True)
             videoNode = avg.VideoNode(parent=root, threaded=False, size=(96,96),
-                    href="../video/testfiles/mpeg1-48x48.mpg")
+                    href="mpeg1-48x48.mov")
             effect = avg.BlurFXNode()
             effect.radius = 0
             videoNode.setEffect(effect)
@@ -382,69 +410,74 @@ class FXTestCase(AVGTestCase):
         node = avg.ImageNode(parent=root, href="rgb24alpha-64x64.png")
         effect = avg.BlurFXNode()
         effect.radius = 0
-        node.setEffect(effect)
-        Player.setFakeFPS(25)
-        self.start((
+        player.setFakeFPS(25)
+        self.start(False,
+                (self.skipIfMinimalShader,
+                 lambda: node.setEffect(effect),
                  changeTexture,
-                 lambda: self.compareImage("testFXUpdateTex", False),
+                 lambda: self.compareImage("testFXUpdateTex"),
                  addMaskTex,
-                 lambda: self.compareImage("testFXUpdateMaskTex1", False),
+                 lambda: self.compareImage("testFXUpdateMaskTex1"),
                  changeMaskTex,
-                 lambda: self.compareImage("testFXUpdateMaskTex2", False),
+                 lambda: self.compareImage("testFXUpdateMaskTex2"),
                  changeMaskPos,
-                 lambda: self.compareImage("testFXUpdateMaskPos", False),
+                 lambda: self.compareImage("testFXUpdateMaskPos"),
                  changeFX,
-                 lambda: self.compareImage("testFXUpdateFX", False),
+                 lambda: self.compareImage("testFXUpdateFX"),
                  addVideo,
                  None,
-                 lambda: self.compareImage("testFXUpdateVideo", False),
+                 lambda: self.compareImage("testFXUpdateVideo"),
+                ))
+
+    def testChromaKeyFX(self):
+
+        def setParams(htol, ltol, stol):
+            effect.htolerance = htol
+            effect.ltolerance = ltol
+            effect.stolerance = stol
+
+        root = self.loadEmptyScene()
+        node = avg.ImageNode(parent=root, href="rgb24-64x64.png")
+        effect = avg.ChromaKeyFXNode()
+        setParams(0.01, 0.01, 0.01)
+        self.start(False,
+                (self.skipIfMinimalShader,
+                 lambda: node.setEffect(effect),
+                 lambda: self.compareImage("testChromaKeyFX1"),
+                 lambda: setParams(0.2, 0.2, 0.2),
+                 lambda: self.compareImage("testChromaKeyFX2"),
+                 lambda: effect.__setattr__("color", "FF0000"),
+                 lambda: self.compareImage("testChromaKeyFX3"),
+                 lambda: effect.__setattr__("spillthreshold", 1),
+                 lambda: self.compareImage("testChromaKeyFX4"),
                 ))
 
     def __createOffscreenCanvas(self):
-        canvas = Player.createCanvas(id="offscreen", size=(160,120))
+        canvas = player.createCanvas(id="offscreen", size=(160,120), mediadir="media")
         root = canvas.getRootNode()
         avg.ImageNode(href="rgb24-32x32.png", parent=root)
         avg.ImageNode(id="test", pos=(32,0), href="rgb24alpha-32x32.png", parent=root)
         return canvas
 
-def areFXSupported():
-    sceneString = """<avg id="avg" width="160" height="120"/>"""
-    Player.loadString(sceneString)
-    root = Player.getRootNode()
-    # XXX: The second of the following two lines prevent an opengl error in
-    # testImageNullFX on the Mac (Snow Leopard) for some reason. 
-    node = avg.ImageNode(href="rgb24-65x65.png", parent=root)
-    node = avg.ImageNode(href="rgb24-65x65.png", parent=root)
-    node.setEffect(avg.BlurFXNode())
-    Player.setTimeout(0, Player.stop)
-    try:
-        Player.play() 
-        return True
-    except RuntimeError:
-        return False
-
 
 def fxTestSuite(tests):
-    if areFXSupported():
-        availableTests = [
-                "testImageNullFX",
-                "testVideoNullFX",
-                "testWordsNullFX",
-                "testCanvasNullFX",
-                "testNodeInCanvasNullFX",
-                "testRenderPipeline",
-                "testBlurFX",
-                "testHueSatFX",
-                "testInvertFX",
-                "testShadowFX",
-                "testWordsShadowFX",
-                "testGamma",
-                "testIntensity",
-                "testContrast",
-                "testFXUpdate",
-            ]
-    else:
-        availableTests = []
+    availableTests = [
+            "testImageNullFX",
+            "testVideoNullFX",
+            "testWordsNullFX",
+            "testCanvasNullFX",
+            "testNodeInCanvasNullFX",
+            "testRenderPipeline",
+            "testBlurFX",
+            "testHueSatFX",
+            "testInvertFX",
+            "testShadowFX",
+            "testWordsShadowFX",
+            "testGamma",
+            "testIntensity",
+            "testWordsIntensity",
+            "testContrast",
+            "testFXUpdate",
+            "testChromaKeyFX",
+        ]
     return createAVGTestSuite(availableTests, FXTestCase, tests)
-
-Player = avg.Player.get()
