@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,7 @@ import atexit
 
 def cleanup(folder):
     if os.path.isdir(folder):
-        print 'Wiping out directory: %s' % folder
+        sys.stderr.write('Wiping out directory: %s\n' % folder)
         shutil.rmtree(folder)
 
 def symtree(src, dest):
@@ -45,7 +45,7 @@ def symtree(src, dest):
         fpath = os.path.join(src, f)
         if (f and f[0] != '.' and
             (os.path.isdir(fpath) or
-            (os.path.isfile(fpath) and os.path.splitext(f)[1] == '.py'))):
+            (os.path.isfile(fpath) and os.path.splitext(f)[1] in ('.py', '.glsl')))):
                 os.symlink(os.path.join(os.pardir, src, f), os.path.join(dest, f))
 
 
@@ -65,18 +65,20 @@ if sys.platform != 'win32':
         
         try:
             symtree('../python', 'libavg')
+            os.symlink('../../graphics/shaders', 'libavg/shaders')
         except OSError:
             pass
     else:
         # Running make distcheck
         symtree('../../../../src/python', 'libavg')
+        os.symlink('../../../../../src/graphics/shaders', 'libavg/shaders')
 
         # distcheck doesn't want leftovers (.pyc files)
         atexit.register(lambda tempPackageDir=tempPackageDir: cleanup(tempPackageDir))
     
     if os.path.exists('../wrapper/.libs/avg.so'):
         # Normal case: use the local version (not the installed one)
-        os.symlink('../../wrapper/.libs/avg.so', 'libavg/avg.so')
+        shutil.copy2('../wrapper/.libs/avg.so', 'libavg/avg.so')
     elif os.path.exists('../../avg.so'):
         # Mac version after installer dmg
         pass
@@ -91,13 +93,17 @@ if sys.platform != 'win32':
     # Meaningful only for distcheck
     os.chdir(srcDir)
 
-    import libavg
-    libavg.avg.Logger.get().trace(libavg.avg.Logger.APP, "Using libavg from: "+
-            os.path.dirname(libavg.__file__))
+import libavg
+libavg.logger.configureCategory(libavg.Logger.Category.APP, libavg.Logger.Severity.INFO)
+libavg.logger.info("Using libavg from: "+ os.path.dirname(libavg.__file__), 
+        libavg.Logger.Category.APP)
+# Ensure mouse is activated
+libavg.player.enableMouse(True)
 
+import testapp
+
+libavg.Player.get().keepWindowOpen()
 
-import testapp   
-   
 import PluginTest
 import PlayerTest
 import OffscreenTest
@@ -112,7 +118,10 @@ import AnimTest
 import EventTest
 import InputDeviceTest
 import AVGAppTest
-import UITest
+import WidgetTest
+import GestureTest
+import LoggerTest
+import AppTest
 
 app = testapp.TestApp()
 
@@ -129,8 +138,11 @@ app.registerSuiteFactory('python', PythonTest.pythonTestSuite)
 app.registerSuiteFactory('anim', AnimTest.animTestSuite)
 app.registerSuiteFactory('event', EventTest.eventTestSuite)
 app.registerSuiteFactory('inputdevice', InputDeviceTest.inputDeviceTestSuite)
-app.registerSuiteFactory('ui', UITest.uiTestSuite)
+app.registerSuiteFactory('widget', WidgetTest.widgetTestSuite)
+app.registerSuiteFactory('gesture', GestureTest.gestureTestSuite)
 app.registerSuiteFactory('avgapp', AVGAppTest.avgAppTestSuite)
+app.registerSuiteFactory('logger', LoggerTest.loggerTestSuite)
+app.registerSuiteFactory('app', AppTest.appTestSuite)
 
 app.run()
 
