@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-#
+
 # libavg - Media Playback Engine.
-# Copyright (C) 2003-2011 Ulrich von Zadow
+# Copyright (C) 2003-2014 Ulrich von Zadow
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -19,33 +19,31 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # Current versions can be found at www.libavg.de
-#
 
 import sys
-from libavg import avg, AVGApp
-import time
+from libavg import avg, app, player
 
-g_Player = avg.Player.get()
 
-def onFrame():
-    curTime = node.getCurTime()
-    g_Player.getElementByID("curtime").text = "Time: "+str(curTime/1000.0)
+class AudioPlayerDiv(app.MainDiv):
+    def onArgvParserCreated(self, parser):
+        parser.set_usage("%prog <filename>")
 
-class AudioPlayer(AVGApp):
-    def init(self):
-        global node
-        self._parentNode.appendChild(node)
-        avg.WordsNode(parent=self._parentNode, id="curtime", pos=(10, 22), 
-                font="arial", fontsize=10)
+    def onArgvParsed(self, options, args, parser):
+        if len(args) != 1:
+            parser.print_help()
+            sys.exit(1)
+        self._audioFName = args[0]
 
-        g_Player.setOnFrameHandler(onFrame)
-    
+    def onInit(self):
+        self._node = avg.SoundNode(parent=self, href=self._audioFName)
+        self._node.play()
+        self._words = avg.WordsNode(parent=self, pos=(10, 22), fontsize=10)
 
-if len(sys.argv) ==1:
-    print "Usage: avg_audioplayer.py <filename>"
-    sys.exit(1)
+    def onFrame(self):
+        curTime = self._node.getCurTime()
+        self._words.text = "Time: "+str(curTime/1000.0)
 
-node = avg.SoundNode(href=sys.argv[1])
-node.play()
-AudioPlayer.start(resolution=(320,200))
+
+if __name__ == "__main__":
+    app.App().run(AudioPlayerDiv(), app_resolution="320x200")
 
