@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,13 +25,14 @@
 #define _AudioDecoderThread_H_
 
 #include "../avgconfigwrapper.h"
-#include "VideoDecoder.h"
 #include "VideoMsg.h"
 
 #include "../base/WorkerThread.h"
 #include "../base/Command.h"
 #include "../audio/AudioParams.h"
 
+#include "WrapFFMpeg.h"
+
 #include <boost/thread.hpp>
 
 #include <string>
@@ -40,18 +41,42 @@ namespace avg {
 
 class AVG_API AudioDecoderThread : public WorkerThread<AudioDecoderThread> {
     public:
-        AudioDecoderThread(CQueue& cmdQ, VideoMsgQueue& msgQ, 
-                VideoDecoderPtr pDecoder, const AudioParams& ap);
+        AudioDecoderThread(CQueue& cmdQ, AudioMsgQueue& msgQ, VideoMsgQueue& packetQ, 
+                AVStream* pStream, const AudioParams& ap);
         virtual ~AudioDecoderThread();
         
         bool work();
-        void seek(double destTime);
-        void setVolume(double volume);
 
     private:
-        VideoMsgQueue& m_MsgQ;
-        VideoDecoderPtr m_pDecoder;
+        void decodePacket(AVPacket* pPacket);
+        void handleSeekDone(AVPacket* pPacket);
+        void discardPacket(AVPacket* pPacket);
+        AudioBufferPtr resampleAudio(char* pDecodedData, int framesDecoded,
+                int currentSampleFormat);
+        void insertSilence(float duration);
+        void planarToInterleaved(char* pOutput, char* pInput, int numChannels, 
+                int numSamples);
+        void pushAudioMsg(AudioBufferPtr pBuffer, float time);
+        void pushSeekDone(float time, int seqNum);
+        void pushEOF();
+        int getBytesPerSample(int sampleFormat);
+
+        AudioMsgQueue& m_MsgQ;
+        VideoMsgQueue& m_PacketQ;
         AudioParams m_AP;
+
+        AVStream * m_pStream;
+
+        int m_InputSampleRate;
+        int m_InputSampleFormat;
+        ReSampleContext * m_pResampleContext;
+        float m_AudioStartTimestamp;
+        float m_LastFrameTime;
+    
+        enum State {DECODING, SEEK_DONE, DISCARDING};
+        State m_State;
+        int m_SeekSeqNum;
+        float m_SeekTime;
 };
 
 }
