@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,43 +23,59 @@
 #define _VideoDecoderThread_H_
 
 #include "../api.h"
-#include "VideoDecoder.h"
 #include "VideoMsg.h"
 
 #include "../base/WorkerThread.h"
 #include "../base/Command.h"
 #include "../base/Queue.h"
 
-#include <boost/thread.hpp>
+#include "WrapFFMpeg.h"
 
-#include <string>
+#include <boost/thread.hpp>
 
 namespace avg {
 
 typedef Queue<Bitmap> BitmapQueue;
 typedef boost::shared_ptr<BitmapQueue> BitmapQueuePtr;
 
+class FFMpegFrameDecoder;
+typedef boost::shared_ptr<FFMpegFrameDecoder> FFMpegFrameDecoderPtr;
+
 class AVG_API VideoDecoderThread: public WorkerThread<VideoDecoderThread> {
     public:
-        VideoDecoderThread(CQueue& cmdQ, VideoMsgQueue& msgQ, 
-                VideoDecoderPtr pDecoder);
+        VideoDecoderThread(CQueue& cmdQ, VideoMsgQueue& msgQ, VideoMsgQueue& packetQ, 
+                AVStream* pStream, const IntPoint& size, PixelFormat pf, bool bUseVDPAU);
         virtual ~VideoDecoderThread();
+        virtual bool init();
+        virtual void deinit();
         
         bool work();
-        void seek(double destTime);
-        void setFPS(double fps);
+        void setFPS(float fps);
         void returnFrame(VideoMsgPtr pMsg);
 
     private:
-        BitmapPtr getBmp(BitmapQueuePtr pBmpQ, const IntPoint& size, 
-                PixelFormat pf);
+        void decodePacket(AVPacket* pPacket);
+        void handleEOF();
+        void handleSeekDone(VideoMsgPtr pMsg);
+        void sendFrame(AVFrame* pFrame);
+        void close();
+        BitmapPtr getBmp(BitmapQueuePtr pBmpQ, const IntPoint& size, PixelFormat pf);
+        void pushMsg(VideoMsgPtr pMsg);
+
         VideoMsgQueue& m_MsgQ;
-        VideoDecoderPtr m_pDecoder;
+        FFMpegFrameDecoderPtr m_pFrameDecoder;
+        VideoMsgQueue& m_PacketQ;
 
         BitmapQueuePtr m_pBmpQ;
         BitmapQueuePtr m_pHalfBmpQ;
         
-//        ProfilingZone * m_pPushMsgProfilingZone;
+        IntPoint m_Size;
+        PixelFormat m_PF;
+        bool m_bUseVDPAU;
+
+        bool m_bSeekDone;
+        bool m_bProcessingLastFrames;
+        AVFrame* m_pFrame;
 };
 
 }
