@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,63 +26,41 @@
 #include "../base/Queue.h"
 #include "../graphics/Bitmap.h"
 
-#include "../audio/AudioBuffer.h"
+#include "../audio/AudioMsg.h"
 
 #include <boost/shared_ptr.hpp>
 
 struct vdpau_render_state;
+struct AVPacket;
 
 namespace avg {
 
-class AVG_API VideoMsg {
+class AVG_API VideoMsg: public AudioMsg {
 public:
-    enum MsgType {NONE, AUDIO, END_OF_FILE, ERROR, FRAME, SEEK_DONE, VDPAU_FRAME};
     VideoMsg();
-    void setAudio(AudioBufferPtr pAudioBuffer, double audioTime);
-    void setEOF();
-    void setError(const Exception& ex);
-    void setFrame(const std::vector<BitmapPtr>& pBmps, double frameTime);
-    void setVDPAUFrame(vdpau_render_state* m_pRenderState, double frameTime);
-    void setSeekDone(double seekVideoFrameTime, double seekAudioFrameTime);
+    void setFrame(const std::vector<BitmapPtr>& pBmps, float frameTime);
+    void setVDPAUFrame(vdpau_render_state* pRenderState, float frameTime);
+    void setPacket(AVPacket* pPacket);
 
     virtual ~VideoMsg();
 
-    MsgType getType();
-
-    AudioBufferPtr getAudioBuffer() const;
-    double getAudioTime() const;
-
-    const Exception& getException() const;
-
     BitmapPtr getFrameBitmap(int i);
-    double getFrameTime();
-
-    double getSeekVideoFrameTime();
-    double getSeekAudioFrameTime();
+    float getFrameTime();
+    AVPacket* getPacket();
+    void freePacket();
 
     vdpau_render_state* getRenderState();
 
 private:
-    MsgType m_MsgType;
-
-    // AUDIO
-    AudioBufferPtr m_pAudioBuffer;
-    double m_AudioTime;
-
-    // ERROR
-    Exception* m_pEx;
-
     // FRAME
     std::vector<BitmapPtr> m_pBmps;
-    double m_FrameTime;
+    float m_FrameTime;
 
     // VDPAU_FRAME
     vdpau_render_state* m_pRenderState;
-
-    // SEEK_DONE
-    double m_SeekVideoFrameTime;
-    double m_SeekAudioFrameTime;
-
+    
+    // PACKET
+    AVPacket * m_pPacket;
 };
 
 typedef boost::shared_ptr<VideoMsg> VideoMsgPtr;
