@@ -1,6 +1,6 @@
 //
 //  libavg - Media Playback Engine. 
-//  Copyright (C) 2003-2011 Ulrich von Zadow
+//  Copyright (C) 2003-2014 Ulrich von Zadow
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,15 +23,18 @@
 #define _WrapHelper_H_
 
 #include "../api.h"
-#include "../base/Point.h"
-#include "../base/Triple.h"
+#include "../base/GLMHelper.h"
 #include "../base/Exception.h"
+#include "../base/ILogSink.h"
 
 #include "../player/BoostPython.h"
 #include "../player/Player.h"
+#include "../player/TypeRegistry.h"
 
 #include <string>
 
+template<typename T> const T copyObject(const T& v) { return v; }
+
 template <typename ContainerType>
 struct to_tuple
 {
@@ -64,6 +67,21 @@ struct to_list
     static const PyTypeObject* get_pytype() { return &PyList_Type; }
 };
 
+template <typename MapType>
+struct to_dict
+{
+    static PyObject* convert(MapType const& a)
+    {
+        boost::python::dict result;
+        typedef typename MapType::const_iterator const_iter;
+        for(const_iter p=a.begin();p!=a.end();p++){
+                result[p->first] = p->second;
+        }
+        return boost::python::incref(boost::python::dict(result).ptr());
+    }
+    static const PyTypeObject* get_pytype() { return &PyDict_Type; }
+};
+
 struct default_policy
 {
   static bool check_convertibility_per_element() { return false; }
@@ -273,48 +291,107 @@ struct from_python_sequence
 };
 
 template<class T>
-double deprecatedGet(T& node)
+float deprecatedGet(T& node)
 {
     throw avg::Exception(AVG_ERR_DEPRECATED, "Attribute has been removed from libavg.");
 }
 
 template<class T>
-void deprecatedSet(T& node, double d)
+void deprecatedSet(T& node, float d)
 {
     throw avg::Exception(AVG_ERR_DEPRECATED, "Attribute has been removed from libavg.");
 }
 
-namespace DPointHelper
+namespace Vec2Helper
 {
-    int len(const avg::DPoint&);
-    double getX(const avg::DPoint& pt);
-    double getY(const avg::DPoint& pt);
-    void setX(avg::DPoint& pt, double val);
-    void setY(avg::DPoint& pt, double val);
+    int len(const glm::vec2&);
+    float getX(const glm::vec2& pt);
+    float getY(const glm::vec2& pt);
+    void setX(glm::vec2& pt, float val);
+    void setY(glm::vec2& pt, float val);
     void checkItemRange(int i);
-    double getItem(const avg::DPoint& pt, int i);
-    void setItem(avg::DPoint& pt, int i, double val);
-    std::string str(const avg::DPoint& pt);
-    std::string repr(const avg::DPoint& pt);
-    long getHash(const avg::DPoint& pt);
+    float getItem(const glm::vec2& pt, int i);
+    void setItem(glm::vec2& pt, int i, float val);
+    std::string str(const glm::vec2& pt);
+    std::string repr(const glm::vec2& pt);
+    long getHash(const glm::vec2& pt);
+    glm::vec2 safeGetNormalized(const glm::vec2& pt);
+    float getNorm(const glm::vec2& pt);
+    float vecAngle(const glm::vec2& pt1, const glm::vec2& pt2);
 }
 
-class ConstDPoint: public avg::DPoint
+class ConstVec2: public glm::vec2
 {
 public:
-    ConstDPoint();
-    ConstDPoint(const avg::DPoint& other);
-    operator avg::DPoint() const;
+    ConstVec2();
+    ConstVec2(const glm::vec2& other);
+    glm::vec2 toVec2() const;
+    //operator glm::vec2() const;
 };
 
-AVG_API void checkEmptyArgs(const boost::python::tuple &args, int numArgs=0);
+AVG_API void checkEmptyArgs(const boost::python::tuple &args, int numArgs=1);
+
+template<const char * pszType> 
+avg::ExportedObjectPtr createExportedObject(const boost::python::tuple &args,
+        const boost::python::dict &attrs)
+{
+    checkEmptyArgs(args);
+    return avg::TypeRegistry::get()->createObject(pszType, attrs);
+}
 
 template<const char * pszType> 
 avg::NodePtr createNode(const boost::python::tuple &args,
         const boost::python::dict &attrs)
 {
     checkEmptyArgs(args);
-    return avg::Player::get()->createNode(pszType, attrs);
+    return avg::Player::get()->createNode(pszType, attrs, args[0]);
+};
+
+template <typename T>struct Exception_to_python_exception
+{
+    static PyObject* convert (const T& ex)
+    {
+        PyObject *arglist = boost::python::incref(
+                Py_BuildValue("(s)", ex.what()));
+        return boost::python::incref(
+                PyObject_CallObject(PyExc_RuntimeError, arglist));
+    }
+};
+
+
+template<typename T> struct ExceptionTranslator
+{
+public:
+    void operator()(const T& ex) const
+    {
+      PyErr_SetString(m_PyExcept, ex.what());
+    }
+
+    ExceptionTranslator(PyObject* py_except): m_PyExcept(py_except)
+    {
+      boost::python::register_exception_translator<T>(*this);
+    }
+
+    ExceptionTranslator(const ExceptionTranslator& other):m_PyExcept(other.m_PyExcept)
+    {
+    }
+
+  private:
+
+    PyObject* m_PyExcept;
+
+};
+
+template <typename T> void translateException(PyObject* e) {
+  ExceptionTranslator<T> my_translator(e);
 }
 
+void exportMessages(boost::python::object& nodeClass, const std::string& sClassName);
+
+void addPythonLogger(PyObject * self, PyObject * pyLogger);
+void removePythonLogger(PyObject * self, PyObject * pyLogger);
+
+void pytrace(PyObject * self, const avg::category_t& category, const avg::UTF8String& sMsg,
+        avg::severity_t severity);
+
 #endif
